/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.DashboardItem");

dojo.require("com.ibm.team.dashboard.web.client.internal.ColumnHelper");
dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletHelper");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletPageHelper");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.dashboard.web.client.internal.ItemNameRegistry");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases
 */
var ColumnHelper = com.ibm.team.dashboard.web.client.internal.ColumnHelper;
var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var ContributorInfoRegistry = com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ViewletHelper = com.ibm.team.dashboard.web.client.internal.ViewletHelper;
var ViewletPageHelper = com.ibm.team.dashboard.web.client.internal.ViewletPageHelper;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ItemNameRegistry = com.ibm.team.dashboard.web.client.internal.ItemNameRegistry;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var messages;

/*
 * A dashboard model item.
 */
dojo.declare("com.ibm.team.dashboard.web.client.internal.DashboardItem",
	null, {
	
	/*
	 * Constructs a new dashboard item from the dashboard bundle sent by the
	 * server. Immediate mode by default.
	 */
	constructor: function(/* DashboardBundleDTO */ dashboardBundle, /* DashboardClient */ client) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		if (dashboardBundle.itemNames)
			ItemNameRegistry.addNames(dashboardBundle.itemNames);
		if (dashboardBundle.defaults && dashboardBundle.defaults.team)
			ItemNameRegistry.addName(dashboardBundle.defaults.team);
		if (dashboardBundle.defaults && dashboardBundle.defaults.project)
			ItemNameRegistry.addName(dashboardBundle.defaults.project);
		if (dashboardBundle.defaults && dashboardBundle.defaults.contributor)
			ItemNameRegistry.addName(dashboardBundle.defaults.contributor);
		if (dashboardBundle.contributorInfo) {
			ContributorInfoRegistry.add(dashboardBundle.contributorInfo);
		}
		var rawItem = dashboardBundle.dashboard;
		this._permissions = dashboardBundle.permissions;
		this._viewletDefinitions = dashboardBundle.viewletDefinitions;
		this._client = client;
		this._id = rawItem.itemId;
		this._title = rawItem.title;
		switch(rawItem.scope) {
			case "CONTRIBUTOR":
				this._scope = DashboardConstants.SCOPE_CONTRIBUTOR;
				break;
			case "TEAM_AREA":
				this._scope = DashboardConstants.SCOPE_TEAM_AREA;
				break;
			case "PROJECT_AREA":
				this._scope = DashboardConstants.SCOPE_PROJECT_AREA;
				break;
		}
		this._scopeItem = {itemId: rawItem.scopeItem.itemId, name: ItemNameRegistry.getName(rawItem.scopeItem.itemId)};
		this._viewletPages = new Array();
		// associative array notation is used since public is a reserved word
		this._isPublic = rawItem["public"];
		if (rawItem.viewletPages) {
			for (var i=0;i<rawItem.viewletPages.length;i++) {
				this._viewletPages.push(new ViewletPageHelper(rawItem.viewletPages[i], this));
			}
		}
		this._modified = rawItem.modified;
		this._defaultTeam = dashboardBundle.defaults.team;
		this._defaultProject = dashboardBundle.defaults.project;
		this._defaultContributor = dashboardBundle.defaults.contributor;
		// queue for modifications committed and waiting to be sent out
		this._outgoingQueue = new Array();
		// queue for modifications made locally but not yet committed
		this._pendingQueue = new Array();
		this._isDeferredMode = false;
		this._isScopeArchived = dashboardBundle.isScopeArchived;
		this._templateExists = dashboardBundle.templateExists ? true : false;
	},

	/*
	 * Queues a modification to be performed on this item, upon calling save.
	 * The object has the form { type: <integer>, ... }. Type must be one of the
	 * Constants.MODIFICATION_TYPE_* constants, and the other properties are
	 * specific to the modification type - see DashboardModifier#applyModifications
	 * for expected properties.
	 */
	addModification: function(/* object */ modification) {
		this._pendingQueue.push(modification);
		if (this._t) {
			this._t = "dirty";
		} else {
			this._completeTransaction();
		}
	},
	
	startTransaction: function() {
		this._t = true;
	},
	
	endTransaction: function() {
		if (this._t == "dirty")
			this._completeTransaction();
		delete this._t;
	},
	
	_completeTransaction: function() {
		if (!this._isDeferredMode) {
			this.save();
		} else if (!(this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this.isPublic())){
			//if we're in deferred mode, and this is not a private dashboard
			this._checkSharedEditLock();
		}
		if (this._notifyDirty) {
			this._notifyDirty();	
		}	
	},
	
	_checkSharedEditLock: function(){
		var serviceObj = {
			self: this,
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					// we have a lock; switch to edit mode
					case "OK":
						this.self.hasLock = true;
						break;
					case "STALE":
						if (responseDTO.msg) {
							// we know who modified it
							alert(dojo.string.substitute(messages.itemEditStaleSpecific, [ responseDTO.msg ]));
						}
						else {
							// don't know who modified it
							alert(messages.itemEditStaleGeneral);
						}
						this.self._dashboardPage.reload();
						break;
					case "LOCKED":
						if (confirm(dojo.string.substitute(messages.itemEditLocked, [ responseDTO.msg ]))) {
							// this time, set override to true
							var srh = new ServiceResponseHandler(this, "success", "failure");
							this.self.lock(true, srh);
						} else {
							this.self._dashboardPage.reload();
						}
						break;
				}
			},
			failure: function(error) {
				alert(dojo.string.substitute(messages.itemEditError, [ error.message ]));
			}
		};
		// override false on first attempt (check with user before override)
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		this.lock(false, srh);
	},
	
	addViewletPage: function(/* ViewletPageHelper */ helper) {
		var viewletPages = this.getViewletPages(); 
		viewletPages.push(helper);
		this.addModification({type: Constants.MODIFICATION_TYPE_ADD_VIEWLET_PAGE, title: helper.getTitle(), numCols: helper.getColumns().length, id: helper.getId() });
	},

	/*
	 * Returns the dashboard item's last modified timestamp.
	 */
	/* string */ getModified: function() {
		return this._modified;
	},

	/*
	 * Returns the dashboard item's id.
	 */
	/* string */ getId: function() {
		return this._id;
	},
	
	/*
	 * Returns the dashboard's title.
	 */
	/* string */ getTitle: function() {
		return this._title;
	},
	
	/*
	 * Returns the dashboard's scope. Must be SCOPE_CONTRIBUTOR, SCOPE_TEAM_AREA,
	 * or SCOPE_PROJECT_AREA.
	 */
	/* number */ getScope: function() {
		return parseInt(this._scope);
	},
	
	/*
	 * Returns the item id of the item (contributor, team area, or project area) this
	 * dashboard is scoped to.
	 */
	/* string */ getScopeItem: function() {
		return this._scopeItem;
	},
	
	/*
	 * Returns an item id for the specified scope to be used as the defaults for viewlets
	 * that do not support the dashboard's scope.
	 */
	/* string */ getDefaultItem: function(/* number */ scope) {
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				return this._defaultContributor;
			case DashboardConstants.SCOPE_TEAM_AREA:
				return this._defaultTeam;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				return this._defaultProject;
		}
	},
	
	/*
	 * Returns the path to the dashboard's team area, if it's a team dashboard.
	 */
	/* string */ getTeamPath: function() {
		if (this._defaultTeam) {
			return this._defaultTeam.path;
		}
	},
	
	getBreadcrumbs: function() {
		var breadcrumbs = new Array();
		breadcrumbs.push({label: messages.allHeading, href: "#action=com.ibm.team.dashboard.viewAllDashboards"});
		if (this._defaultTeam) {
			if (this.getScope() != DashboardConstants.SCOPE_CONTRIBUTOR && this.getScope() != DashboardConstants.SCOPE_PROJECT_AREA) {
				breadcrumbs.push({label: this._defaultProject.name, href: "#action=com.ibm.team.dashboard.viewDashboard"});
			}
			var path = this._defaultTeam.path;
			var split = path.split("/");
			var curChar = 0;
			for (var i = 0; i < split.length - 1; i++) {
				breadcrumbs.push({label: decodeURIComponent(split[i]), href: "#action=com.ibm.team.dashboard.viewDashboard&team=" + path.substring(0,curChar+split[i].length)});
				curChar += split[i].length + 1;	
			}
		}
		return breadcrumbs;
	},

	/*
	 * Returns the permissions for this dashboard and the currently logged
	 * in user.
	 */
	/* PermissionsDTO */ getPermissions: function() {
		return this._permissions;
	},
	
	/*
	 * Returns the user ID of the contributor whose profile dashboard this is,
	 * if it's a profile dashboard.
	 */
	/* string */ getUserId: function() {
		if (this._defaultContributor) {
			return this._defaultContributor.userId;
		}
	},

	/*
	 * Returns the dashboard's viewlet definitions, if they were requested.
	 */
	/* ViewletDefinitionDTO[] */ getViewletDefinitions: function() {
		return this._viewletDefinitions;
	},

	/*
	 * Returns the viewlet pages in this dashboard.
	 */
	/* ViewletPageHelper[] */ getViewletPages: function() {
		return this._viewletPages;
	},
	
	/*
	 * Returns whether or not the item is in deferred mode.
	 */
	/* boolean */ isDeferredMode: function() {
		return this._isDeferredMode;
	},
	
	/*
	 * Returns whether or not the item represents a public dashboard (only applicable to contributor scope).
	 */
	/* boolean */ isPublic: function() {
		return this._isPublic;
	},

	/*
	 * Returns whether or not the associated scope item is archived.
	 */
	/* boolean */ isScopeArchived: function() {
		return this._isScopeArchived;
	},
	
	/*
	 * Indicates if a template exists for this dashboard's scope.
	 */
	/* boolean */ hasTemplate: function() {
		return this._templateExists;
	},
	
	/*
	 * Locks a dashboard for a period of time, preventing others from
	 * modifying it or obtaining a lock. The lock remains in effect until
	 * the dashboard is unlocked, the lock is overriden by another user,
	 * or the keep-alive poll times out.
	 */
	lock: function(/* boolean */ override, /* ServiceResponseHandler */ srh) {
		this._client.lockDashboard(this._id, override, this._modified, srh);
	},
	
	/*
	 * Moves a viewlet within a dashboard. Null is allowed for destPageId, which
	 * means same as src page.
	 */
	moveViewlet: function(/* string */ viewletId, /* number */ destPageIndex, /* number */ destColumnIndex, /* number */ destViewletIndex) {
		// get src and dest page indexes
		var srcPage;
		var srcPageIndex;
		var viewletPages = this.getViewletPages();
		var destPage = viewletPages[destPageIndex];
		for (var i=0;!srcPageIndex && i<viewletPages.length;i++) {
			var page = viewletPages[i];
			var columns = page.getColumns();
			for (var j=0;!srcPageIndex && j<columns.length;j++) {
				var column = columns[j];
				var viewlets = column.getViewlets();
				for (var k=0;k<viewlets.length;k++) {
					var viewlet = viewlets[k];
					if (viewlet.getId() == viewletId) {
						srcPage = page;
						srcPageIndex = i;
						if (!destPageIndex && destPageIndex != 0) {
							destPage = srcPage;
							destPageIndex = srcPageIndex;
						}
						break;
					}
				}
			}
		}
		
		// get src column and viewlet index
		var srcColumn;
		var srcColumnIndex;
		var srcViewletIndex;
		var columns = srcPage.getColumns();
		var isDone = false;
		for (var i=0;!isDone && i<columns.length;i++) {
			var column = columns[i];
			var viewlets = column.getViewlets();
			for (var j=0;j<viewlets.length;j++) {
				if (viewlets[j].getId() == viewletId) {
					srcColumn = column;
					srcColumnIndex = i;
					srcViewletIndex = j;
					isDone = true;
					break;
				}
			}
		}

		// move viewlet from source column to destination column
		var viewlet = srcColumn.getViewlets()[srcViewletIndex];
		srcColumn.getViewlets().splice(srcViewletIndex, 1);
		destPage.getColumns()[destColumnIndex].getViewlets().splice(destViewletIndex, 0, viewlet);
		
		// save state
		this.addModification({type: Constants.MODIFICATION_TYPE_MOVE_VIEWLET, srcPageIndex:srcPageIndex, srcColumnIndex:srcColumnIndex, srcViewletIndex:srcViewletIndex, destPageIndex:destPageIndex, destColumnIndex:destColumnIndex, destViewletIndex:destViewletIndex});
	},
	
	moveViewletPage: function(startIndex, endIndex) {
		// move the page in the client model
		var pages = this.getViewletPages();
		var page = pages[startIndex];
		pages.splice(startIndex,1);
		pages.splice(endIndex, 0, page);
		// save state
		this.addModification({type: Constants.MODIFICATION_TYPE_MOVE_VIEWLET_PAGE, startIndex: startIndex, endIndex: endIndex});
	},
	
	changeColumns: function(srcPageIndex,newQuantity) {
		var srcPage = this.getViewletPages()[srcPageIndex];
		var columns = srcPage.getColumns();
		var diff = columns.length - newQuantity;
		
		if (diff > 0) {
			for (var i = 0; i < diff; i++) {
				var srcColumn = columns[columns.length-1];
				for (var j = srcColumn.getViewlets().length - 1; j >= 0; j--) {
					var viewlet = srcColumn.getViewlets()[j];
					srcColumn.getViewlets().splice(j,1);
					columns[j%newQuantity].getViewlets().splice(0,0,viewlet);
				}
				columns.splice(columns.length-1,1);
			}
		} else {
			while(columns.length<newQuantity) {
				srcPage.getColumns().push(new ColumnHelper({}, this));
			}
		}

		this.addModification({type: Constants.MODIFICATION_TYPE_CHANGE_COLUMNS, srcPageIndex: parseInt(srcPageIndex), newQuantity: newQuantity});
	},
	
	/*
	 * Changes the display title of the specified tab.
	 */
	setTabTitle: function(/* string */ title, /* number */ pageIndex) {
		var srcPage = this.getViewletPages()[pageIndex];
		srcPage.setTitle(title);
	},
	
	/*
	 * Deletes the specified tab.
	 */
	deleteTab: function(/* number */ pageIndex) {
		delete this.getViewletPages()[pageIndex];
		this.getViewletPages().splice(pageIndex,1);
		this.addModification({type: Constants.MODIFICATION_TYPE_REMOVE_VIEWLET_PAGE, pageIndex: pageIndex});
	},
	
	/*
	 * Removes the viewlet with the given id from the dashboard.
	 */
	removeViewlet: function(/* string */ viewletId) {
		var viewletPageHelpers = this.getViewletPages();
		var isDone = false;
		for (var i=0;!isDone && i<viewletPageHelpers.length;i++) {
			var viewletPage = viewletPageHelpers[i];
			var columns = viewletPage.getColumns();
			for (var j=0;!isDone && j<columns.length;j++) {
				var column = columns[j];
				var viewlets = column.getViewlets();
				for (var k=0;k<viewlets.length;k++) {
					if (viewlets[k].getId() == viewletId) {
						// remove from array
						viewlets.splice(k, 1);
						isDone = true;
						break;
					}
				}
			}
		}
		
		// save state
		this.addModification({type: Constants.MODIFICATION_TYPE_REMOVE_VIEWLET, viewletId:viewletId});
	},

	/*
	 * Saves all pending modifications to the working copy to the repository.
	 */
	save: function() {
		if (this._pendingQueue.length > 0) {
			// send mods to outgoing and flush
			this._outgoingQueue = this._outgoingQueue.concat(this._pendingQueue);
			this._pendingQueue = new Array();
			this._flushOutgoingQueue();
			if (this.isDeferredMode()){
				this.unlock();	
			}
		}
	},
	/*
	 * In deferred mode, returns whether or not there are modifications 
	 * in the queue that have not been sent to the server.
	 */
	isDirty: function() {
		if(this.isDeferredMode && this._pendingQueue.length > 0) {
			return true;
		}
		return false;
	},
	setDirtyListener: function(handler, context) {
		this._notifyDirty = dojo.hitch(context, handler);
	},
	removeDirtyListener: function() {
		this._notifyDirty = null;
	},
	/*
	 * Sets whether to work in deferred mode (as opposed to immediate mode,
	 * which is the default). In deferred mode, modifications are only propagated
	 * to the server upon calling save(). In immediate mode, it happens
	 * immediately (no call to save() required).
	 */
	setDeferredMode: function(/* boolean */ value) {
		if (!(this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this.isPublic())) {
			//not private contributor dashboard
			if (this._isDeferredMode && !value) {
				// switching from deferred mode to immediate mode
				this._checkSharedEditLock();
			} else if (!this._isDeferredMode && value && this.hasLock) {
				// switching from immediate to deferred
				this.unlock();
			}
		}
		this._isDeferredMode = value;
	},
	
	/*
	 * Sets the dashboard title.
	 */
	setTitle: function(/* string */ title) {
		this._title = title;
		this.addModification({type:Constants.MODIFICATION_TYPE_SET_DASHBOARD_TITLE, title:title});
	},

	/*
	 * Unlocks a dashboard that was previously locked.
	 */
	unlock: function() {
		this._client.unlockDashboard(this._id);
		this.hasLock = false;
	},
	
	/*
	 * Flushes any committed outgoing modifications, or waits for the next
	 * batch if a transaction is already in progress.
	 */
	_flushOutgoingQueue: function() {
		if (!this._isModificationInProgress && this._outgoingQueue.length > 0) {
			this._isModificationInProgress = true;
			var srh = new ServiceResponseHandler(this, "_handleModificationSuccess", "_handleModificationError");
			this._client.postModifications(this.getId(), this._outgoingQueue, this._modified, srh);
			this._outgoingQueue = new Array();
		} 
	},

	/*
	 * Callback for successful modification service call.
	 */
	_handleModificationSuccess: function(/* ResponseDTO */ responseDTO) {
		switch(responseDTO.status) {
			case "OK":
				// sync timestamps to detect stale data
				this._modified = responseDTO.msg;
				break;
			case "STALE":
				if (responseDTO.msg) {
					// msg present means modified by someone else
					alert(dojo.string.substitute(messages.itemSaveStaleSpecific, [responseDTO.msg]));
				}
				else {
					// no msg means modified by current user
					alert(messages.itemSaveStaleGeneral);
				}
				break;
			case "LOCKED":
				alert(dojo.string.substitute(messages.itemSaveLocked, [responseDTO.msg]));
				break;
			case "ERROR":
				alert(responseDTO.msg);
				break;
		}
		// send out any mods that were waiting in the outgoing queue
		this._isModificationInProgress = false;
		this._flushOutgoingQueue();
	},
	
	/*
	 * Callback for service call errors.
	 */
	_handleModificationError: function(/* Error */ error) {
		// send out any mods that were waiting in the outgoing queue
		this._isModificationInProgress = false;
		this._flushOutgoingQueue();
		dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
	}
});

})();
