 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.util");

(function() {

var getCurrentProjectArea	= com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var checkNotNull			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;
var ServiceRequest			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient		= com.ibm.team.repository.web.transport.TeamServerClient;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var Utils					= com.ibm.team.workitem.web.client.util;

// See IProcessReport
var OK= 0;
var WARNING= 2;
var ERROR= 4; 

function validateWorkItemParams(appArgs, method) {
	
	// validate that we have any request parameters
	if (!appArgs) {
		throw new Error("WorkItemClient#" + method + ": missing request parameters.");
	}
	
	// validate that we have only one of 'id', 'itemId', or 'newWorkItem'
	var count= 0;
	dojo.forEach(["id", "itemId", "newWorkItem"],
		function(x) {
			if (appArgs[x]) { count++; var setStr= x + " "; }
		});
	if (count > 1) {
		throw new Error("WorkItemClient#" + method +
			": may only set one of 'id', 'itemId', or 'newWorkItem' (saw: [" +
			setStr + "]");
	}

}

/**
 * Provides helper methods to fetch workitem-related data from the team server.
 */
com.ibm.team.workitem.web.client.internal.WorkItemClient= {

	// service URI
	SERVICE_URI: "com.ibm.team.workitem.common.internal.rest.IWorkItemRestService",
	
	// get methods
	GET_VALUES_METHOD: "getValues",
	GET_CONTRIBUTORS_METHOD: "getContributors",

	/*
	 * Invokes the work item REST service specified by 'method' with optional
	 * request parameters specified by 'appArgs'.  Calls back to the functions
	 * defined by 'responseHandler'.
	 */
	_invokeWorkItemService: function(responseHandler, method, appArgs) {
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length === 2) {
			var serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method);
		} else {
			serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	getWorkItemEditableProperties: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemEditableProperties");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		WorkItemClient._invokeWorkItemService(responseHandler, "getWorkItemEditableProperties", appArgs);
	},
	
	getAllValues: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getAllValues");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		if (!appArgs.ids) {
			appArgs.ids= [];
		}
		WorkItemClient._invokeWorkItemService(responseHandler, "getAllValues", appArgs);
	},
	
	getTime: function(responseHandler, appArgs) {
		WorkItemClient._invokeWorkItemService(responseHandler, "getTime", appArgs);
	},
	
	/**
	 * Returns a set of owners; dependent on project area, category, and target.
	 */
	getOwners: function(responseHandler, appArgs) {
		appArgs.attributeName= "owner";
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		WorkItemClient._invokeWorkItemService(responseHandler, "getValues", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemDTO.
	 */
	getWorkItemDTO: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemDTO");
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		}
		this._invokeWorkItemService(responseHandler, "getWorkItemDTO", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemDTO2.
	 */
	getWorkItem: function(responseHandler, appArgs) {
		if (appArgs.newWorkItem) {
			appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
			if (!appArgs.typeId)
				appArgs.typeId= "defect"; // TODO: Remove. Type must be passed in
		}
		
		var internalHandler= {
			success: function(workItemDTO) {
				if(workItemDTO == null) {
					responseHandler.doCallback(null);
					return;
				}
				var attributeMap= Utils.buildMap(workItemDTO.attributes)
				workItemDTO.attributes= attributeMap;
//				console.debug(workItemDTO);
				responseHandler.doCallback(workItemDTO);
			},
			failure: function(error) {
				responseHandler.doErrorCallback(error);
			}
		};
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		this._invokeWorkItemService(handler, "getWorkItemDTO2", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemSummaryPage.
	 */
	getWorkItemSummaryPage: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, "getWorkItemSummaryPage", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#postWorkItem.
	 */
	postWorkItem: function(responseHandler, appArgs) {
		// in the case of a new work item, add the project area item id
		if (!appArgs.stateId) {
			appArgs.projectAreaItemId = getCurrentProjectArea().itemId;
		}
		
		var internalHandler= {
			success: function(operationReportDTO) {
				if (operationReportDTO.severity == OK) {
					var workItemDTO= operationReportDTO.workItem;
					var attributeMap= Utils.buildMap(workItemDTO.attributes)
					workItemDTO.attributes= attributeMap;
					responseHandler.doCallback(workItemDTO);
				} else {
					responseHandler.doErrorCallback(operationReportDTO);
				}
			},
			failure: function(error) {
				console.debug(error);
			}
		};
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		this._invokeWorkItemService(handler, "postWorkItem2", appArgs);
	},
	
	getResolutions: function(responseHandler, appArgs) {
		if (!(appArgs.status || appArgs.actionId)) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (none set)");
		}
		if (appArgs.status && appArgs.actionId) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (both set)");
		}
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		appArgs.attributeName= "internalResolution";
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_VALUES_METHOD, appArgs);
	},
	
	getContributors: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_CONTRIBUTORS_METHOD, appArgs);
	},
	
	getTeamMembers: function(responseHandler) {
		var appArgs= { projectAreaItemId: getCurrentProjectArea().itemId};
		this._invokeWorkItemService(responseHandler, "getTeamMembers", appArgs);
	},
	
	getEditorPresentation: function(responseHandler, appArgs) {
		if (!appArgs.typeId && !appArgs.itemId) {
			throw new Error("WorkItemClient#getEditorPresentation: must set appArgs.typeId (the work item type id) or appArgs.itemId (the work item itemId)");
		}
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		this._invokeWorkItemService(responseHandler, "getEditorPresentation", appArgs);
	},
	
	getQuickInformationPresentation: function(responseHandler, appArgs) {
		if (!appArgs.configurationId) {
			throw new Error("WorkItemClient#getQuickInformationPresentation: must set appArgs.configurationId");
		}
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		this._invokeWorkItemService(responseHandler, "getQuickInformationPresentation", appArgs);
	},
		
	getApprovalStates: function(responseHandler) {
		this._invokeWorkItemService(responseHandler, "getApprovalStates");
	},
		
	getApprovalTypes: function(responseHandler) {
		this._invokeWorkItemService(responseHandler, "getApprovalTypes");
	},
	
	getEnumerationValues: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= getCurrentProjectArea().itemId;
		this._invokeWorkItemService(responseHandler, "getEnumerationValues", appArgs);
	},
	
	// TODO: Wrong place. Needs to be in ProcessClient. See work item 41829;
	getProcessAreaHierarchies: function(responseHandler) {
		var serviceRequest= new ServiceRequest("com.ibm.team.process.internal.common.service.IProcessRestService", "getProcessAreaHierarchies");
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	}
};

// must be defined after com.ibm.team.workitem.web.client.internal.WorkItemClient
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;

})();