/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.web.ui.internal.ProjectAreaApplication"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.internal.Dialog"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal._JazzApplication"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.Footer"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.Header"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var checkNotNull 	= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 	= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var isGuest			= com.ibm.team.repository.web.client.session.isGuest;

var _JazzApplication        = com.ibm.team.repository.web.ui.internal._JazzApplication;
var Footer					= com.ibm.team.repository.web.ui.internal.Footer;
var Header					= com.ibm.team.repository.web.ui.internal.Header;
var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var PageContainer			= net.jazz.ajax.ui.PageContainer;
var PlatformUI				= net.jazz.ajax.ui.PlatformUI;
var RepositoryClient 		= com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Dialog                  = net.jazz.ajax.ui.internal.Dialog;

function getProjectAreaName(){
	//dojo.lang.assert(typeof com.ibm.team.process.web.projectArea != "undefined", "ProjectArea has not been initialized correctly");
	var paName = com.ibm.team.process.web.projectArea.name;
	//if (paName && dojo.trim(paName.toLowerCase()) === "jazz project") {
	//	return "The Jazz Project";
	//}
	return paName;
}

dojo.declare("com.ibm.team.process.web.ui.internal.ProjectAreaApplication", _JazzApplication, { //$NON-NLS-1$

	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	init: function() {
		var srh = new ServiceResponseHandler(
			this, "_fetchInitDataSuccess", "_fetchInitDataFailure"); //$NON-NLS-1$ //$NON-NLS-2$
		RepositoryClient.getInitializationData(srh);
	},
	
	_fetchInitDataSuccess: function(initializationData) {
		checkNotNull(initializationData, "initializationData"); //$NON-NLS-1$
		
		var loadSuccess = this._loadInitializationData(initializationData);
		if (!loadSuccess) {
			return;
		}
		// init UI in superclass
		var args = {
			label: getProjectAreaName(),
			showLoginPanel: true,
			projectAreas: this.projectAreas,
			archived: com.ibm.team.process.web.projectArea.archived
		};
		this._initUI(args);
		PlatformUI.getWorkbench()._setDefaultTitle(this.messages.ProjectAreaApplication_1);
		ProjectAreaApplication.superclass.start.call(this);
	},
	
	_loadInitializationData: function(initializationData) {
		InitializationData._load(initializationData);
		
		// set the global var for the logged-in contributor
		try {
			dojo.setObject(
				"com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", //$NON-NLS-1$
				InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor); //$NON-NLS-1$
		} catch (e) {
			document.title = this.messages.ProjectAreaApplication_2;
			var content = document.createElement("div"); //$NON-NLS-1$
			content.style.padding = "1em"; //$NON-NLS-1$
			var mainContent = document.createElement("div"); //$NON-NLS-1$
			mainContent.appendChild(document.createTextNode(this.messages.ProjectAreaApplication_3));
			var adminContent = document.createElement("div"); //$NON-NLS-1$
			adminContent.style.paddingTop = "8px"; //$NON-NLS-1$
			var adminWebUILink = dojo.string.substitute("<a href=\"${0}\">${1}</a>", [ net.jazz.ajax._contextRoot + "/admin", this.messages.ProjectAreaApplication_4]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			adminContent.innerHTML = dojo.string.substitute(this.messages.ProjectAreaApplication_5, [ adminWebUILink ]);
			content.appendChild(mainContent);
			content.appendChild(adminContent);
			var dialog = new Dialog({primaryTitle: this.messages.ProjectAreaApplication_6, contentNode: content});
			return;
		}
		
		// if the user is the guest user and we're in HTTPS mode, display the
		// appropriate error
		if(isGuest() && window.location.href.match(/^https/)) {
			alert(this.messages.ProjectAreaApplication_7);
			return false; // early return
		}
		
		// set the global far for the current project area based on the last
		// segment of the document's URI and the set of project areas in the
		// repository
		var pathname = window.location.pathname;
		var projectName = pathname.substring(pathname.lastIndexOf('/')+1); //$NON-NLS-1$
		var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas; //$NON-NLS-1$
		this.projectAreas = projectAreas;
		var projectArea = dojo.setObject("com.ibm.team.process.web.projectArea", {}); //$NON-NLS-1$
		for (var i = 0; i < projectAreas.length; i++) {
			var decodedName = decodeURI(projectName);
			// decodeURI does not properly decode %3F to "?".  We need to force the decoding here.
			decodedName = decodedName.replace(/%3F/g,"?"); //$NON-NLS-1$
			if (projectAreas[i].name == decodedName) {
				projectArea.itemId = projectAreas[i].itemId;
				projectArea.name = projectAreas[i].name;
				projectArea.archived = projectAreas[i].archived;
				break;
			}
		}
		
		// if we couldn't find a project area for the current URI, redirect to
		// root web UI
		if (!projectArea.itemId) {
			alert(dojo.string.substitute(this.messages.ProjectAreaApplication_8, [ projectName ]));
			var webBaseURL = pathname.substring(0, pathname.lastIndexOf('/')); //$NON-NLS-1$
			webBaseURL = webBaseURL.substring(0, webBaseURL.lastIndexOf('/')); //$NON-NLS-1$
			var url = window.location.protocol + "//" + window.location.host + webBaseURL; //$NON-NLS-1$
			window.location.href = url;
			return false;
		}
		
		// store the configuration properties in global vars with corresponding
		// names
		var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer"); //$NON-NLS-1$
		for (var propName in configProperties) {
			var propVal = configProperties[propName];
			dojo.setObject(propName, propVal);
		}
		return true; // load succeeded
	},
	
	_fetchInitDataFailure: function(errorObject) {
		if(errorObject.status != 401)
			throw new Error(dojo.string.substitute(this.messages.ProjectAreaApplication_9, [ errorObject.message ]));
		
		var guestAccessIsAllowed= ws.allow.guest.access === "true"; //$NON-NLS-1$
		
		// TODO factor http/https logic into a common place
		var isHttp= location.href.match(/^https/) === null;
		
		var message= this.messages.ProjectAreaApplication_11;
		
		if (isHttp) {
			// if login failed in HTTP mode, redirect to HTTPS mode to force
			// the user to log in
			alert(message);
			window.location = "https://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.https + window.location.pathname + window.location.search + window.location.hash; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (guestAccessIsAllowed) {
			// if login failed in HTTPS mode and guest access is allowed,
			// redirect to HTTP mode
			alert(dojo.string.substitute(this.messages.ProjectAreaApplication_10, [ message ]));
			window.location = "http://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.http + window.location.pathname + window.location.search + window.location.hash; //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			// otherwise just alert the user that login failed
			alert(message);
		}
	}
	
});

var ProjectAreaApplication = com.ibm.team.process.web.ui.internal.ProjectAreaApplication;

})();
