/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.StatisticsViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.UUIDUtility");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ParameterNameChooserDialog");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StatisticsMessages");

(function() {
	
/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var WorkItemQueryPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;
var UUIDUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.UUIDUtility;
var ParameterNameChooserDialog = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ParameterNameChooserDialog;
var ResizeNotifier = com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier();
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

/*
 * TODOs
 */
// TODO: MP: Show labels on zoom in for PIE
// TODO: MP: Show labels on zoom in for BAR
// TODO: MP: Verify done refresh called everywhere there is a return statement (i.e. configure message)

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.StatisticsViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/StatisticsViewlet.html"),

	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * @return StatisticsViewlet
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "StatisticsMessages");
		
	},
	
	init: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		this._viewletID = this._static._instanceCount;
		this._queryPreferenceProvider = new WorkItemQueryPreferenceProvider(
				{ prefId: this._static._PREFERENCE_ID_QUERY });
		this._cachedChartParameters = null;
		this._currentQueryItemID = null;
		this._lastClientWidth = this._static._DEFAULT_CLIENT_WIDTH;
		this._lastClientHeight = this._static._DEFAULT_CLIENT_HEIGHT;
		this._excessParameterStartIndex = -1;
		
		// Window resize listener related variables
		this._windowResizeHandler = null;
		// Tables
		this._tableLabelElements = null;
		// Pie / Bar Charts
		this._chartImageElement = null;
		this._chartRequestTimeoutID = null;
		// Lists
		this._listEntryFilter = null;
		
		// Registries
		this._internalValueRegistry = new Array();
		this._iconURLRegistry = new Array();
		
		// Report node reference
		this._reportNode = null;
		
		// Tag cloud frequency control value
		this._frequenciesVisible = false;
		// Tag cloud frequency area node
		this._frequenciesAreaNode = null;
		
		// TODO: MP: Will become obsolete when getTitle API available
		this._currentViewletTitle = null;
		
		// Global handlers
		this._viewletMouseOverHandlers = new Object();
		this._viewletMouseOutHandlers = new Object();

		// Create global viewlet handlers
		this._initializeGlobalHandlers();
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.StatisticsViewlet";

		this._static._PREFERENCE_ID_QUERY = "query";
		this._static._PREFERENCE_ID_PRESENTATION = "presentation";
		this._static._PREFERENCE_ID_CHARTID = "chartid";
		this._static._PREFERENCE_ID_PARAMETER = "parameter";
		
		this._static._PREFERENCE_VALUE_SEVERITY = "severity";
		this._static._PREFERENCE_VALUE_PRIORITY = "priority";
		this._static._PREFERENCE_VALUE_CATEGORY = "category";
		this._static._PREFERENCE_VALUE_OWNEDBY = "ownedby";
		this._static._PREFERENCE_VALUE_CREATEDBY = "createdby";
		this._static._PREFERENCE_VALUE_RESOLUTION = "resolution";
		this._static._PREFERENCE_VALUE_STATE = "state";
		this._static._PREFERENCE_VALUE_TYPE = "type";		
		this._static._PREFERENCE_VALUE_TAGS = "tags";
		
		this._static._PREFERENCE_VALUE_PIE = "pie";
		this._static._PREFERENCE_VALUE_BAR = "bar";
		this._static._PREFERENCE_VALUE_TABLE = "table";
		this._static._PREFERENCE_VALUE_LIST = "list";
		
		this._static._PREFERENCE_VALUE_UNDEFINED = "-1";

		this._static._STATISTICS_REST_SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.statistics.IStatisticsRestService";

		this._static._STATISTICS_RAW_IMAGE_SERVICE_URI = URLUtility.getContextRoot() + "/service/com.ibm.team.dashboard.viewlets.service.internal.statistics.IStatisticsImageRawService";

		this._static._ATTRIBUTE_VALUE_MAP = "chart-image-map";

		this._static._PARAMETER_NAME_SEVERITY = this.messages.msgSeverity;
		this._static._PARAMETER_NAME_PRIORITY = this.messages.msgPriority;
		this._static._PARAMETER_NAME_CATEGORY = this.messages.msgCategory;
		this._static._PARAMETER_NAME_OWNER = this.messages.msgOwnedBy;
		this._static._PARAMETER_NAME_CREATOR = this.messages.msgCreatedBy;
		this._static._PARAMETER_NAME_RESOLUTION = this.messages.msgResolution;
		this._static._PARAMETER_NAME_STATE = this.messages.msgStatus;
		this._static._PARAMETER_NAME_TYPE = this.messages.msgType;
		this._static._PARAMETER_NAME_TAGS = this.messages.msgTags;

		// Work Item INTERNAL code
		// Used to filter query results based on parameter
		this._static._INTERNAL_NAME_SEVERITY = "internalSeverity";
		this._static._INTERNAL_NAME_PRIORITY = "internalPriority";
		this._static._INTERNAL_NAME_CATEGORY = "category";
		this._static._INTERNAL_NAME_OWNER = "owner";
		this._static._INTERNAL_NAME_CREATOR = "creator";
		this._static._INTERNAL_NAME_RESOLUTION = "internalResolution";
		this._static._INTERNAL_NAME_STATE = "internalState";
		this._static._INTERNAL_NAME_TYPE = "workItemType";
		this._static._INTERNAL_NAME_TAGS = "tags";

		this._static._IMAGE_URI = "../../web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/";

		this._static._PARAMETER_UNDEFINED = this.messages.msgUndefined;
		this._static._PARAMETER_OTHER = this.messages.msgOther;

		// TODO: MP: Temporary workaround for 1.0.1. Resolve in 1.1
		// 0: Unassigned
		// 1: Low
		// 2: Medium
		// 3: High
		this._static._COLOUR_RGB_PRIORITY = ["R240G240B240", "R196G220B245", "R250G234B187", "R255G191B191"];
		this._static._COLOUR_REGEX_PRIORITY = ["unassign", "low", "medium", "high"];
		
		// TODO: MP: Temporary workaround for 1.0.1. Resolve in 1.1
		// 0: Unclassified
		// 1: Minor
		// 2: Normal
		// 3: Major
		// 4: Critical
		// 5: Blocker
		this._static._COLOUR_RGB_SEVERITY = ["R240G240B240", "R255G255B222", "R229G218B184", "R255G204B204", "R223G127B127", "R179G107B107"];
		this._static._COLOUR_REGEX_SEVERITY = ["unassign", "minor", "normal", "major", "critical", "blocker"];

		// TODO: MP: Temporary workaround for 1.0.1. Resolve in 1.1
		// 0: Defect
		// 1: Enhancement
		// 2: Task
		// 3: RFS
		// 4: Retrospective
		// 5: Plan Item
		// 6: Track Build Item
		// 7: Story
		this._static._COLOUR_RGB_TYPE = ["R206G206B242", "R229G247B210", "R189G245B188", "R204G163B163", "R255G191B191", "R217G195B173", "R207G250B250", "R229G229B161"];
		this._static._COLOUR_REGEX_TYPE = ["bug", "enhanc", "task", "rfs", "risk", "planitem", "build", "story"];

		this._static._COLOUR_RGB_UNDEFINED = "0";

		// Table Icons
		this._static._IMAGE_DEFAULT = "icon-default.gif";
		this._static._IMAGE_CATEGORY = "icon-category.gif";
		this._static._IMAGE_TAGS = "icon-tags.gif";
		this._static._IMAGE_OTHER = "icon-other.gif";

		this._static._FONT_SIZE_RATIO = 40;
		
		this._static._REPORT_DOM_ID = "live_report";
		
		this._static._DEFAULT_MIN_HEIGHT = 10;
		this._static._TIMEOUT_ADJUST_HEIGHT = 250;
		this._static._TIMEOUT_GET_CHART = 1000;

		this._static._MIN_CHART_FONT_SIZE = 9;  // pixels
		this._static._MAX_CHART_FONT_SIZE = 9;  // pixels
		
		this._static._DEFAULT_CLIENT_WIDTH = 325; // pixels
		this._static._DEFAULT_CLIENT_HEIGHT = 162; // pixels (assuming pie chart ratio)
		
		this._static._LEGEND_ITEM_HEIGHT = 21;  // pixels
		
		this._static._CHART_SCALE_RATIO = 0.95;
		this._static._CHART_MINIMUM_WIDTH = 250;
		this._static._CHART_MAXIMUM_WIDTH = 1000;	
		this._static._CHART_HEIGHT_SCALE_RATIO_PIE_CHART = 2;
		this._static._CHART_HEIGHT_SCALE_RATIO_BAR_CHART = 1.5;	
		
		this._static._MIN_LIST_FONT_SIZE = 12;  // pixels
		this._static._MAX_LIST_FONT_SIZE = 24;  // pixels		
		this._static._DEFAULT_LIST_FONT_SIZE = 12;  // pixels
		this._static._MAX_LIST_DISPLAY_ITEMS = 30;	

		this._static._LIST_FILTER_ACTION_MORE = "more";
		this._static._LIST_FILTER_ACTION_LESS = "less";
		this._static._LIST_FILTER_ICON_MORE = "chev-dwn.gif";
		this._static._LIST_FILTER_ICON_LESS = "chev-up.gif";
		this._static._LIST_FILTER_ICON_ID = "list-filter-icon";

		this._static._LIST_FREQUENCY_ICON_ENABLED = "show-details.gif";
		this._static._LIST_FREQUENCY_ICON_DISABLED = "hide-details.gif";

		this._static._MAX_TABLE_DISPLAY_ITEMS = 10;	
		
		// Palette types
		this._static._PALETTE_TYPE_FIXED = 1;
		this._static._PALETTE_TYPE_RANDOM = 2;
		// Colours
		this._static._RGB_COLOUR_GREY = "R229G229B229";
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeGlobalHandlers: function() {
		// Handle mouse over events
		this.connect(this.domNode, "onmouseover", this._handleEventMouseOver);
		// Hanlde mouse out events
		this.connect(this.domNode, "onmouseout", this._handleEventMouseOut);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventMouseOver: function(event) {
		// Cannot stop event.  Viewlet framework needs to handle this event
		for (var listener in this._viewletMouseOverHandlers) {
			this[listener](event);
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleEventMouseOut: function(event) {
		// Cannot stop event.  Viewlet framework needs to handle this event
		for (var listener in this._viewletMouseOutHandlers) {
			this[listener](event);
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number index
 	 * @return void
 	 */	
	_setExcessParameterStartIndex: function(index) {
		this._excessParameterStartIndex = index;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getExcessParameterStartIndex: function() {
		return this._excessParameterStartIndex;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array - Element values
 	 */	
	_getTableLabelElements: function() {
		return this._tableLabelElements;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_resetTableLabelElements: function() {
		this._tableLabelElements = new Array();
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element element
 	 * @return void
 	 */	
	_addTableLabelElement: function(element) {
		this._tableLabelElements.push(element);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element element
 	 * @return void
 	 */	
	_setChartImageElement: function(element) {
		this._chartImageElement = element;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Element element
 	 */	
	_getChartImageElement: function() {
		return this._chartImageElement;
	},	
	
	/*
	 * Private Method
	 * 
	 * @param Number ratio
	 * @return Object
	 * 			. Number width
	 * 			. Number height
	 */	
	_getPreferredChartDimensions: function(ratio) {
		// Create the container
		var container = new Object();
		// Get the current width of the viewlet
		var width = this._getClientWidth();
		
		// Constrain width to acceptable boundaries
		var preferredWidth = width;
		if (width < this._static._CHART_MINIMUM_WIDTH) {
			preferredWidth = this._static._CHART_MINIMUM_WIDTH;
		} else if (width > this._static._CHART_MAXIMUM_WIDTH) {
			preferredWidth = this._static._CHART_MAXIMUM_WIDTH;
		}
		// Adjust height proportionally
		var preferredHeight = Math.round(preferredWidth / ratio);
		
		// Set the width
		container.width = preferredWidth;
		// Set the height
		container.height = preferredHeight;

		return container;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number height
 	 * @return Number
 	 */	
	_getMaxLegendItems: function(height) {
		// Note:  This calculation is only valid if the legend item font size
		// specified on the server to BIRT is 7.0 pt
		var maxLegendItems = Math.round(height / this._static._LEGEND_ITEM_HEIGHT);
		return maxLegendItems;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * Note:  A direct reference to the report node is now used instead of
 	 * retrieving the node by ID
 	 * 
 	 * @param void
 	 * @return String
 	 */	
	_getReportDomID: function() {
		return this._static._REPORT_DOM_ID + "_" + this._viewletID;	
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String uri
 	 * @param String imageMapHtml
 	 * @return <none>
 	 */	
	_createImage: function(uri, imageMapHtml) {
		// Get the attach point
		var node = this._contentArea;
		// Create a wrapping div
		var div = document.createElement("div");
		dojo.addClass(div, "image");
		div.setAttribute("id", this._getReportDomID());
		// Create the image
		var img = document.createElement("img");
		// Create the unique image map name
		var imageMapName = this._static._ATTRIBUTE_VALUE_MAP +
							"_" +
							this._viewletID;
		// Set the image attributes		
		img.setAttribute("src", uri);
		// Set the hover
		img.setAttribute("alt", this._currentViewletTitle);		
		// Last parameter means case insensitive (0) - workaround for IE
		img.setAttribute("usemap", "#" + imageMapName, 0);	
		// Add the image to the div
		div.appendChild(img);
		// Register the image
		this._setChartImageElement(img);
		
		// Create the image map
		var map = document.createElement("map");
		// Set the map attributes
		map.setAttribute("id", imageMapName); 
		map.setAttribute("name", imageMapName); 
		// Add the image map html
		map.innerHTML = imageMapHtml;
		// Add the image map to the div
		div.appendChild(map);
		
		// Add the div to the node
		this._addReportNode(node, div);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getMaxTableLabelWidth: function() {
		// All units are in pixels
		var viewletAreaWidth = this._getClientWidth();
		// Matches property in "label-entry" class
		var columnWidthPercentage = 0.7; // 70%
		var labelColumnWidth = Math.round(viewletAreaWidth * columnWidthPercentage);
		var margins = 20;
		var imageWidth = 16;
		var maxWidth = labelColumnWidth - margins - imageWidth;
		
		return maxWidth;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateTableLabelElementWidths: function() {
		// Get the elements
		var elements = this._getTableLabelElements();
		// Ensure we have elements
		if (elements == null) {
			return;
		}		
		// Get the maximum table label length
		var maxWidth = this._getMaxTableLabelWidth();

		// Update their lengths
		for (var i = 0; i < elements.length; i++) {
			var element = elements[i];
			element.style.width = maxWidth + "px";
		}		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_disconnectWindowResizeHandler: function() {
		// Disconnect the previous handler (if any)
		if (this._windowResizeHandler != null) {
			ResizeNotifier.unregister(this._windowResizeHandler);
			this._windowResizeHandler = null;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_registerTableResizeHandler: function() {
		// Clear previous elements (if any)
		this._resetTableLabelElements();
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();
		// Connect a new resize handler
		this._windowResizeHandler = ResizeNotifier.register({context: this, callback: this._updateTableLabelElementWidths}); 
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 
 	 * @return void
 	 */		
	_createTable: function(chartParameters) {
		// Register the table resize handler
		this._registerTableResizeHandler();
		
		// Regroup data to fit the viewlet area
		var regroupedChartParams = this._regroupDataToFitViewletArea(
			chartParameters, 
			this._static._MAX_TABLE_DISPLAY_ITEMS);
		
		// Get the attach point
		var node = this._contentArea;
		// Create the table
		var table = document.createElement("table");
		table.setAttribute("id", this._getReportDomID());
		table.setAttribute("cellspacing", "1");
		// Create the table body container
		var tbody = document.createElement("tbody");
		// Create the table body
 		for (var i = 0; i < regroupedChartParams.parameterCount; i++) {
			// Get the name
			var name = regroupedChartParams.parameterNames[i];
			// Get the value
			var value = regroupedChartParams.parameterValues[i];
			// Create the table row
			this._createTableRow(tbody, name, value, regroupedChartParams.parameterID);
		}
		this._removeLastTableRowBorder(tbody);

		// Create the table header
		var thead = this._createTableHeader(regroupedChartParams);
 		// Assemble table contents
 		table.appendChild(thead);
 		table.appendChild(tbody);
		
		this._addReportNode(node, table);

		// Update the table label widths
		this._updateTableLabelElementWidths();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element tbody
 	 * @return void
 	 */	
	_removeLastTableRowBorder: function(tbody) {
		// Remove the border from the last table row
		var lastRow = tbody.lastChild;
		// Column #1
		if (this._isObjectDefined(lastRow.firstChild, false, null)) {
			dojo.removeClass(lastRow.firstChild, "bottom-line");
		}
		// Column #2
		if (this._isObjectDefined(lastRow.lastChild, false, null)) {
			dojo.removeClass(lastRow.lastChild, "bottom-line");
		}	
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return Element
 	 */		
	_createTableHeader: function(chartParameters) {
		// Create the table head container
		var thead = document.createElement("thead");
		
		// Row #1
		var row1 = document.createElement("tr");
		// Table head 1
		var row1_th1 = document.createElement("th");
		dojo.addClass(row1_th1, "header-topline");
		var line1 = document.createElement("div");
		dojo.addClass(line1, "topline");
		row1_th1.appendChild(line1);
		// Table head 2
		var row1_th2 = document.createElement("th");	
		dojo.addClass(row1_th2, "header-topline");
		var line2 = document.createElement("div");
		dojo.addClass(line2, "topline");
		row1_th2.appendChild(line2);
		// Assemble contents
		row1.appendChild(row1_th1);
		row1.appendChild(row1_th2);		
		
		// Row #2
 		// Create the table header
		var row2 = document.createElement("tr");
		// Table head 1
		var row2_th1 = document.createElement("th");
		dojo.addClass(row2_th1, "parameter-header");
		// Table head 2
		var row2_th2 = document.createElement("th");	
		dojo.addClass(row2_th2, "count-header");
		// Create headings
		var header1 = document.createTextNode(chartParameters.displayName);
		var header2 = document.createTextNode(this.messages.msgCount);
		// Assemble table header contents
		row2_th1.appendChild(header1);
		row2_th2.appendChild(header2);
		row2.appendChild(row2_th1);
		row2.appendChild(row2_th2);
		
		thead.appendChild(row1);
		thead.appendChild(row2);
		
		return thead;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element tbody
 	 * @param String name
 	 * @param String value
 	 * @param String parameterID
 	 * @return void
 	 */		
	_createTableRow: function(tbody, name, value, parameterID) {
 		// Row
		var tr = document.createElement("tr");
		// Column 1
		var td1 = document.createElement("td");
		dojo.addClass(td1, "label-entry");
		dojo.addClass(td1, "bottom-line");
		// Column 2
		var td2 = document.createElement("td");
		dojo.addClass(td2, "count-entry");
		dojo.addClass(td2, "bottom-line");
		// Icon
		var img = document.createElement("img");
		var iconURL = this._getIconURL(parameterID, name);
		img.src = iconURL;
		img.width = "16";
		img.height = "16";
		img.alt = this.messages.msgIcon;
		// Name
		var link = this._createTableClickThroughLink(name, value);
		var span1 = document.createElement("span");
		dojo.addClass(span1, "label");
		this._addTableLabelElement(span1);
		span1.appendChild(link);
		// Value
		var data2 = document.createTextNode(value);
		var span2 = document.createElement("span");
		span2.appendChild(data2);		
		// Assemble contents
		td1.appendChild(img);
		td1.appendChild(span1);
		td2.appendChild(span2);
		tr.appendChild(td1);
		tr.appendChild(td2);
		tbody.appendChild(tr);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * @param String value
 	 * @return Element
 	 */	
	_createTableClickThroughLink: function(name, value) {
		// Name
		var data1 = document.createTextNode(name);		
		// Create click through link on name
		var link = document.createElement("a");
		link.setAttribute("title", value);
		link.appendChild(data1);
		// Create a reference to this object to use in the service object
		var self = this;		
		// Handle click events
		this.connect(link, "onclick", 
			function() {
				var label = name;
				var count = -1;  // Not used
				var queryItemID = self._getQueryItemID();
				// Get chart parameters
				var chartParameters = self._getCachedChartParameters();
				// Validate chart parameters
				if (chartParameters == null) {
					// This should never happen
					self._alertErrorMessage(
						self.messages.msgDrillChartParamsUndef, null);
					return;
				}
				// Get the parameter ID
				var parameterID = chartParameters.parameterID;
				// Get the filtered work item query results
				self._getWorkItemQueryFilteredResults(label, count, queryItemID, parameterID);
			}
		);
		
		return link;				
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return void
 	 */		
	_createList: function(chartParameters) {
		// Create the list entry filter (if necessary)
		this._createListEntryFilter(chartParameters);
		// Calculate the list font size ratio
		var maxValue = chartParameters.maxValue;
		var minValue = chartParameters.minValue;
		var fontSizeRatio = this._calculateListFontSizeRatio(
			minValue, maxValue);
		// Get the attach point
		var node = this._contentArea;
		// Create the list
		var list = document.createElement("div");
		list.setAttribute("id", this._getReportDomID());
		dojo.addClass(list, "weighted-list");
		dojo.addClass(list, "hide-frequencies");
		// Create the table body
 		for (var i = 0; i < chartParameters.parameterCount; i++) {
			// Get the name
			var name = chartParameters.parameterNames[i];
			// Get the value
			var value = chartParameters.parameterValues[i];
			// Add the entry to the list
			this._createListEntry(list, name, value, fontSizeRatio, maxValue);
		}
		// Add the list entry filter control (if necessary)
		this._createListControlArea(list);
 		// Assemble table contents
		this._addReportNode(node, list);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return void
 	 */		
	_createListEntryFilter: function(chartParameters) {
		// Clear filter
		this._listEntryFilter = null;
		// Determine if there is anything to do
		if (chartParameters.parameterCount <= this._static._MAX_LIST_DISPLAY_ITEMS) {
			return;
		}
		// Create filter
		this._listEntryFilter = new Array();		
		// Clone the parameters
		var newContainer = this._cloneContainer(chartParameters);
		// Sort the names and values by descending value
		this._parameterValuesShellSort(newContainer);
		// Only the list entries with the highest values will be displayed
		// Populate filter with the list items that exceed the maximum number
		// of displayable items
		for (var i = this._static._MAX_LIST_DISPLAY_ITEMS; i < newContainer.parameterCount; i++) {
			// Get the name
			var name = newContainer.parameterNames[i];
			// Add name to filter
			// Name is key
			// Value is null for now (will be populated with DOM element later)
			this._listEntryFilter[name] = null;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return void
 	 */		
	_createListControlArea: function(parent) {
		// Create container
		var container = document.createElement("div");
		dojo.addClass(container, "list-control-area");
		// Add the list entry filter control (if necessary)
		this._createListEntryFilterControl(container);
		// Add the list entry frequency control
		this._createListEntryFrequencyControl(container);
		parent.appendChild(container);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return String
 	 */	
	_getFilterImageDomID: function() {
		return this._static._LIST_FILTER_ICON_ID + "_" + this._viewletID;	
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return void
 	 */		
	_createListEntryFilterControl: function(parent) {
		// Determine if there is anything to do
		if (this._listEntryFilter == null) {
			return;
		}
		// Create container
		var container = document.createElement("span");
		dojo.addClass(container, "list-filter-area");
		// Create image
		var image = document.createElement("img");
		image.src = this._static._IMAGE_URI + this._static._LIST_FILTER_ICON_MORE;
		image.alt = this.messages.msgDownArrow;
		image.id = this._getFilterImageDomID();
		// Create link
		var link = document.createElement("a");
		dojo.addClass(link, "list-control-link");
		link.name = this._static._LIST_FILTER_ACTION_MORE;
		link.innerHTML = this.messages.msgShowMore;
		// Handle click events
		this.connect(link, "onclick", this._handleEventFilterControlClicked);
		// Assemble contents
		container.appendChild(image);
		container.appendChild(link);
		parent.appendChild(container);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @return void
 	 */		
	_createListEntryFrequencyControl: function(parent) {
		// Create container
		this._frequenciesAreaNode = document.createElement("span");
		dojo.addClass(this._frequenciesAreaNode, "list-frequency-area");
		if (dojo.isIE) {
			this._frequenciesAreaNode.style.bottom = "2px";
		}
		
		// Create image
		var image = document.createElement("img");
		image.src = this._static._IMAGE_URI + this._static._LIST_FREQUENCY_ICON_DISABLED;
		image.alt = this.messages.msgShowFrequencies;
		image.title = this.messages.msgShowFrequencies;

		// Update toggle status
		this._frequenciesVisible = false;

		// Handle click events
		this.connect(this._frequenciesAreaNode, "onclick", this._handleEventFrequencyControlClicked);
		// Handle mouse over events
		this._viewletMouseOverHandlers["_handleFrequencyEventMouseOver"] = true;
		// Handle mouse out events
		this._viewletMouseOutHandlers["_handleFrequencyEventMouseOut"] = true;
		
		// Assemble contents
		this._frequenciesAreaNode.appendChild(image);
		parent.appendChild(this._frequenciesAreaNode);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleFrequencyEventMouseOver: function(event) {
		if (this._frequenciesAreaNode == null) {
			return;
		}
		this._updateVisibility(this._frequenciesAreaNode, true);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_handleFrequencyEventMouseOut: function(event) {
		if (this._frequenciesAreaNode == null) {
			return;
		}
		this._updateVisibility(this._frequenciesAreaNode, false);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventFilterControlClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get the target and action
		var target = null;
		var action = null;
		try {
			this._hasProperty(event, "target", true, "event");
			target = event.target;
			this._isObjectDefined(target, true, "target");
			this._hasProperty(target, "name", true, "target");
			action = target.name;
			this._isStringDefined(action, true, "name");			
		} catch(errorObject) {
			this._alertErrorMessage(this.messages.msgEventParseFailure, errorObject);
			return;
		}
		// Get the image
		var image = dojo.byId(this._getFilterImageDomID());
		// Verify we got the image
		if (image == null) {
			this._alertErrorMessage(this.messages.msgFilterImageNotFound, null);
			return;
		}
		// Select action
		if (action == this._static._LIST_FILTER_ACTION_MORE) {
			// Toggle link
			target.innerHTML = this.messages.msgShowLess;
			target.name = this._static._LIST_FILTER_ACTION_LESS;
			// Toggle image
			image.src = this._static._IMAGE_URI + this._static._LIST_FILTER_ICON_LESS;
			image.alt = this.messages.msgUpArrow;
			// Perform action
			this._updateFilterListEntries(true);
		} else if (action == this._static._LIST_FILTER_ACTION_LESS) {
			// Toggle link
			target.innerHTML = this.messages.msgShowMore;
			target.name = this._static._LIST_FILTER_ACTION_MORE;
			// Toggle image
			image.src = this._static._IMAGE_URI + this._static._LIST_FILTER_ICON_MORE;
			image.alt = this.messages.msgDownArrow;
			// Perform action
			this._updateFilterListEntries(false);
		} else {
			this._alertErrorMessage(dojo.string.substitute(this.messages.msgUnknownFilterAction, [action]), null);
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventFrequencyControlClicked: function(event) {
		// Stop event
		dojo.stopEvent(event);
		// Get image
		var image = this._frequenciesAreaNode.firstChild;

		if (this._frequenciesVisible) {
			// Toggle image
			image.src = this._static._IMAGE_URI + this._static._LIST_FREQUENCY_ICON_DISABLED;
			image.alt = this.messages.msgShowFrequencies;
			image.title = this.messages.msgShowFrequencies;

			// Update toggle status
			this._frequenciesVisible = false;

			// Perform action
			this._updateFrequencyListEntries(false);
		} else {
			// Toggle image
			image.src = this._static._IMAGE_URI + this._static._LIST_FREQUENCY_ICON_ENABLED;
			image.alt = this.messages.msgHideFrequencies;
			image.title = this.messages.msgHideFrequencies;

			// Update toggle status
			this._frequenciesVisible = true;

			// Perform action
			this._updateFrequencyListEntries(true);
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	_updateFrequencyListEntries: function(show) {
		// Weighted list div
		var node = this._contentArea.firstChild;
		
		if (show) {
			// Show
			if (dojo.hasClass(node, "show-frequencies") == false) {
				dojo.addClass(node, "show-frequencies");
			}
			if (dojo.hasClass(node, "hide-frequencies")) {
				dojo.removeClass(node, "hide-frequencies");
			}
		} else {
			// Hide
			if (dojo.hasClass(node, "hide-frequencies") == false) {
				dojo.addClass(node, "hide-frequencies");
			}
			if (dojo.hasClass(node, "show-frequencies")) {
				dojo.removeClass(node, "show-frequencies");
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */		
	_updateFilterListEntries: function(show) {
		// Determine if there is anything to do
		if (this._listEntryFilter == null) {
			return;
		}
		// Determine whether to show or hide
		var display = null;
		if (show) {
			display = "inline";
		} else {
			display = "none";
		}
		// Perform show or hide
		for (var name in this._listEntryFilter) {
			// Get the element
			var element = this._listEntryFilter[name];
			// Ensure element is defined
			if (element == null) {
				this._alertErrorMessage(dojo.string.substitute(this.messages.msgNoElementWithName, [name]), null);
				continue;
			}
			// Update display
			element.style.display = display;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element targetParent
 	 * @param Element newReportNode
 	 * @return void
 	 */		
	_addReportNode: function(targetParent, newReportNode) {
		// Get the old report node (if any)
		var oldReportNode = this._reportNode;		
		
		if (oldReportNode == null) {
			// To be safe, remove all children to prevent duplicate report nodes
			this._removeAllChildren(targetParent);
			// Add new report node
			targetParent.appendChild(newReportNode);
		} else {
			// Replace old report node with new report node
			targetParent.replaceChild(newReportNode, oldReportNode);
		}

		// Set new report node
		this._reportNode = newReportNode;
		// Unset the loading message and show the content area
		this._displayLoadingMessage(null);
		// Make the content and message areas the same height as the report
		this._readjustDisplayAreasHeight();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element list
 	 * @param String name
 	 * @param String value
 	 * @param Number fontSizeRatio
 	 * @param Number maxValue
 	 * @return void
 	 */		
	_createListEntry: function(list, name, value, fontSizeRatio, maxValue) {
		// Create the link
		var link = this._createTableClickThroughLink(name, value);
		// Create the text
		var frequency = document.createElement("span");
		dojo.addClass(frequency, "frequency");
		frequency.innerHTML = "(" + value + ")";
		// Create the span
		var container = document.createElement("span");
		dojo.addClass(container, "list-entry");
		container.style.fontSize = this._calculateListEntryFontSize(value, fontSizeRatio, maxValue);
		// Determine whether to filter this entry
		if (this._isListEntryFiltered(name)) {
			// Store node
			this._listEntryFilter[name] = container;
			// Hide node
			container.style.display = "none";
		}
		// Assemble contents
		// Do not allow breaking between the link and frequency
		var nobr = document.createElement("nobr");
		nobr.appendChild(link);
		nobr.appendChild(frequency);
		container.appendChild(nobr);
		// Create whitespace in order to allow breaking between list entries
		container.appendChild(document.createTextNode(" "));
		
		list.appendChild(container);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * @return boolean
 	 */		
	_isListEntryFiltered: function(name) {
	
		if (this._listEntryFilter == null) {
			return false;
		} else if ((name in this._listEntryFilter) == false) {
			return false;
		}
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Number minValue
 	 * @param Number maxValue
 	 * @return Number
 	 */		
	_calculateListFontSizeRatio: function(minValue, maxValue) {
		var fontSizeDifference = this._static._MAX_LIST_FONT_SIZE -
								 this._static._MIN_LIST_FONT_SIZE;
		var valueDifference = maxValue - minValue;
		var ratio = 0;
		if (valueDifference > 0) {
			ratio = fontSizeDifference / valueDifference;
		} else {
			// Edge case
			// The min and max values are the same (i.e. the list values are 
			// all of equal weight)
			// A ratio of -1 indicates that the default font size should be
			// used for all values (used to be maximum font size)
			ratio = -1;
		}

		return ratio;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Number value
 	 * @param Number ratio
 	 * @param Number maxValue
 	 * @return String
 	 */		
	_calculateListEntryFontSize: function(value, ratio, maxValue) {
		var fontSize;
		if (ratio < 0) {
			// Edge case
			// The list values are all of equal weight
			fontSize = this._static._DEFAULT_LIST_FONT_SIZE;
		} else {
			var scaledValue = Math.abs(value - maxValue) * ratio;
			fontSize = Math.round(this._static._MAX_LIST_FONT_SIZE - scaledValue);			
		}

		return fontSize + "px";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getCurrentWidth: function() {
		if (this._isObjectDefined(this.domNode, false, "domNode") == false) {
			return 0;
		} else if (this._hasProperty(this.domNode, "clientWidth", false, "domNode") == false) {
			return 0;
		}
		
		return this.domNode.clientWidth;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getCurrentHeight: function() {
		if (this._isObjectDefined(this.domNode, false, "domNode") == false) {
			return 0;
		} else if (this._hasProperty(this.domNode, "clientHeight", false, "domNode") == false) {
			return 0;
		}

		return this.domNode.clientHeight;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getClientWidth: function() {
		// Calculate client width
		// Use content area width (target)
		var clientWidth = Math.round(this.getSite().getWidth() *
			this._static._CHART_SCALE_RATIO);
		// Validate the width
		if (clientWidth <= 0) {
			// Use the current width
			clientWidth = Math.round(this._getCurrentWidth() *
				this._static._CHART_SCALE_RATIO);
		}
		// Validate the width
		if (clientWidth <= 0) {
			// Use the last valid width
			clientWidth = this._lastClientWidth;
		} else {
			// Set as the last valid width
			this._lastClientWidth = clientWidth;
		}

		return clientWidth;
	},	

	/*
 	 * DEAD CODE (at the moment)
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	_getClientHeight: function() {
		// Calculate client height
		// Use content area height (target)
		var clientHeight = Math.round(this.getSite().getHeight() *
			this._static._CHART_SCALE_RATIO);
		// Validate the height
		if (clientHeight <= 0) {
			// Use the current height
			clientHeight = Math.round(this._getCurrentHeight() *
				this._static._CHART_SCALE_RATIO);
		}
		// Validate the height
		if (clientHeight <= 0) {
			// Use the last valid height
			clientHeight = this._lastClientHeight;
		} else {
			// Set as the last valid height
			this._lastClientHeight = clientHeight;
		}
		
		return clientHeight;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterName
 	 * @return String
 	 */	
	_getIconURL: function(parameterID, parameterName) {

		if (parameterName === this._static._PARAMETER_OTHER) {
			// Other
			return this._static._IMAGE_URI + this._static._IMAGE_OTHER;
		} else if (parameterID == this._static._PREFERENCE_VALUE_CATEGORY) {
			// Category
			return this._static._IMAGE_URI + this._static._IMAGE_CATEGORY;
		} else if (parameterID == this._static._PREFERENCE_VALUE_TAGS) {
			// Tags
			return this._static._IMAGE_URI + this._static._IMAGE_TAGS;
		} else if (parameterName == this._static._PARAMETER_UNDEFINED) {
			// Undefined
			return this._static._IMAGE_URI + this._static._IMAGE_DEFAULT;
		} else if (parameterName in this._iconURLRegistry) {
			// Severity
			// Priority
			// Resolution
			// Status
			// Type
			// Owned By
			// Created By
			return this._iconURLRegistry[parameterName];
		}
		// Default
		return this._static._IMAGE_URI + this._static._IMAGE_DEFAULT;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showMessageArea: function() {
		// Hide the content area
		this._updateVisibility(this._contentArea, false);
		// Show the message area		
		this._updateVisibility(this._messageArea, true);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param Number minHeight
 	 * @return void
 	 */	
	_setMinHeight: function(node, minHeight) {
		node.style.minHeight = minHeight + "px";
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_readjustDisplayAreasHeight: function() {
		// Operation not applicable when in zoom mode
		if (this.isZoomed()) {
			return;
		}
		// Create a reference to this object to use in the service object
		var self = this;
		// Adjust the report height after the image loads
		var timeoutID = window.setTimeout(
			function() {
				// Get the current report height
				var reportHeight = self._static._DEFAULT_MIN_HEIGHT;
				if (self._reportNode != null) {
					reportHeight = self._reportNode.clientHeight;
				}
				// Make the heights of the message and content areas the same
				// height as the report
				self._setMinHeight(self._contentArea, reportHeight);
				self._setMinHeight(self._messageArea, reportHeight);
 			}
			, this._static._TIMEOUT_ADJUST_HEIGHT);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showContentArea: function() {
		// Hide the message area
		this._updateVisibility(this._messageArea, false);	
		// Show the content area
		this._updateVisibility(this._contentArea, true);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	_generateChartID: function() {
		// ID Format
		// <year>-<month>-<day>_<hour>-<minute>_<second>-<millisecond>_<viewletID>
		// e.g 2007-11-13_13-35_20-500_1
		var dash = "-";
		var underscore = "_";
		var currentTime = new Date();

		var chartID = 
			currentTime.getUTCFullYear() +
			dash +
			currentTime.getUTCMonth() +
			dash +
			currentTime.getUTCDate() +
			underscore +
			currentTime.getUTCHours() +
			dash +
			currentTime.getUTCMinutes() +
			underscore +
			currentTime.getUTCSeconds() +
			dash +
			currentTime.getUTCMilliseconds() +
			underscore +
			this._viewletID;

		return chartID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String presentation
 	 * @param Object chartParameters
 	 * @param Number chartHeightScaleRatio
 	 * @return <none>
 	 */
	_getChart: function(presentation, chartParameters, chartHeightScaleRatio) {
		// Set loading message
		this._displayLoadingMessage(this.messages.msgLoading);
		// Generate a unique chart ID
		var chartID = this._generateChartID();
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(html) {
				self._createChart(html, chartID, presentation, chartParameters.total);
			},
			// Service failure
			failure: function(errorObject) {
				self._displayErrorMessage(self.messages.msgChartLoadingFailure, errorObject);
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		// Calculate the preferred chart width and height
		var dimensions = this._getPreferredChartDimensions(chartHeightScaleRatio);
		// Get the maximum number of legend items that can fit in the 
		// vertical area of the viewlet
		var maxLegendItems = this._getMaxLegendItems(dimensions.height);
		// Regroup data to fit the viewlet area
		var parameters = this._regroupDataToFitViewletArea(chartParameters, maxLegendItems);

		// Invoke the service
		this._callRESTGetChart(
					handler, 
					chartID, 
					presentation, 
					parameters.parameterNames,
					parameters.parameterValues,
					parameters.parameterColours,
					parameters.parameterCount,
					dimensions.width,
					dimensions.height,
					this.id,
					this._getQueryItemID(),
					chartParameters.parameterID,
					chartParameters.paletteType);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return void
 	 */	
	_dumpObject: function(container) {
		
		console.log("=== DUMP OBJECT START ===");
		
		for (var propertyName in container) {
			console.log(" ");
			console.log("Property Name: " + propertyName);
			
			var propertyValue = container[propertyName];
			
			if (propertyValue instanceof Array) {
				// Array
				console.log("Property Type: Array");
				console.log("Property Values: Length = " + propertyValue.length);
				this._dumpArray(propertyValue);
			} else if (typeof(propertyValue) === "string") {
				// String
				console.log("Property Type: String");
				console.log("Property Value: " + propertyValue);
			} else if (typeof(propertyValue) === "number") {
				// Number
				console.log("Property Type: Number");
				console.log("Property Value: " + propertyValue);
			} else if (typeof(propertyValue) === "object") {
				// Object
				console.log("Property Type: Object");
				this._dumpObject(propertyValue);
			} else if (typeof(propertyValue) === "function") {
				// Function
				console.log("Property Type: Function");
				console.log("Property Value: " + propertyValue);
			} else {
				console.log("Property Type: ?");
				console.log("Property Value: " + propertyValue);
			}			
		}

		console.log("=== DUMP OBJECT END ===");

	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array array - PREREQ: Scalar indexed entries
 	 * 
 	 * @return void
 	 */	
	_dumpArray: function(array) {
		for (var index = 0; index < array.length; index++) {
			console.log("[" + index + "]: " + array[index]);			
		}				
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return Object
 	 */	
	_cloneObject: function(container) {
		var newContainer = new Object();
		
		for (var propertyName in container) {
			var propertyValue = container[propertyName];
			
			if (propertyValue instanceof Array) {
				// Array
				newContainer[propertyName] = this._cloneArray(propertyValue);
			} else if (typeof(propertyValue) === "string") {
				// String
				newContainer[propertyName] = propertyValue;
			} else if (typeof(propertyValue) === "number") {
				// Number
				newContainer[propertyName] = propertyValue;
			} else if (typeof(propertyValue) === "object") {
				// Object
				newContainer[propertyName] = this._cloneObject(propertyValue);
			} else if (typeof(propertyValue) === "function") {
				// Function
				newContainer[propertyName] = propertyValue;
			} else {
				newContainer[propertyName] = propertyValue;
			}			
		}
		
		return newContainer;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array oldArray - PREREQ: Scalar indexed entries
 	 * 
 	 * @return Array
 	 */	
	_cloneArray: function(oldArray) {
		var newArray = new Array();
		
		for (var index = 0; index < oldArray.length; index++) {
			newArray[index] = oldArray[index];
		}				
		
		return newArray;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Array parameterColours : [Number] = String
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * @param Number maxItems
 	 * 
 	 * @return Object 
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount 								- MAX ITEMS
 	 * 					. Array parameterIcons : [Number] = String				- LENGTH = MAX ITEMS
 	 * 					. Array parameterInternalValues : [Number] = String		- LENGTH = MAX ITEMS
 	 * 					. Array parameterNames : [Number] = String				- LENGTH = MAX ITEMS
 	 * 					. Array parameterValues : [Number] = Number				- LENGTH = MAX ITEMS
 	 * 					. Array parameterColours : [Number] = String			- LENGTH = MAX ITEMS (IF FIXED)
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 */
	_regroupDataToFitViewletArea: function(chartParameters, maxItems) {
		// PREREQ:  Parameter names, values and colours are already sorted
		// in descending order by parameter value.

		// Determine if a regrouping is necessary
		if (maxItems <= 0) {
			// Bad input
			return chartParameters;
		} else if (chartParameters.parameterCount <= maxItems) {
			// Already fits.  No regrouping necessary
			return chartParameters;
		}

		// Container for regrouped parameters
		var result = this._cloneObject(chartParameters);

		// Get the start index to regroup items into the Other category
		// 0 <= replaceStartIndex <= (parameterCount - 1)
		var replaceStartIndex = maxItems - 1;
		// Store the index to perform a drill-through on the Other category
		// if requested
		this._setExcessParameterStartIndex(replaceStartIndex);
		// Tally count for the Other category
		var otherCategoryTally = 0;
		for (var i = replaceStartIndex; i < chartParameters.parameterCount; i++) {
			otherCategoryTally = otherCategoryTally + 
								 chartParameters.parameterValues[i];
		}
		
		// Parameter Names
		// Extract the parameter names that fit in the viewlet area
		var newParameterNames = chartParameters.parameterNames.slice(0, maxItems);
		// Replace the last parameter name with the Other category name
		newParameterNames[replaceStartIndex] = this._static._PARAMETER_OTHER;
		result.parameterNames = newParameterNames;
		
		// Parameter Values
		// Extract the parameter values that fit in the viewlet area
		var newParameterValues = chartParameters.parameterValues.slice(0, maxItems);
		// Replace the last parameter value with the Other category tally
		newParameterValues[replaceStartIndex] = otherCategoryTally;
		result.parameterValues = newParameterValues;
		
		// Parameter Colours
		if (chartParameters.paletteType == this._static._PALETTE_TYPE_FIXED) {
			// Extract the parameter colours that fit the viewlet area
			var newParameterColours = chartParameters.parameterColours.slice(0, maxItems);
			// Replace the last parameter colour with the Other category colour
			newParameterColours[replaceStartIndex] = this._static._RGB_COLOUR_GREY;			
			result.parameterColours = newParameterColours;
		}
		
		// Parameter Icons
		if (chartParameters.parameterIcons.length >= maxItems) {
			// Extract the parameter icons that fit in the viewlet area
			var newParameterIcons = chartParameters.parameterIcons.slice(0, maxItems);
			// Replace the last parameter value with the Other category tally
			newParameterIcons[replaceStartIndex] = this._static._IMAGE_URI + this._static._IMAGE_OTHER;
			result.parameterIcons = newParameterIcons;
		}

		// Parameter Internal Values
		// Extract the parameter names that fit in the viewlet area
		var newParameterInternal = chartParameters.parameterInternalValues.slice(0, maxItems);
		// Replace the last parameter name with the Other category name
		newParameterInternal[replaceStartIndex] = this._static._PARAMETER_OTHER;
		result.parameterInternalValues = newParameterInternal;
	
		// Parameter Count	
		result.parameterCount = maxItems;

		// Debug
		// console.log("=== BEFORE REGROUP ===");
		// this._dumpObject(chartParameters);
		// console.log("=== AFTER REGROUP ===");
		// this._dumpObject(result);

		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_getWorkItemQuery: function() {
		// Set loading message
		this._displayLoadingMessage(this.messages.msgLoading);
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(queryDTO) {
				self._processQueryDTO(queryDTO);
			},
			// Service failure
			failure: function(errorObject) {
				self._displayErrorMessage(self.messages.msgWorkItemQueryFailure, errorObject);
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetQuery(handler);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_getWorkItemQueryResults: function() {
		// Set loading message
		this._displayLoadingMessage(this.messages.msgLoading);
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(queryResultsDTO) {
				self._processQueryResultsDTO(queryResultsDTO);
			},
			// Service failure
			failure: function(errorObject) {
				self._displayErrorMessage(self.messages.msgWorkItemQueryFailure, errorObject);
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		this._callRESTGetFilteredQueryResults(handler);
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param Object event
 	 * 			. String queryItemID 
 	 * 			. String parameterID
 	 * 			. String parameterName
 	 * 			. Number parameterValue
 	 * 			. boolean doIt
 	 * @return void
 	 */
	handleEventDialogExit: function(event) {
		// Execute only if the OK button was clicked
		if (event.doIt == false) {
			return;
		}
		// Get the filtered work item query results
		this._getWorkItemQueryFilteredResults(
			event.parameterName, 
			event.parameterValue, 
			event.queryItemID, 
			event.parameterID);		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * @return void
 	 */
	_createParameterNameChooserDialog: function(queryItemID, parameterID) {
		// Create the dialog parameters
		var index = this._getExcessParameterStartIndex();
		var chartParameters = this._getCachedChartParameters();
		// Validate chart parameters
		if (chartParameters == null) {
			// This should never happen
			this._alertErrorMessage(this.messages.msgDrillChartParamsUndef, null);
			return;
		}
		var parameters = new Object();
		parameters.parameterNames = chartParameters.parameterNames.slice(index);
		parameters.parameterValues = chartParameters.parameterValues.slice(index);
		parameters.queryItemID = queryItemID;
		parameters.parameterID = parameterID;
		// Create the dialog container
		var div = document.createElement("div");
		// Create the dialog configuration
		var widgetParameters = new Object();
		widgetParameters.duration = 100;
		widgetParameters.style = "position: absolute; top: 200px; left: 250px; width: 250px;";
		widgetParameters.title = this.messages.msgSelectParameterName;		
		// Create the dialog
		var dialog = new ParameterNameChooserDialog(widgetParameters, div); 
		dialog.setParameters(parameters);
		dialog.setListSize(parameters.parameterValues.length);
		dialog.setCallBackDialogExit(new Object({self: this}));
		dialog.show();			
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String label
 	 * @param Number count
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * @return void
 	 */
	_getWorkItemQueryFilteredResults: function(label, count, queryItemID, parameterID) {
		// Handle undefined parameters
		if (label == this._static._PARAMETER_UNDEFINED) {
			alert(this.messages.msgNoDrillInUndefParams);
			return;
		}
		// Handle the Other bucket
		if (label == this._static._PARAMETER_OTHER) {
			// Create the parameter name chooser dialog
			this._createParameterNameChooserDialog(queryItemID, parameterID);
			return;
		}
		// Get the internal filter name
		var internalFilterName = this._getInternalFilterName(parameterID);
		// Ensure we have a name
		if (this._isStringDefined(internalFilterName, false, "internalFilterName") == false) {
			this._alertErrorMessage(
				dojo.string.substitute(this.messages.msgNoFilterNameForParam, [parameterID]), 
				null);
			return;			
		}
		// Get the internal filter value
		var internalFilterValue = this._getInternalFilterValue(parameterID, label);
		// Ensure we have a value
		if (this._isStringDefined(internalFilterValue, false, "internalFilterValue") == false) {
			this._alertErrorMessage(
				dojo.string.substitute(this.messages.msgNoFilterValueForLabel, [label]), 
				null);
			return;			
		}
		// Get the URL of the filtered query
		var url = WorkItemUtil.getRunSavedQueryURI(
					queryItemID, 
					null, 
					null, 
					null,
					null,
					/* isUnique */ true, 
					/* preserve filter */ false, 
					internalFilterName, 
					internalFilterValue);
		// Validate the URL
		if (URLUtility.isValidURL(url) == false) {
			// Inform user of invalid URL
			this._alertErrorMessage(
				dojo.string.substitute(this.messages.msgInvalidURLDetected, [url]), 
				null);
			return;
		}
		// Check to see if we can leave the Dashboard
		if (!this.getSite().navigateAwayEvent(false, true)) {
			// Abort drill-in
			// e.g. User attempted to navigate away from an unsaved Dashboard
			// and clicked "Cancel" to stay on the Dashboard.
			return;
		}

		// Redirect to the work item query results page
		location.href = url;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String chartID
 	 * @param String presentation
 	 * @param Array parameterNames : String
 	 * @param Array parameterValues : Number
 	 * @param Array parameterColours : String
 	 * @param Number parameterCount
 	 * @param Number width
 	 * @param Number height
 	 * @param String viewletID
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * @param Number paletteType
 	 * @return void
 	 */	
	_callRESTGetChart: function(handler, chartID, presentation, parameterNames, 
			parameterValues, parameterColours, parameterCount, width, height, viewletID, 
			queryItemID, parameterID, paletteType) {
		// Create the arguments
		var arguments = {
			chartID: chartID,
			presentation: presentation,
			parameterNames: parameterNames,
			parameterValues: parameterValues,
			parameterColours: parameterColours,
			parameterCount: parameterCount,
			width: width,
			height: height,
			viewletID: viewletID,
			queryItemID: queryItemID,
			parameterID: parameterID,
			paletteType: paletteType,
			transparentBackground: this._useTransparentBackground()
		};
		this._callRESTChartsService(handler, "getChart", arguments);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @return void
 	 */	
	_callRESTGetQuery: function(handler) {
		// Get the query preference
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		// Create the arguments
		var arguments = new Object();
		arguments.includeCount = false;
		// Determine whether we have a query item ID or a query string ID
		if (UUIDUtility.isValidUUID(query)) {
			// Query item ID
			// e.g. _H9EB5esEEdycivs5PH0c-g
			// (from work item query chooser)
			arguments.itemId = query
		} else {
			// Query string ID
			// e.g. com.ibm.team.workitem.queries.predefined.openCreated
			// (from process or extension point templates)
			arguments.id = query;
		}
		// Execute the query
		QueryClient.getQuery2(handler, arguments);
	},	

	/*
 	 * DEAD CODE FOR NOW
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String queryItemID
 	 * @return <none>
 	 */	
	_callRESTGetCompleteQuery: function(handler, queryItemID) {
		QueryClient.getCompleteQuery(handler, queryItemID, false);
	},	

	/*
 	 * DEAD CODE FOR NOW
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param Object queryDTO
 	 * @return <none>
 	 */	
	_callRESTGetQueryResults: function(handler, queryDTO) {
		// TODO: MP: Determine upper limit - is 500 enough
		QueryClient.execute(0, 500, "", queryDTO, new Array(), new Array(), handler);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * 
 	 * @return void
 	 */	
	_callRESTGetFilteredQueryResults: function(handler) {
		var targetParameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		
		// Create the arguments
		var arguments = {
			queryItemId: this._getQueryItemID(),
			attributeId: this._getInternalFilterName(targetParameter),
			includeCount: true
		};
		
		QueryClient.getHistogram(handler, arguments);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * @return <none>
 	 */	
	_callRESTChartsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					this._static._STATISTICS_REST_SERVICE_URI,
					method,
					arguments);
		// Invoke the service
		this.getServiceTracker().register(request, handler);
		TeamServerClient.invokeService(request, handler);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 */	
	_useTransparentBackground: function() {
		if (dojo.isIE == 6) {
			// PNG charts with transparent backgrounds show up as grey on IE6
			// because IE6 does not support PNG transparency.
			return false;
		}
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String imageMapHtml
 	 * @param String chartID
 	 * @param String presentation
 	 * @param Number total
 	 * @return <none>
 	 */		
	_createChart: function(imageMapHtml, chartID, presentation, total) {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("_createChart");
			return;
		}		

		// Register the table resize handler
		this._registerChartResizeHandler();		
		// Generate the chart
		var uri = this._static._STATISTICS_RAW_IMAGE_SERVICE_URI +
					"?" +
					this._static._PREFERENCE_ID_CHARTID +
					"=" +
					chartID;
		// Workaround: Add javascript type to script tag for IE (BIRT BUG)
		// TODO: MP: Better way to do this?
		imageMapHtml = this._customizeJSDeclaration(imageMapHtml);
		// Workaround: Force javascript callback function name generated by BIRT to be
		// unique (BIRT BUG)
		imageMapHtml = this._customizeJSCallbackName(imageMapHtml);
		// BIRT BUG # 230401: categoryData value is incorrect for Legend click trigger that invokes a script
		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=230401
		imageMapHtml = this._customizeJSLegendCategoryData(imageMapHtml);

		// Add the chart to the content area
		this._createImage(uri, imageMapHtml);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_registerChartResizeHandler: function() {
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();
		// Connect a new resize handler
		this._windowResizeHandler = ResizeNotifier.register({context: this, callback: this._updateChartDimensions});
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updateChartDimensions: function() {
		// All units are in pixels
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Calculate the height / width ratio
		var chartHeightScaleRatio;
		if (presentation == this._static._PREFERENCE_VALUE_PIE) {
			chartHeightScaleRatio = this._static._CHART_HEIGHT_SCALE_RATIO_PIE_CHART;
		} else if (presentation == this._static._PREFERENCE_VALUE_BAR) {
			chartHeightScaleRatio = this._static._CHART_HEIGHT_SCALE_RATIO_BAR_CHART;
		}		
		// Calculate the preferred chart width and height
		var dimensions = this._getPreferredChartDimensions(chartHeightScaleRatio);
		// Get the image element
		var chartImage = this._getChartImageElement();
		
		// Abort the resize operation if the new image width matches the 
		// current image width 
		if (dimensions.width == chartImage.clientWidth) {
			return;
		}
		
		// Set new dimensions
		chartImage.style.width = dimensions.width + "px";
		chartImage.style.height = dimensions.height + "px";
		
		// Buffer a chart update request (for IE)
		// FF calls window resize event handler after resize is done
		// IE continuously calls the window resize event handler during the 
		// resize
		this._bufferRequestGetChart();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_bufferRequestGetChart: function() {
		// Clear the pending request and create a new one
		// (In effect postponing the request until the resize is finished)
		if (this._chartRequestTimeoutID != null) {
			clearTimeout(this._chartRequestTimeoutID);
		}
		
		// Get the cached chart parameters from the last REST service call
		var chartParameters = this._getCachedChartParameters();
		// Validate chart parameters
		if (chartParameters == null) {
			// This should never happen
			var errorObject = new Error(this.messages.msgChartParamsUndef);
			this._displayErrorMessage(this.messages.msgChartDimenUpdateFailed, errorObject);			
			return;
		}
		// Create a reference to this object to use in the service object
		var self = this;
		// Adjust the report height after the image loads
		this._chartRequestTimeoutID = window.setTimeout(
			function() {
				// Abort operation if the viewlet is not visible
				if (self._isVisible() == false) {
					self._abortOperationNoVisibility("_bufferRequestGetChart");
					return;
				}
				
				// Disconnect the previous handler (if any)
				self._disconnectWindowResizeHandler();		
				// Display the report using cached parameters
				self._displayReport(chartParameters);
				// Reset the timeout ID
				self._chartRequestTimeoutID = null; 		
 			}
			, this._static._TIMEOUT_GET_CHART);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String imageMapHtml
 	 * @return String
 	 */		
	_customizeJSDeclaration: function(imageMapHtml) {
		var string = new String(imageMapHtml);
		var pattern = /<script>/i;
		// IE Workaround:  http://msdn2.microsoft.com/en-us/library/ms533897.aspx
		// In order to inject script tags using innerhtml, a scoped element
		// (input element) must be inserted before the unscoped script element	
		// and its execution must be deferred on load
		var replacement = "<input type=\"hidden\"/><script type=\"text/javascript\" defer=\"defer\">";		
		var result = string.replace(pattern, replacement);
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String imageMapHtml
 	 * @return String
 	 */		
	_customizeJSCallbackName: function(imageMapHtml) {
		var string = new String(imageMapHtml);
		var id = "v" + this._viewletID + "_";
		var pattern = /userCallBack_onclick/ig;
		var result = string.replace(pattern, id);
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String imageMapHtml
 	 * 
 	 * @return String
 	 */		
	_customizeJSLegendCategoryData: function(imageMapHtml) {
		// Split image map by open start element bracket
		// Note: Start element bracket not included in resulting array
		var elements = imageMapHtml.split(/</);
		var result = "";
		var pattern = /^area.+?'(org.eclipse.birt.chart.model.component.impl.LabelImpl.+?)'.+?title="(.+?)"/i;
		
		// BIRT incorrectly places the legend label class name (using its toString method)
		// instead of the legend label text as the categoryData invocation value
		// for the JavaScript callback function.
		// This effectively breaks legend entry drill-in.
		// Replace all instances of the LabelImpl class name with the legend
		// label text obtained from the generated image map area tooltip.
		for (var index = 0; index < elements.length; index++) {
			var element = elements[index];
			// Match "area" elements that contain the "LabelImpl" class name
			// and have a "title" attribute value
			var matches = element.match(pattern);
			if (this._isArrayDef(matches) &&
				(matches.length >= 3)) {
				// 0 : The whole match
				// 1 : The remembered class match
				var classMatch = matches[1];		
				// 2 : The remembered name match		
				var nameMatch = matches[2];
				// Replace the class with the name
				var normalizedElement = element.replace(classMatch, nameMatch);		
				element = normalizedElement;
			}
			// Reassemble the image map using the normalized elements 
			// (and elements not requiring normalization)
			if (this._isStringDef(element)) {
				result = result + "<" + element;
			} 
		}
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String operationName
 	 * @return void
 	 */	
	_abortOperationNoVisibility: function(operationName) {
		// Clear the loading status (if any)
		this.getSite().setLoading(false);
		// Inform framework refresh is done (if refresh was executed)
		this.getSite().doneRefresh();
		// Debug
		// Log warning to console
		// this._logWarning("The viewlet was not visible. Aborted operation: " + operationName);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMessageSelectPreferences: function() {
		// Container
		var div = document.createElement("div");
		div.id = this._getReportDomID();
		dojo.addClass(div, "status");
		
		// Add action to invite the user to select an initial query.
		this._addActionOpenSettings(
			div, 
			this.messages.msgNoWorkItemQueryDefAction,
			this.messages.msgNoWorkItemQueryDef);
		
		// Add the div to the node
		this._addReportNode(this._contentArea, div);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_displayMessageNoResults: function() {
		// Get the attach point
		var node = this._contentArea;
		// Create the container
		var div = document.createElement("div");
		div.setAttribute("id", this._getReportDomID());
		dojo.addClass(div, "status");
		// Create text node
		var message = document.createTextNode(this.messages.msgNoWorkItemsFound);
		// Assemble content
		div.appendChild(message);
		
		// Add the div to the node
		this._addReportNode(node, div);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMessageNonExistingQuery: function() {
		// Container
		var div = document.createElement("div");
		div.id = this._getReportDomID();
		dojo.addClass(div, "status");
		
		// Add action to invite the user to select a new query.
		this._addActionOpenSettings(
			div, 
			this.messages.msgWorkItemQueryNotExistAction,
			this.messages.msgWorkItemQueryNotExist);
		
		this._addReportNode(this._contentArea, div);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String text
 	 * @param String fallbackText
 	 * 
 	 * @return void
 	 */	
	_addActionOpenSettings: function(parent, text, fallbackText) {

		if (this.getSite().isDashboardEditing()) {
			// The user has permission to edit the Dashboard
			// Add action
			var links = I18nUtil.addMessage(parent, text);
			var link = links[0];
			link.href = "javascript:void(0);";
			this.connect(link, "onclick", this._handleEventPrefLinkClicked);
			
		} else if (this._isStringDef(fallbackText)) {
			// The user does NOT have permission to edit the Dashboard
			// Do NOT add action.  Use fallback text
			
			var fallbackMessage = document.createTextNode(fallbackText);
			parent.appendChild(fallbackMessage);
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_displayMessageNoParameters: function() {
		// Get the parameter preference name
		// TODO: MP: Should use new query DTO attribute label insteading of hardcoding
		var parameterName = this._getPreferenceParameterName();	
		// Get the attach point
		var node = this._contentArea;
		// Create the container
		var div = document.createElement("div");
		div.setAttribute("id", this._getReportDomID());
		dojo.addClass(div, "status");
		// Create text node
		var message = document.createTextNode(
			dojo.string.substitute(this.messages.msgNoWorkItemsFoundGrouping, [parameterName]));
		// Assemble content
		div.appendChild(message);
		
		// Add the div to the node
		this._addReportNode(node, div);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return <none>
 	 */	
	_displayLoadingMessage: function(text) {
		// Determine whether we are loading or not
		var loading;
		if (this._isStringDefined(text, false, null)) {
			loading = true;
			// Get the old report node (if any)
			var oldReportNode = this._reportNode;
			// Show message area only if there is no previous content	
			if (this._isObjectDefined(oldReportNode, false, "oldReportNode") == false) {
				// No previous report
				this._showMessageArea();
			} else if (dojo.hasClass(oldReportNode, "status")) {
				// "Click here to configure this report" message
				this._showMessageArea();
			}
		} else {
			loading = false;
			this.getSite().doneRefresh();
			// Show content area
			this._showContentArea();
		}
		// Update the loading message		
		this._updateLoadingMessage(text, this._messageArea, loading);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Error errorObject
 	 * @return <none>
 	 */	
	_displayErrorMessage: function(text, errorObject) {
		// Clear the loading status
		this.getSite().setLoading(false);
		// Inform framework refresh is done
		this.getSite().doneRefresh();
		// Log error to console
		this._logError(errorObject);
		// Update error message in viewlet
		this._updateErrorMessage(text, errorObject, this._messageArea);
		// Show message area
		this._showMessageArea();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Error errorObject
 	 * @return <none>
 	 */	
	_alertErrorMessage: function(message, errorObject) {
		// Qualify message
		var text = dojo.string.substitute(this.messages.msgError, [message]);
		// Check if error object is defined
		if (errorObject == null) {
			// Display alert
			alert(text);
			return;	
		}
		// Log error to console
		this._logError(errorObject);
		// Get the error message
		var errorMessage = errorObject.message;
		// Add the error message to the alert text
		if ((typeof(errorMessage) == "string") &&
			(errorMessage.length != 0)) {
			text = text + 
					": " +
					errorMessage;
		} 			
		// Display alert
		alert(text);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 */	
	_handleEventPrefLinkClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Load the settings area
		this.getSite().toggleSettings();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return void
 	 */		
	_processQueryResultsDTO: function(queryResultsDTO) {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("_processQueryResultsDTO");
			return;
		}		
		
		// Debug
		// console.debug(queryResultsDTO);
		// this._displayErrorMessage("ABORT", new Error("Test"));
		// return;		
		
		// Validate filtered query results DTO
		try {
			this._validateFilteredQueryResultsDTO(queryResultsDTO);
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgWorkItemDTOParseFailed, errorObject);
			return;
		}
		var total = queryResultsDTO.totalCount;
		// Set the secondary viewlet title
		this._updateSecondaryTitle(total);		
		// Check to see if there are any query results
		if (total === 0) {
			this._displayMessageNoResults();
			return;
		}
		
		// Check to see if any of the results found are grouped by the selected
		// parameter
		if (this._hasParameterResults(queryResultsDTO) === false) {
			this._displayMessageNoParameters();
			return;
		}

		// Generate the chart parameters
		var chartParameters = null;
		try {
			chartParameters = this._generateChartParameters(queryResultsDTO);
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgChartParamGenFailure, errorObject);
			return;
		}
		
		// Debug
		// TODO: MP: Temp
		//this._debugChartParameters(chartParameters);
		
		// Cache the chart parameters for zoom-in and zoom-out operations
		this._setCachedChartParameters(chartParameters);
		// Display the report
		this._displayReport(chartParameters);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String
 	 * 
 	 * @return void
 	 */		
	_displayReport: function(chartParameters) {
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Check preference value
		if (presentation == this._static._PREFERENCE_VALUE_PIE) {
			// Create the chart
			this._getChart(
				presentation, 
				chartParameters, 
				this._static._CHART_HEIGHT_SCALE_RATIO_PIE_CHART);
		} else if (presentation == this._static._PREFERENCE_VALUE_BAR) {
			// Create the chart
			this._getChart(
				presentation, 
				chartParameters,
				this._static._CHART_HEIGHT_SCALE_RATIO_BAR_CHART);
		} else if (presentation == this._static._PREFERENCE_VALUE_TABLE) {
			// Create the table
			this._createTable(chartParameters);
		} else if (presentation == this._static._PREFERENCE_VALUE_LIST) {
			// Create the list
			this._createList(chartParameters);
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnknownPresentationVal, [presentation]));
			this._displayErrorMessage(this.messages.msgUnknownPresentation, errorObject);
			return;
		}			
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return Object
 	 * @throws Error
 	 */	
	_generateChartParameters: function(queryResultsDTO) {
		// TODO: MP: Use new DTO attribute label - No need for this method anymore
		// TODO: MP: Validate DTO attribute object when start using label
		// Get the parameter preference
		var parameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		// Process accordingly to value	
		if (parameter == this._static._PREFERENCE_VALUE_SEVERITY) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_SEVERITY,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_PRIORITY) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_PRIORITY,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_CATEGORY) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_CATEGORY,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_OWNEDBY) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_OWNER,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_STATE) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_STATE,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_TYPE) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_TYPE,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_CREATEDBY) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_CREATOR,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_RESOLUTION) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_RESOLUTION,
				parameter);
		} else if (parameter == this._static._PREFERENCE_VALUE_TAGS) {
			return this._generateChartParametersHelper(
				queryResultsDTO, 
				this._static._PARAMETER_NAME_TAGS,
				parameter);
		} else {
			throw new Error(dojo.string.substitute(this.messages.msgUnknownParameterVal, [parameter]));
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * @return void
 	 */	
	_sortParameters: function(container) {
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Get the target parameter
		var targetParameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);

		// Sorting Note:
		//
		// The following DTO arrays ALWAYS need to be sorted:
		//
		// * Presentation Names: parameterNames
		// * Presentation Values: parameterValues
		// * Drill-in Internal Values:  parameterInternalValues
		// * Presentation Icons:  parameterIcons
		// 
		// The following DTO arrays NEVER need to be sorted:
		//
		// * Presentation Colours:  parameterColours
		//   => They are added after sorting
		
		// Sort according to selected preferences
		if ((targetParameter === this._static._PREFERENCE_VALUE_SEVERITY) ||
			(targetParameter === this._static._PREFERENCE_VALUE_PRIORITY) ||
			(targetParameter === this._static._PREFERENCE_VALUE_STATE) ||
			(targetParameter === this._static._PREFERENCE_VALUE_RESOLUTION)) {
			// Sort the names and values by descending internal value
			// The internal values for these grouping parameters are defined
			// as numeric IDs in the process specification and are organized
			// via desired sorting order
			this._parameterInternalValuesShellSort(container);
		} else if (presentation == this._static._PREFERENCE_VALUE_LIST) {
			// Sort the names and values by ascending name
			this._parameterNamesShellSort(container);			
		} else {
			// Sort the names and values by descending value
			this._parameterValuesShellSort(container);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterInternalValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterInternalValuesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = internalValues.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < internalValues.length; i++) {
				// Descending sort
				for (var j = i; j >= increment && internalValues[j - increment] < internalValues[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterNamesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = names.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < names.length; i++) {
				// Ascending sort
				for (var j = i; j >= increment && names[j - increment] > names[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterIcons
 	 * @return void
 	 */	
	_parameterValuesShellSort: function(container) {
		// Validate data
		if (container.parameterCount <= 0) {
			// Nothing to do
			return;
		}
		// Custom sort required to sort names and values simultaneously
		var names = container.parameterNames;
		var values = container.parameterValues;
		var internalValues = container.parameterInternalValues;
		var icons = container.parameterIcons;
		// Shell sort O(n^2)
		for (var increment = values.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < values.length; i++) {
				// Descending sort
				for (var j = i; j >= increment && values[j - increment] < values[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
					// Internal Values
					var temp3 = internalValues[j];
					internalValues[j] = internalValues[j - increment];
					internalValues[j - increment] = temp3;
					// Icons
					var temp4 = icons[j];
					icons[j] = icons[j - increment];
					icons[j - increment] = temp4;					
				}
			}
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 
 	 * @return void
 	 */	
	_addColoursToChartParameters: function(container) {
		var parameter = container.parameterID;
		var values = container.parameterIcons;

		if (parameter === this._static._PREFERENCE_VALUE_SEVERITY) {
			// Severity
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values,  
											this._static._COLOUR_REGEX_SEVERITY,
											this._static._COLOUR_RGB_SEVERITY);

		} else if (parameter === this._static._PREFERENCE_VALUE_PRIORITY) {
			// Priority
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values, 
											this._static._COLOUR_REGEX_PRIORITY,
											this._static._COLOUR_RGB_PRIORITY);

		} else if (parameter === this._static._PREFERENCE_VALUE_TYPE) {
			// Type
			container.paletteType = this._static._PALETTE_TYPE_FIXED;
			container.parameterColours = this._generateParameterColours(
											values,
											this._static._COLOUR_REGEX_TYPE,
											this._static._COLOUR_RGB_TYPE);

		} else {
			// Rest
			container.paletteType = this._static._PALETTE_TYPE_RANDOM;
			container.parameterColours = new Array();
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Array values : String
 	 * @param Array patterns : String
 	 * @param Array colours : String
 	 * 
 	 * @return Array
 	 */	
	_generateParameterColours: function(values, patterns, colours) {

		var parameterColours = new Array();
			
		for (var index = 0; index < values.length; index++) {
			var value = this._getIconLocalFilename(values[index]);
			var colour = this._getAssociatedColour(value, patterns, colours);
			parameterColours.push(colour);
		}
			
		return parameterColours;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String iconPath
 	 * 
 	 * @return String
 	 */	
	_getIconLocalFilename: function(iconPath) {
		var elements = iconPath.split(/\//);
		if (elements.length === 0) {
			return "";
		}
		var lastIndex = elements.length - 1;
		return elements[lastIndex];
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param Array patterns : String
 	 * @param Array colours : String
 	 * 
 	 * @return String
 	 */	
	_getAssociatedColour: function(value, patterns, colours) {
		
		for (var index = 0; index < patterns.length; index++) {
			var pattern = new RegExp(patterns[index]);
			var found = value.match(pattern);
			if (found) {
				return colours[index];
			}
		}
		
		return this._static._COLOUR_RGB_UNDEFINED;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String colour
 	 * @return boolean
 	 */	
	 // DEAD CODE: Not used at the moment
	_isValidColour: function(colour) {
		// e.g. R128G100B23			
		var pattern = /R\d\d?\d?G\d\d?\d?B\d\d?\d?/;
		return pattern.test(colour);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object container
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterInternalValues
 	 * 				. Array parameterIcons
 	 * 
 	 * @return Object
 	 * 				. Number parameterCount
 	 * 				. Array parameterNames
 	 * 				. Array parameterValues
 	 * 				. Array parameterInternalValues
 	 * 				. Array parameterIcons
 	 */	
	_cloneContainer: function(container) {
		// Container
		var newContainer = new Object();
		// Parameter count
		newContainer.parameterCount = container.parameterCount;
		// Parameter names
		newContainer.parameterNames = new Array();
		// Parameter values
		newContainer.parameterValues = new Array();
		// Parameter internal values
		newContainer.parameterInternalValues = new Array();
		// Parameter icons
		newContainer.parameterIcons = new Array();
		
		// Populate
		for (var i = 0; i < container.parameterCount; i++) {
			newContainer.parameterNames[i] = container.parameterNames[i];
			newContainer.parameterValues[i] = container.parameterValues[i];
			newContainer.parameterInternalValues[i] = container.parameterInternalValues[i];
			newContainer.parameterIcons[i] = container.parameterIcons[i];
		}
		
		return newContainer;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterName
 	 * @return String
 	 */	
	_normalizeParameterName: function(parameterName) {
		// BIRT renders the name as a label enclosed in single quotes
		// This breaks the image map click-through for names that include
		// single quotes:  Curtis D'Entremont
		// Replace single quotes
		var pattern = /'/g;
		var result = parameterName.replace(pattern, "`");
		return result;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryDTO
 	 * 
 	 * @return void
 	 */		
	_processQueryDTO: function(queryDTO) {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("_processQueryDTO");
			return;
		}		
		
		// Check if the query exists
		if (this._isObjectDef(queryDTO) === false) {
			this._displayMessageNonExistingQuery();
			return;
		}
		
		// Debug
		// console.debug(queryDTO);
		try {
			this._validateQueryDTO(queryDTO);
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgWorkItemDTOProblem, errorObject);
			return;
		}
		// Initialize query preference provider
		this._initQueryPreferenceProvider(queryDTO);
		// Store the work item query item ID
		this._setQueryItemID(queryDTO.itemId);
		
		// Execute the query
		this._getWorkItemQueryResults();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryDTO
 	 * 					. String itemId
 	 * 					. String name
 	 * @return void
 	 */		
	_initQueryPreferenceProvider: function(queryDTO) {
		// Get the query preference
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		// Check query ID type:  Item or String
		if (UUIDUtility.isValidUUID(query)) {
			// Query item ID
			// Nothing to do
			return;
		}
		// Query string ID
		// Initialize defaults
		this._queryPreferenceProvider.initDefault(queryDTO);
		// Update the primary title
		// Do not persist the title; otherwise, the Dashboard will become
		// dirty
		this._updatePrimaryTitle(queryDTO.itemId, false);			
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_getPreferenceParameterName: function() {
		// TODO: MP: Enhancement:  Use registry
		// Get the parameter preference
		var parameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		// Determine the parameter name
		var parameterName;
		if (parameter == this._static._PREFERENCE_VALUE_SEVERITY) {
			parameterName = this._static._PARAMETER_NAME_SEVERITY;
		} else if (parameter == this._static._PREFERENCE_VALUE_PRIORITY) {
			parameterName = this._static._PARAMETER_NAME_PRIORITY;
		} else if (parameter == this._static._PREFERENCE_VALUE_CATEGORY) {
			parameterName = this._static._PARAMETER_NAME_CATEGORY;
		} else if (parameter == this._static._PREFERENCE_VALUE_OWNEDBY) {
			parameterName = this._static._PARAMETER_NAME_OWNER;
		} else if (parameter == this._static._PREFERENCE_VALUE_STATE) {
			parameterName = this._static._PARAMETER_NAME_STATE;
		} else if (parameter == this._static._PREFERENCE_VALUE_TYPE) {
			parameterName = this._static._PARAMETER_NAME_TYPE;
		} else if (parameter == this._static._PREFERENCE_VALUE_CREATEDBY) {
			parameterName = this._static._PARAMETER_NAME_CREATOR;
		} else if (parameter == this._static._PREFERENCE_VALUE_RESOLUTION) {
			parameterName = this._static._PARAMETER_NAME_RESOLUTION;
		} else if (parameter == this._static._PREFERENCE_VALUE_TAGS) {
			parameterName = this._static._PARAMETER_NAME_TAGS;
		} else {
			parameterName = this.messages.msgUnknown;
		}
		return parameterName;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Number queryTotal
 	 * @return void
 	 */	
	_updateSecondaryTitle: function(queryTotal) {
		// Get the parameter preference name
		var parameterName = this._getPreferenceParameterName();
		// Create the secondary title
		var secondaryTitle = "(" +
					queryTotal +
					") " +
					parameterName;
		// Set the secondary title
		this.getSite().setSecondaryTitle(secondaryTitle);
		// Store the current title for later use
		this._currentViewletTitle = this.getSite().getTitle() + " " + secondaryTitle;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String queryItemID
 	 * @param boolean persist
 	 * @return void
 	 */	
	_updatePrimaryTitle: function(queryItemID, persist) {
		// Get the new query name
		var queryName = this._queryPreferenceProvider.getTextLabel(queryItemID);
		// Set the primary title
		if (this._isStringDefined(queryName, false, "queryName")) {
			this.getSite().setTitle(queryName, persist);
		}	
	},

	/*
 	 * DEAD CODE (at the moment)
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @return <none>
 	 */	
	_adjustTotalFontSize: function(node) {
		var width = this._getClientWidth();
		var preferredFontSize = Math.round(width / this._static._FONT_SIZE_RATIO);
		// Ensure font size does not go below the minimum
		// MIN <= font size <= MAX
		if (preferredFontSize < this._static._MIN_CHART_FONT_SIZE) {
			preferredFontSize = this._static._MIN_CHART_FONT_SIZE;
		} else if (preferredFontSize > this._static._MAX_CHART_FONT_SIZE) {
			preferredFontSize = this._static._MAX_CHART_FONT_SIZE;
		}
		node.style.fontSize = preferredFontSize + "px";
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryDTO
 	 * 
 	 * @return void
 	 * @throws Error
 	 */	
	_validateQueryDTO: function(queryDTO) {
		// Name property
		this._hasProperty(queryDTO, "name", true, "queryDTO");
		this._isStringDefined(queryDTO.name, true, "name");
		// ItemId property
		this._hasProperty(queryDTO, "itemId", true, "queryDTO");
		this._isStringDefined(queryDTO.itemId, true, "itemId");
	},	

	/*
 	 * DEAD CODE FOR NOW
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * @return <none>
 	 */	
	_validateQueryResultsDTO: function(queryResultsDTO) {
		// Validate the query DTO
		this._isObjectDefined(queryResultsDTO, true, "queryResultsDTO");
		// Total result count property
		this._hasProperty(queryResultsDTO, "totalQueryResultCount", true, "queryResultsDTO");
		this._isNumberDefined(queryResultsDTO.totalQueryResultCount, true, "totalQueryResultCount");
		// Work item summary DTOs property
		if (queryResultsDTO.totalQueryResultCount > 0) {
			this._hasProperty(queryResultsDTO, "workItemSummaryDTOs", true, "queryResultsDTO");
			this._isArray(queryResultsDTO.workItemSummaryDTOs, true, "workItemSummaryDTOs");
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return void
 	 */	
	_validateFilteredQueryResultsDTO: function(queryResultsDTO) {
		// DTO
		this._isObjectDefined(queryResultsDTO, true, "queryResultsDTO");
		
		// DTO . Total Count
		this._hasProperty(queryResultsDTO, "totalCount", true, "queryResultsDTO");
		this._isNumberDefined(queryResultsDTO.totalCount, true, "totalCount");		
		
		if (queryResultsDTO.totalCount === 0) {
			// When the Total Count is 0, the Counts property does not exist
			// Abort validation
			return;
		}
		
		// DTO . Counts
		this._hasProperty(queryResultsDTO, "counts", true, "queryResultsDTO");
		this._isArray(queryResultsDTO.counts, true, "counts");
		
		for (var index = 0; index < queryResultsDTO.counts.length; index++) {
			var result = queryResultsDTO.counts[index];
			var displayPath = "queryResultsDTO.counts[" + index + "]";	
			
			// DTO . Counts[x]
			this._isObjectDefined(result, true, displayPath);
			
			// DTO . Counts[x] . Count
			this._hasProperty(result, "count", true, displayPath);
			this._isNumberDefined(result.count, true, displayPath + ".count");		
			
			// DTO . Counts[x] . Value
			this._hasProperty(result, "value", true, displayPath);
			this._isObjectDefined(result.value, true, displayPath + ".value");
			
			// Icon URL not specified for certain grouping parameters
			// DTO . Counts[x] . Value . Icon URL
			//this._hasProperty(result.value, "iconUrl", true, displayPath + ".value");
			//this._isStringDefined(result.value.iconUrl, true, displayPath + ".value.iconUrl");
			
			// DTO . Counts[x] . Value . ID
			this._hasProperty(result.value, "id", true, displayPath + ".value");
			// ID can be an empty string for the tags grouping parameter
			//this._isStringDefined(result.value.id, true, displayPath + ".value.id");
			
			// DTO . Counts[x] . Value . Label
			// Label can be an empty string for the tags grouping parameter
			this._hasProperty(result.value, "label", true, displayPath + ".value");
			//this._isStringDefined(result.value.label, true, displayPath + ".value.label");
		}
		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String queryItemID
 	 * @return <none>
 	 */	
	_setQueryItemID: function(queryItemID) {
		this._currentQueryItemID = queryItemID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getQueryItemID: function() {
		return this._currentQueryItemID;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object chartParameters
 	 * @return <none>
 	 */	
	_setCachedChartParameters: function(chartParameters) {
		this._cachedChartParameters = chartParameters;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_clearCachedChartParameters: function() {
		this._cachedChartParameters = null;
		this._internalValueRegistry = new Array();
		this._iconURLRegistry = new Array();
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param <none>
 	 * @return Object
 	 */	
	_getCachedChartParameters: function() {
		return this._cachedChartParameters;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String label
 	 * @param Number count
 	 * @param String queryItemID
 	 * @param String parameterID
 	 * @return <none>
 	 */	
	_callBackOnClickChartImage: function(label, count, queryItemID, parameterID) {
		// Get the filtered work item query results
		this._getWorkItemQueryFilteredResults(label, count, queryItemID, parameterID);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterID
 	 * @return String
 	 */	
	_getInternalFilterName: function(parameterID) {
		
		if (parameterID == this._static._PREFERENCE_VALUE_SEVERITY) {
			return this._static._INTERNAL_NAME_SEVERITY;
		} else if (parameterID == this._static._PREFERENCE_VALUE_PRIORITY) {
			return this._static._INTERNAL_NAME_PRIORITY;
		} else if (parameterID == this._static._PREFERENCE_VALUE_CATEGORY) {
			return this._static._INTERNAL_NAME_CATEGORY;
		} else if (parameterID == this._static._PREFERENCE_VALUE_OWNEDBY) {
			return this._static._INTERNAL_NAME_OWNER;
		} else if (parameterID == this._static._PREFERENCE_VALUE_STATE) {
			return this._static._INTERNAL_NAME_STATE;
		} else if (parameterID == this._static._PREFERENCE_VALUE_TYPE) {
			return this._static._INTERNAL_NAME_TYPE;
		} else if (parameterID == this._static._PREFERENCE_VALUE_CREATEDBY) {
			return this._static._INTERNAL_NAME_CREATOR;
		} else if (parameterID == this._static._PREFERENCE_VALUE_RESOLUTION) {
			return this._static._INTERNAL_NAME_RESOLUTION;
		} else if (parameterID == this._static._PREFERENCE_VALUE_TAGS) {
			return this._static._INTERNAL_NAME_TAGS;
		} else {
			return null;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String parameterID
 	 * @param String label
 	 * @return String
 	 */	
	_getInternalFilterValue: function(parameterID, label) {
		
		if (parameterID == this._static._PREFERENCE_VALUE_SEVERITY) {
			// SEVERITY_MINOR = "2";
			// SEVERITY_NORMAL = "3";
			// SEVERITY_MAJOR = "4";
			// SEVERITY_CRITICAL = "5";
			// SEVERITY_BLOCKER = "6";
			// SEVERITY_UNCLASSIFIED = "1";
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_PRIORITY) {
			// PRIORITY_UNASSIGNED = "1";
			// PRIORITY_LOW = "2";
			// PRIORITY_MEDIUM = "3";
			// PRIORITY_HIGH = "4";
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_CATEGORY) {
			// Item ID
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_OWNEDBY) {
			// Item ID
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_STATE) {
			// STATE_UNCONFIRMED = "0";		
			// STATE_NEW = "1";
			// STATE_INPROGRESS = "2";
			// STATE_RESOLVED = "3";
			// STATE_VERIFIED = "4";
			// STATE_CLOSED = "5";
			// STATE_REOPENED = "6";
			// STATE_TRIAGED = "7";
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_TYPE) {
			// RFS = "com.ibm.team.workitem.extension.workItemType.rfs";		
			// RETROSPECTIVE = "com.ibm.team.workitem.workItemType.retrospective";
			// DEFECT = "defect";
			// ENHANCEMENT = "enhancement";
			// TASK = "task";
			// STORY = "com.ibm.team.apt.workItemType.story";
			// PLAN_ITEM = "com.ibm.team.workitem.workItemType.planitem";
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_CREATEDBY) {
			// Item ID
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_RESOLUTION) {
			// RESOLUTION_UNRESOLVED = "0";		
			// RESOLUTION_FIXED = "1";		
			// RESOLUTION_DUPLICATE = "2";
			// RESOLUTION_WONTFIX = "3";
			// RESOLUTION_WORKSFORME = "4";
			// RESOLUTION_INVALID = "5";
			// RESOLUTION_REMIND = "6";
			// RESOLUTION_LATER = "7";
			// RESOLUTION_FIXEDUPSTREAM = "8";
			return this._internalValueRegistry[label];
		} else if (parameterID == this._static._PREFERENCE_VALUE_TAGS) {
			return label;
		} else {
			return null;
		}
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_synchronizeDisplayAreasHeight: function() {
		// Operation not applicable when in zoom mode
		if (this.isZoomed()) {
			return;
		}		
		// Get the current content area height
		// Note: Has to be visible to have a value
		var contentAreaHeight = this._contentArea.clientHeight;
		// Synchronize the heights of the message and content areas
		if (contentAreaHeight > this._static._DEFAULT_MIN_HEIGHT) {
			this._setMinHeight(this._messageArea, contentAreaHeight);
			this._setMinHeight(this._contentArea, contentAreaHeight);
		}		
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 */		
	_isVisible: function() {
		if (this._isObjectDefined(this.domNode, false, "domNode") == false) {
			// The DOM nodes is not visible
			return false;			
		} else if (this.getSite()._container.viewletPage.dashboardContainer.getCurrentPage() != 
			this.getSite()._container.viewletPage) {
			// The viewlet's page is not visible
			// Solves duplicate image problem:  Resize browser window and 
			// switch tabs before the refresh is executed
			// The DOM node's ID is not in the DOM when the page is not visible.  
			// As a result, an append is executed instead of a replace.
			return false;
		}
		
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * @param String displayName
 	 * @param String parameterID
 	 * 
 	 * @return Object
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String
 	 * @throws Error
 	 */	
	_generateChartParametersHelper: function(queryResultsDTO, 
			displayName, parameterID) {
				
		// DTO to chart parameters mappings:
		//
		// . Array counts : [Number] = Object   				counts.length => . Number parameterCount
		// 				. Number count							=> . Number parameterValues[index]
		// 				. Object value
		// 						. String iconUrl				=> . String parameterIcons[index]
		// 						. String id						=> . String parameterInternalValues[index]
		// 						. String label					=> . String parameterNames[index]
		// . Number totalCount 									=> . Number total
		
		var parameterValues = new Array();
		var parameterNames = new Array();
		var parameterIcons = new Array();
		var parameterInternalValues = new Array();
		var maxValue = Number.MIN_VALUE;
		var minValue = Number.MAX_VALUE;
		
		var startIndex = 0;
		if (parameterID === this._static._PREFERENCE_VALUE_TAGS) {
			// First result is always work items that have no specified tags
			// Represented by an empty string that breaks drill-in support 
			startIndex = 1;
		}
		
		for (var index = startIndex; index < queryResultsDTO.counts.length; index++) {
			// Parameter Value
			var count = queryResultsDTO.counts[index].count;
			parameterValues.push(count);
			
			// Track max value
			if (count > maxValue) {
				maxValue = count;
			}
			// Track min value
			if (count < minValue) {
				minValue = count;
			}

			// Process value
			var value = queryResultsDTO.counts[index].value;
			// Debug
			//console.debug(value);
			
			// Parameter Name
			var label;
			if (this._isStringDefined(value.label, false, null)) {
				label = this._normalizeParameterName(value.label);
			} else {
				label = this._static._PARAMETER_UNDEFINED;
			}			
			parameterNames.push(label);
			
			// Icon URL
			var iconUrl;
			if (this._hasProperty(value, "iconUrl", false, null) &&
				this._isStringDefined(value.iconUrl, false, null)) {
				iconUrl = value.iconUrl;
			} else {
				iconUrl = this._static._PARAMETER_UNDEFINED;
			}
			parameterIcons.push(iconUrl);
			// Register Icon URL by Parameter Name
			this._iconURLRegistry[label] = iconUrl;
			
			// Parameter Internal Value
			var id;
			if (this._isStringDefined(value.id, false, null)) {
				id = value.id;
			} else {
				id = this._static._PARAMETER_UNDEFINED;
			}
			parameterInternalValues.push(id);
			// Register Parameter Internal Value by Parameter Name
			this._internalValueRegistry[label] = id;

		}
		
		var chartParameters = new Object();
		// Max Value
		chartParameters.maxValue = maxValue;
		// Min Value
		chartParameters.minValue = minValue;
		// Parameter Count
		chartParameters.parameterCount = queryResultsDTO.counts.length - startIndex;
		// Parameter Icons
		chartParameters.parameterIcons = parameterIcons;
		// Parameter Internal Values
		chartParameters.parameterInternalValues = parameterInternalValues;
		// Parameter Names
		chartParameters.parameterNames = parameterNames;
		// Parameter Values		
		chartParameters.parameterValues = parameterValues;
		// Total
		chartParameters.total = queryResultsDTO.totalCount;
		// Parameter ID
		chartParameters.parameterID = parameterID;
		// Display Name
		chartParameters.displayName = displayName; 
		
		// Sort the parameters
		this._sortParameters(chartParameters);
		// Add colours
		this._addColoursToChartParameters(chartParameters);
		
		return chartParameters;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object queryResultsDTO
 	 * 						. Object attribute   					(NOT USED)
 	 * 									. String attributeTypeId	(NOT USED)
 	 * 									. String id					(NOT USED)
 	 * 									. String label				(NOT USED)
 	 * 						. Array counts : [Number] = Object
 	 * 									. Number count
 	 * 									. Object value
 	 * 											. String iconUrl
 	 * 											. String id
 	 * 											. String label
 	 * 						. String queryItemId					(NOT USED)
 	 * 						. Number totalCount
 	 * 
 	 * @return boolean
 	 */		
	_hasParameterResults: function(queryResultsDTO) {
		if ((queryResultsDTO.counts.length === 1) &&
			(this._isStringDefined(queryResultsDTO.counts[0].value.id, false, null) === false)) {
			// Only known scenario:  Tags grouping parameter
			return false;
		}
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @return Object chartParameters
 	 * 					. Number maxValue
 	 * 					. Number minValue
 	 * 					. Number parameterCount
 	 * 					. Array parameterIcons : [Number] = String
 	 * 					. Array parameterInternalValues : [Number] = String
 	 * 					. Array parameterNames : [Number] = String
 	 * 					. Array parameterValues : [Number] = Number
 	 * 					. Number total
 	 * 					. String parameterID
 	 * 					. String displayName
 	 * 					. Number paletteType
 	 * 					. Array parameterColours : [Number] = String

 	 * 
 	 * @return void
 	 */		
	_debugChartParameters: function(chartParameters) {
		console.log("= START ==========================");	
		console.log("Display Name: " + chartParameters.displayName);
		console.log("Parameter ID: " + chartParameters.parameterID);
		console.log("Total: " + chartParameters.total);
		console.log("Parameter Count: " + chartParameters.parameterCount);
		console.log("Max Value: " + chartParameters.maxValue);
		console.log("Min Value: " + chartParameters.minValue);
		console.log("Palette Type: " + chartParameters.paletteType);
		
		for (var index = 0; index < chartParameters.parameterCount; index++) {
			console.log("---------------------");
			console.log("Index: " + index);
			console.log("---------------------");
			console.log("Parameter Name: " + chartParameters.parameterNames[index]);
			console.log("Parameter Value: " + chartParameters.parameterValues[index]);
			console.log("Parameter Internal Value: " + chartParameters.parameterInternalValues[index]);
			console.log("Parameter Icon: " + chartParameters.parameterIcons[index]);
			console.log("Parameter Colour: " + chartParameters.parameterColours[index]);
		}
		
		console.log("= END   ==========================");		
	},

	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 * @overrides super
 	 */		
	hasZoomableContent: function() {
		if (this.getSite().isLoading()) {
			return false;
		}
		return true; 
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		// NO-OP
		return new Array();
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return void
 	 * @overrides super
 	 */		
	settingsChanged: function(oldSettings, newSettings) {
		// Relevant preferences
		var query = this._static._PREFERENCE_ID_QUERY;
		var presentation = this._static._PREFERENCE_ID_PRESENTATION;
		var parameter = this._static._PREFERENCE_ID_PARAMETER;

		var doRefresh = false;
		var doUpdate = false;

		// Query
		if (oldSettings.preferences[query] != newSettings.preferences[query]) {
			// Update the primary title
			this._updatePrimaryTitle(newSettings.preferences[query], true);			
			// Refresh required
			doRefresh = true;
		}
		// Presentation
		if (oldSettings.preferences[presentation] != newSettings.preferences[presentation]) {
			// Update required
			doUpdate = true;
		}
		// Parameter
		if (oldSettings.preferences[parameter] != newSettings.preferences[parameter]) {
			// Refresh required
			doRefresh = true;
		}
		
		if (doRefresh) {
			// Do refresh
			this.getSite().refresh();
		} else if (doUpdate) {
			// Do update
			this._updatePresentation();
		}
		// Do nothing
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_updatePresentation: function() {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("_updatePresentation");
			return;
		}		
		
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();		
		// Get the cached chart parameters from the last REST service call
		var chartParameters = this._getCachedChartParameters();
		// Nothing to update if the viewlet has not been configured yet
		if (chartParameters == null) {
			return;
		}
		// Display the report using cached parameters
		this._displayReport(chartParameters);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_resetCachedObjects: function() {
		// Reset global handlers
		this._viewletMouseOverHandlers = new Object();
		this._viewletMouseOutHandlers = new Object();
		
		// Reset frequencies area node
		this._frequenciesAreaNode = null;
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		// Only one custom preference:  query
		return this._queryPreferenceProvider;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("refresh");
			return;
		}
		
		// Reset cached objects
		this._resetCachedObjects();
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();		
		// Clear the cached chart parameters
		this._clearCachedChartParameters();
		// Synchronize message and content area heights
		this._synchronizeDisplayAreasHeight();
		// Get the current preference values 
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		var query = this.getPreference(this._static._PREFERENCE_ID_QUERY);
		var parameter = this.getPreference(this._static._PREFERENCE_ID_PARAMETER);
		// Verify that the preferences are defined
		try {
			this._isStringDefined(presentation, true, "presentation");
			this._isStringDefined(query, true, "query");
			this._isStringDefined(parameter, true, "parameter");
		} catch(errorObject) {
			this._displayErrorMessage(this.messages.msgMissingPreference, errorObject);
			return;
		}
		// Determine whether a query was specified in a previous session
		if (query == this._static._PREFERENCE_VALUE_UNDEFINED) {
			this._displayMessageSelectPreferences();
		} else {
			this._getWorkItemQuery();
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * Called to update UI (e.g. Zoom)
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides super
 	 */	
	update: function() {
		// Abort operation if the viewlet is not visible
		if (this._isVisible() == false) {
			this._abortOperationNoVisibility("update");
			return;
		}		
		
		// Disconnect the previous handler (if any)
		this._disconnectWindowResizeHandler();		
		// Get the cached chart parameters from the last REST service call
		var chartParameters = this._getCachedChartParameters();
		// Nothing to update if the viewlet has not been configured yet
		if (chartParameters == null) {
			return;
		}
		// Get the presentation preference
		var presentation = this.getPreference(this._static._PREFERENCE_ID_PRESENTATION);
		// Check preference value
		if (presentation == this._static._PREFERENCE_VALUE_TABLE) {
			// There is no need to re-create the table unless there is an
			// explicit refresh as the table will automatically resize itself
			// within the content area (e.g. on zoom-in, zoom-out).
			// However, absolute table label widths needed to be updated to
			// prevent field overlapping 
			this._updateTableLabelElementWidths();			
			return;
		} else if (presentation == this._static._PREFERENCE_VALUE_LIST) {
			// There is no need to re-create the list unless there is an
			// explicit refresh as the list will automatically resize itself
			// within the content area (e.g. on zoom-in, zoom-out) 
			return;
		}
		// Display the report using cached parameters
		this._displayReport(chartParameters);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getTitleHyperlinkURL: function() {
		// Get the current work item query item ID
		var queryItemID = this._getQueryItemID();
		var url;
		
		// Get URL
		if (!queryItemID) {
			// No work item query selected
			// Show Work Items welcome page
			url = WorkItemUtil.getWelcomePageURI();
		} else {
			// Work item query selected
			// Show work item query results
			url = WorkItemUtil.getRunSavedQueryURI(queryItemID);
		}

		return url;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called just before a viewlet zoom operation is invoked
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	preZoom: function() {
		var chartImage = this._getChartImageElement();
		if (this._isObjectDef(chartImage) === false) {
			return;
		}	
		// Scale the image proportionally to the container during zoom
		// (No need to unset the width because the image is replaced)
		chartImage.style.width = "100%";
	},

	// Utility Methods -------------------------------------------------------->

	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * 
 	 * @return boolean
 	 */	
	_isObjectDef: function(variable) {
	    if (typeof(variable) !== "object") {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array variable
 	 * 
 	 * @return boolean
 	 */	
	_isArrayDef: function(variable) {
		if ((variable instanceof Array) === false) {
			return false;
		} else if (variable.length === 0) {
			return false;
		}
		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * 
 	 * @return boolean
 	 */	
	_isStringDef: function(text) {
	    if (typeof(text) != 'string') {
			return false;
		} else if (text.length === 0) {
			return false;
		}		
		return true;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isObjectDefined: function(variable, throwError, variableName) {
		// Check if the variable exists (Any scope)
	    if (typeof(variable) != 'object') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgNotValidObjectType, [variableName]));
			}
			return false;
		}
		// Check if the variable is not null (with casting)
		if (variable == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Number number
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isNumberDefined: function(number, throwError, variableName) {
		// Check if the number is a Number
	    if (typeof(number) != 'number') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotANumber, [variableName]));
			}
			return false;
		}
		
		return true;
	},			
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 * @throws Error
 	 */	
	_isStringDefined: function(text, throwError, variableName) {
		// Check if the text is null (with casting)
		if (text == null) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNull, [variableName]));
			}
			return false;
		}		
		// Check if the text is a String
	    if (typeof(text) != 'string') {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAString, [variableName]));
			}
			return false;
		}
		// Check if the text has content
		if (text.length == 0) {
			if (throwError) {
				throw new Error(dojo.string.substitute(this.messages.msgVariableIsZeroLength, [variableName]));
			}
			return false;
		}		
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Array variable
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_isArray: function(variable, throwError, variableName) {
		// Check to see if the variable is an array
		if (variable instanceof Array) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgVariableIsNotAnArray, [variableName]));
		}
		return false;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param String property
 	 * @param boolean throwError
 	 * @param String variableName
 	 * @return boolean
 	 */	
	_hasProperty: function(variable, property, throwError, variableName) {
		// Check to see if the property is in the variable
		if (property in variable) {
			return true;
		}
		if (throwError) {
			throw new Error(dojo.string.substitute(this.messages.msgPropertyDoesNotExist, [variableName, property]));
		}
		return false;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Error
 	 * @return <none>
 	 */
	_logError: function(errorObject) {
		
		var errorMessage = "[ERROR]: " +
							this._static._CLASSNAME + 
							": " +
							errorObject.message;
		console.error(errorMessage);
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String warningText
 	 * @return void
 	 */
	_logWarning: function(warningText) {
		
		var warningMessage = "[WARNING]: " +
							this._static._CLASSNAME + 
							": " +
							warningText;
		console.warn(warningMessage);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String message
 	 * @param Error errorObject
 	 * @param Element node
 	 * @return <none>
 	 */	
	_updateErrorMessage: function(message, errorObject, node) {
		var text = dojo.string.substitute(this.messages.msgError, [message]);
		var errorMessage = errorObject.message;
		// Add the error message
		if ((typeof(errorMessage) == "string") &&
			(errorMessage.length != 0)) {
			text = text + 
					": " +
					errorMessage;
		} 
		// Update the message
		this._updateMessage(text, node, "#FF0000");	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Element node
 	 * @param boolean loading
 	 * @return <none>
 	 */	
	_updateLoadingMessage: function(text, node, loading) {
		// Set the loading viewlet indicator
		this.getSite().setLoading(loading);
		// Update the message
		this._updateMessage(text, node, "#BBBBBB");	
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param Element node
 	 * @param String color
 	 * 
 	 * @return void
 	 */	
	_updateMessage: function(text, node, color) {
		// Update text colour
		node.style.color = color;
		// Get the text node
		var textNode = node.firstChild;
		
		if ((textNode == null) &&
			(this._isStringDefined(text, false, null) == false)) {
			// Nothing to do
		} else if (this._isStringDefined(text, false, null) == false) {
			// Remove the old text node
			node.removeChild(textNode);
		} else if (textNode == null) {
			// Create a new text node
			textNode = document.createTextNode(text);
			node.appendChild(textNode);
		} else {
			// Modify the existing text node
			textNode.replaceData(0, textNode.length, text); 
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Node node
 	 * @return void
 	 */	
	_removeAllChildren: function(node) {
		// Remove all child nodes if any are defined
		while (node.firstChild != null) {
			node.removeChild(node.firstChild);
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return <none>
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			// Show
			if (node.style.display != "block") {
				node.style.display = "block";	
			}
		} else {
			// Hide
			if (node.style.display != "none") {	
				node.style.display = "none";	
			}
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 * @overrides super
 	 */		
	destroy: function(){
		this._disconnectWindowResizeHandler();
		this.inherited(arguments);
	},
	
	// Class Methods ---------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}

});


})();

