/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportWorkItemsV2Viewlet");

/*
 * Imports 
 */
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");

(function() {
	
/*
 * Aliases 
 */
var TrendReportViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet;
var ReportsPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * TODOs
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportWorkItemsV2Viewlet",
	TrendReportViewlet, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {

		this.inherited(arguments);

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		// Cascading parameters map
		this._groupParametersTeam = null;
		this._groupParametersInterval = null;
		
		// Cascading parameter groups
		this._parameterGroupTeamName = null;
		this._parameterGroupIntervalName = null;

		// Report class
		this._reportClass = this._static._VIEWLET_CLASS_NONE;
		this._setReportClass();
		
		// Preference providers
		this._teamPreferenceProvider = null;
		this._categoryPreferenceProvider = null;
		this._intervalPreferenceProvider = null;
		this._severityPreferenceProvider = null;

	},	

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		// Verify preferences are defined
		try {
			// Team
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_TEAM), true, "team");
			// Category
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_CATEGORY), true, "category");
			// Interval
			this._isStringDefined(this._getPreferenceInterval(), true, "interval");
			
			if (this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) {
				// Severity
				this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_SEVERITY), true, "severity");
			}
		} catch (errorObject) {
			this._displayMsgSelectPreferences();
			this.getSite().doneRefresh();
			return;
		}

		// Get the rendered report
		this._getReport(
			this.getParameterNames(), 
			this.getParameterValues());		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		
		if (preferenceId == this._static._PREFERENCE_ID_TEAM) {
			// Team
			if (this._teamPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/team-dash-icon.gif";
				
				this._teamPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceTeam",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizeTeamAreaPreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedTeam"
					});
			}
			return this._teamPreferenceProvider;		
				
		} else if (preferenceId == this._static._PREFERENCE_ID_CATEGORY) {
			// Category
			if (this._categoryPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-category.gif";

				this._categoryPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceCategory",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizePreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedTeam"
					});
			}
			return this._categoryPreferenceProvider;		

		} else if (preferenceId == this._static._PREFERENCE_ID_INTERVAL) {
			// Interval - Closed Work Items Daily
			if (this._intervalPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/milestone_obj.gif";

				this._intervalPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceInterval",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizeIntervalPreferenceSingleValue",
						parseValueListMethodName: "_parseIntervalPreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedInterval"
					});
			}
			return this._intervalPreferenceProvider;	
		} else if (preferenceId == this._static._PREFERENCE_ID_SEVERITY) {
			// Severity
			if (this._severityPreferenceProvider == null) {
				
				var iconMap = new Array();
				var contextRoot = URLUtility.getContextRoot();
				iconMap[0] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_blocker.gif";
				iconMap[1] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_critical.gif";
				iconMap[2] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_major.gif";
				iconMap[3] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_normal.gif";
				iconMap[4] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_minor.gif";
				iconMap[5] = contextRoot + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/sev_unassign.gif";

				this._severityPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceSeverity",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizePreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getSeverityIconType",
						isParametersLoadedMethodName: "_isParametersLoadedInterval"
					});
			}
			return this._severityPreferenceProvider;		
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedPreference, [preferenceId]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgPrefLoadingFailed, errorObject);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		if (preferenceId == this._static._PREFERENCE_ID_INTERVAL) {
			// Interval - Deferred Work Items
			return this._queryPreferenceInterval(currentSettings);
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedPreference, [preferenceId]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgPrefLoadingFailed, errorObject);
			return this._getPreferenceFailureList();
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		var depedencies = new Array();

		if (preferenceId == this._static._PREFERENCE_ID_TEAM) {
			// Team -> Category
			depedencies.push(this._static._PREFERENCE_ID_CATEGORY);
		} else if ((this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) &&
					(preferenceId == this._static._PREFERENCE_ID_INTERVAL)) {
			// Interval -> Severity
			depedencies.push(this._static._PREFERENCE_ID_SEVERITY);
		}

		return depedencies;
	},

	// Private Methods -------------------------------------------------------->
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportWorkItemsV2Viewlet";
		
		// Preferences
		this._static._PREFERENCE_ID_TEAM = "team";
		this._static._PREFERENCE_ID_CATEGORY = "category";
		this._static._PREFERENCE_ID_INTERVAL = "interval";
		this._static._PREFERENCE_ID_SEVERITY = "severity";	
		
		// Viewlet classes
		this._static._VIEWLET_CLASS_NONE = 0;
		this._static._VIEWLET_CLASS_CLOSED_DAILY = 1;
		this._static._VIEWLET_CLASS_DEFERRED = 2;
		this._static._VIEWLET_CLASS_OPEN_VS_CLOSED = 3;
		this._static._VIEWLET_CLASS_OPEN_BY_TYPE = 4;
		this._static._VIEWLET_CLASS_OPEN_BY_PRIORITY = 5;
		this._static._VIEWLET_CLASS_BLOCKING = 6;
		this._static._VIEWLET_CLASS_NEW_BY_SEVERITY = 7;
		this._static._VIEWLET_CLASS_CLOSED_BY_PRIORITY = 8;
		
		// Cascading Parameter Group Names
		this._static._PARAMETER_GROUP_NAME_0 = "CascadingParameterGroup";
		this._static._PARAMETER_GROUP_NAME_1 = "CascadingParameterGroup1";
		this._static._PARAMETER_GROUP_NAME_2 = "CascadingParameterGroup2";
		
		// Legacy current iteration special variable
		// REVISIT: In the future, we should use a migration handler
		this._static._LEGACY_CURRENT_ITERATION = "<<Current>>";
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setReportClass: function() {
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
		
		// Determine report class
		if (this._reportUUID == this._staticCommon._WORK_ITEMS_CLOSED_DAILY) {
			// Closed Work Items Daily
			this._reportClass = this._static._VIEWLET_CLASS_CLOSED_DAILY;
			this._setMessageNoResults(this.messages.msgNoWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_1);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_2);
			
		} else if (this._reportUUID == this._staticCommon._WORK_ITEMS_DEFERRED) {
			// Deferred Work Items
			this._reportClass = this._static._VIEWLET_CLASS_DEFERRED;
			this._setMessageNoResults(this.messages.msgNoDeferredWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_1);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_2);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_BLOCKING) {
			// Blocking Work Items
			this._reportClass = this._static._VIEWLET_CLASS_BLOCKING;
			this._setMessageNoResults(this.messages.msgNoBlockingWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_OPEN_BY_PRIORITY) {
			// Open Work Items By Priority
			this._reportClass = this._static._VIEWLET_CLASS_OPEN_BY_PRIORITY;
			this._setMessageNoResults(this.messages.msgNoOpenWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_CLOSED_BY_PRIORITY) {
			// Closed Work Items By Priority
			this._reportClass = this._static._VIEWLET_CLASS_CLOSED_BY_PRIORITY;
			this._setMessageNoResults(this.messages.msgNoClosedWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_NEW_BY_SEVERITY) {
			// New Work Items By Severity
			this._reportClass = this._static._VIEWLET_CLASS_NEW_BY_SEVERITY;
			this._setMessageNoResults(this.messages.msgNoWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_OPEN_BY_TYPE) {
			// Open Work Items By Type
			this._reportClass = this._static._VIEWLET_CLASS_OPEN_BY_TYPE;
			this._setMessageNoResults(this.messages.msgNoOpenWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else if (this._reportUUID === this._staticCommon._WORK_ITEMS_OPEN_VS_CLOSED) {
			// Open vs. Closed Work Items
			this._reportClass = this._static._VIEWLET_CLASS_OPEN_VS_CLOSED;
			this._setMessageNoResults(this.messages.msgNoWorkItemsFound);
			this._setParameterGroupTeamName(this._static._PARAMETER_GROUP_NAME_0);
			this._setParameterGroupIntervalName(this._static._PARAMETER_GROUP_NAME_1);
			
		} else {
			// BUG: Error message gets overidden in viewlet content area
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedReport, [this._reportUUID]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgReportSettingFailed, errorObject);
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getPreferenceInterval: function() {
		var interval = this.getPreference(this._static._PREFERENCE_ID_INTERVAL);
		// Adapt legacy current interval special variable to new one
		// REVISIT: In the future, we should use a migration handler
		var regex = new RegExp(this._static._LEGACY_CURRENT_ITERATION);
		interval = interval.replace(regex, this._staticCommon._SPECIAL_CURRENT_ITERATION);
	
		return interval;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * 
 	 * @return Array : String
 	 */	
	_parseIntervalPreferenceMultiValue: function(value) {
		var parsedValues = this._parsePreferenceMultiValue(value);
		// Adapt legacy current interval special variable to new one
		// REVISIT: In the future, we should use a migration handler
		for (var index = 0; index < parsedValues.length; index++) {
			if (parsedValues[index] === this._static._LEGACY_CURRENT_ITERATION) {
				parsedValues[index] = this._staticCommon._SPECIAL_CURRENT_ITERATION;
				break;
			}
		}		
		return parsedValues;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * 
 	 * @return void
 	 */		
	_setParameterGroupTeamName: function(name) {
		this._parameterGroupTeamName = name;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * 
 	 * @return void
 	 */		
	_setParameterGroupIntervalName: function(name) {
		this._parameterGroupIntervalName = name;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param String label
 	 * @return Number
 	 */		
	_getSeverityIconType: function(value, label) {
		if (value == "Blocker") {
			return 0;
		} else if (value == "Critical") {
			return 1;
		} else if (value == "Major") {
			return 2;
		} else if (value == "Normal") {
			return 3;
		} else if (value == "Minor") {
			return 4;
		}
		return 5;
	},	

	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceTeam: function(currentSettings) {
		// Team Area
		if (this._groupParametersTeam == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersTeam == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionTeam();
	},
	
	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceCategory: function(currentSettings) {
		// Category
		if (this._groupParametersTeam == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersTeam == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionCategory(currentSettings);
	},

	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceInterval: function(currentSettings) {
		// Interval
		if (this._groupParametersInterval == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersInterval == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		
		return this._getPreferenceOptionInterval(currentSettings);
	},

	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceSeverity: function(currentSettings) {
		// Severity
		if (this._groupParametersInterval == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersInterval == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		
		return this._getPreferenceOptionSeverity(currentSettings);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoadedTeam: function() {
		return this._isParametersLoaded(this._groupParametersTeam);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoadedInterval: function() {
		return this._isParametersLoaded(this._groupParametersInterval);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_resetCascadingParameters: function() {
		this._groupParametersTeam = null;
		this._groupParametersInterval = null;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_getParameters: function() {
		// Update loading message
		this._updateLoadingMessage(this.messages.msgLoading);
		// Show the loading message
		this._showLoadingArea();
		// Asynchronous call
		this._getGroupParameters(
			this._parameterGroupTeamName,
			this._callBackSuccessGetTeamParameters,
			this._callBackErrorGetTeamParameters);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetTeamParameters: function(self, groupName, dto) {
		// Create a map out of the DTO
		var map = self._processDTOGroupParameters(dto);
		// Store global
		self._groupParametersTeam = map;
		// Debug
		// console.debug(dto); 
		// self._debugCascadingParameters(map, "", 0);
		
		// Chain parameter call
		// Asynchronous call
		self._getGroupParameters(
			self._parameterGroupIntervalName,
			self._callBackSuccessGetIntervalParameters,
			self._callBackErrorGetIntervalParameters);
		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @return void
 	 */
	_callBackErrorGetTeamParameters: function(self, groupName) {
		// Set error
		self._groupParametersTeam = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetIntervalParameters: function(self, parameterName, dto) {
		// Create a map out of the DTO
		var map = self._processDTOGroupParameters(dto);
		// Store global
		self._groupParametersInterval = map;
		// Debug
		// console.debug(dto);
		// self._debugCascadingParameters(map, "", 0);
		
		// Show the display area
		self._showDisplayArea();
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @return void
 	 */
	_callBackErrorGetIntervalParameters: function(self, parameterName) {
		// Set error
		self._groupParametersInterval = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},			
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 * @overrides super
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {

		if (oldSettings.preferences.team != newSettings.preferences.team) {
			// Team
			return false;
		} else if (oldSettings.preferences.category != newSettings.preferences.category) {
			// Category
			return false;
		} else if (oldSettings.preferences.interval != newSettings.preferences.interval) {
			// Interval
			return false;
		} else if ((this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) &&
					(oldSettings.preferences.severity != newSettings.preferences.severity)) {
			// Severity
			return false;
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @return void
 	 * @overrides super
 	 */	
	_addParametersToNode: function(node) {
		// Create parameter names
		var parameterNames = new Array();
		// Create parameter values
		var parameterValues = new Array();		

		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeMultiValue(team, "_normalizeContentAreaSingleTeam");
		parameterNames.push(this.messages.msgTeamArea);
		parameterValues.push(team);
		
		// Category
		var category = this.getPreference(this._static._PREFERENCE_ID_CATEGORY);		
		category = this._normalizePreferenceMultiValue(category);
		parameterNames.push(this.messages.msgCategory);		
		parameterValues.push(category);

		// Interval
		var interval = this._getPreferenceInterval();		
		interval = this._normalizePreferenceMultiValue(interval);
		parameterNames.push(this.messages.msgIteration);		
		parameterValues.push(interval);
		
		if (this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) {
			// Severity
			var severity = this.getPreference(this._static._PREFERENCE_ID_SEVERITY);		
			severity = this._normalizePreferenceMultiValue(severity);
			parameterNames.push(this.messages.msgSeverity);		
			parameterValues.push(severity);
		}

		this._displayParameters(node, parameterNames, parameterValues);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array : String
 	 */	
	getParameterNames: function() {
		
		if (this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) {
			return ["ProjectAreaName", "TeamAreaName", "Category", "ProjectAreaName1", "Interval", "Severity", this._staticCommon._PARAMETER_NAME_ZOOM];
		}
		return ["ProjectAreaName", "TeamAreaName", "Category", "ProjectAreaName1", "Interval", this._staticCommon._PARAMETER_NAME_ZOOM];
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */	
	getParameterValues: function() {
		// Example
		// https://jazz.net/jazz/secure/service/com.ibm.team.reports.common.internal.service.IReportRestService/renderedReport?
		// reportUUID=_G_UEsOS4EdyzKOUwaBSWPw
		// embeddable=true
		// parameterName=ProjectAreaName
		// parameterName=VisibleTeamAreaName
		// parameterName=Category
		// parameterName=ProjectAreaName1
		// parameterName=Interval
		// parameterName=Severity
		// parameterValue='%7BCurrent%20Project%20Area%7D'
		// parameterValue='%2FJazz%20Maintenance%2FRepository'%2C'%2FJazz%20Maintenance%2FWork%20Item'
		// parameterValue='Repository'%2C'Work%20Items'
		// parameterValue='%7BCurrent%20Project%20Area%7D'
		// parameterValue='0.6%20M5'%2C'0.6%20M5D1'%2C'%7BCurrent%20Iteration%7D'
		// parameterValue='Blocker'%2C'Critical'%2C'Major'
		// webui=true
		// showArchived=false
		
		// Example
		// Multiple parameter value:  '/Jazz/Build','/Jazz/Dashboard'
		// &parameterValue='%2FJazz%2FBuild'%2C'%2FJazz%2FDashboard'
		
		// Note:  No need to encode the parameter values; since, this will be
		// done when making the REST service call
		
		var parameterValues = new Array();
		
		// Project
		parameterValues.push(this._quote(this._staticCommon._SPECIAL_PROJECT_AREA));
		
		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeParameterMultiValue(team);
		parameterValues.push(team);
		
		// Category
		var category = this.getPreference(this._static._PREFERENCE_ID_CATEGORY);
		category = this._normalizeParameterMultiValue(category);
		parameterValues.push(category);

		// Project 2
		parameterValues.push(this._quote(this._staticCommon._SPECIAL_PROJECT_AREA));
		
		// Interval
		var interval = this._getPreferenceInterval();
		interval = this._normalizeParameterMultiValue(interval);
		parameterValues.push(interval);
		
		if (this._reportClass == this._static._VIEWLET_CLASS_CLOSED_DAILY) {
			// Severity
			var severity = this.getPreference(this._static._PREFERENCE_ID_SEVERITY);
			severity = this._normalizeParameterMultiValue(severity);
			parameterValues.push(severity);
		}
		
		// Zoom
		parameterValues.push(this._quote(this.isZoomed()));
	
		return parameterValues;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionTeam: function() {
		// Get parameters
		var projectAreas = this._groupParametersTeam;
		var teamArea = null;
		try {
			// Get team area
			teamArea = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA];
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(teamArea, "_normalizeTeamAreaPreferenceSingleValue");
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionCategory: function(currentSettings) {
		// Get parameters
		var projectAreas = this._groupParametersTeam;
		var categories;
		try {
			// Get the current preference value for the team area
			// Cascades: Team Area -> Category
			var teamAreaValue = currentSettings.preferences.team;
			
			if (teamAreaValue === this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
				categories = this._getAllParameterChildren(
					projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA]);
			} else {
				categories = new Array();
				var parsedTeamAreaNames = this._parsePreferenceMultiValue(teamAreaValue);
				// Get categories
				for (var index = 0; index < parsedTeamAreaNames.length; index++) {
					var value = parsedTeamAreaNames[index];
					var parameterObject = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA].map[value];
					categories.push(parameterObject);
				}				
			}
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(categories[0], null);
		for (var index = 1; index < categories.length; index++) {
			var nextResult = this._adaptParametersToPreferences(categories[index], null);
			result = this._mergePreferences(result, nextResult);
		}
		
		return result;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionInterval: function(currentSettings) {
		// Get parameters
		var projectAreas = this._groupParametersInterval;
		var interval = null;
		try {
			// Get team area
			interval = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA];
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}

		var result;
		if (this._reportClass == this._static._VIEWLET_CLASS_DEFERRED) {
			// Adapt parameters to preferences
			result = this._adaptParametersToPreferences(interval, "_normalizeIntervalPreferenceSingleValue2");
			// Sort preferences
			result.options.sort(this._sortPreferencesArray);
		} else {
			// Adapt parameters to preferences
			result = this._adaptParametersToPreferences(interval, "_normalizeIntervalPreferenceSingleValue");
		}
		
		return result;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionSeverity: function(currentSettings) {
		// Get parameters
		var projectAreas = this._groupParametersInterval;
		var severities = new Array();
		try {
			// Get the current preference value for the interval
			// Cascades: Interval -> Severity
			var intervalValue = currentSettings.preferences.interval;
			var parsedIntervalNames = this._parsePreferenceMultiValue(intervalValue);
			// Get severities
			for (var index = 0; index < parsedIntervalNames.length; index++) {
				var value = parsedIntervalNames[index];
				var parameterObject = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA].map[value];
				severities.push(parameterObject);
			}
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(severities[0], null);
		var valueMap = this._createPreferenceValueMap(result);
		for (var index = 1; index < severities.length; index++) {
			var nextResult = this._adaptParametersToPreferences(severities[index], null);
			result = this._integratePreferences(valueMap, result, nextResult);
		}
		
		return result;
	},
	
	// Class Methods ---------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}

});


})();

