/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dojo.fx");
dojo.require("dojo.i18n"); 
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ContributorInfoRegistry = com.ibm.team.dashboard.web.client.internal.ContributorInfoRegistry;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var SettingsArea = com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea;
var ResizeNotifier = com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier();
var messages;

/*
 * The viewlet container that houses the viewlet trim, titlebar, etc.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletContainer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletContainer.html"),
	
	/*
	 * Constructs a new viewlet container.
	 */
	constructor: function(data) {
		this._dashboardItemId = data.viewletHelper.getDashboardItem().getId();
		this._viewletHelper = data.viewletHelper;
		this._viewletId = this._viewletHelper.getId();		
		this._viewletDefinition = data.viewletDefinition;
		this.viewletPage = data.viewletPage;
		this._columns = data.columns;
		this._columnIndex = data.columnIndex;
		
		this._toBoolean(this._viewletDefinition,"titleAsHyperlink");
		this._toBoolean(this._viewletDefinition,"zoomable");
		this._toBoolean(this._viewletDefinition,"scopeSensitive");
		this._toBoolean(this._viewletDefinition,"editable");
			
		this.isEditing = this.viewletPage.isEditing;
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		/*
		 * Indirectly call the dojo.require function; if we call it
		 * directly, it confuses the optimizer because we're not
		 * using a string literal.
		 */
		var dojoRequire = dojo.require;
		dojoRequire(this._viewletDefinition.widget);
			
		this.setTrim(this._viewletHelper.getTrim());
		this.showBackground(this._viewletHelper.isShowBackground());

		// create and add the viewlet (the content)
		var viewletConstructor = eval(this._viewletDefinition.widget);
		this._viewlet = new viewletConstructor({
			definition: this._viewletDefinition,
			helper: this._viewletHelper,
			container: this
		});
		this._bodyContainer.appendChild(this._viewlet.domNode);
		if (this._viewletDefinition.titleAsHyperlink) {
			var link = document.createElement("a");
			link.className = "title-link";
			link.appendChild(document.createTextNode(this._viewletHelper.getTitle()));
			this._headerPrimary.appendChild(link);
			this.headerLinkEvent = this.connect(link,"onmouseover",this.getHeaderLink);
		} else {
			this._headerPrimary.appendChild(document.createTextNode(this._viewletHelper.getTitle()));	
		}
		
		this.domNode.id = dojo.dnd.getUniqueId();
		
		//add viewlet icon
		this._addViewletIcon();

		//init expand/collapse
		this._collapsed = ContributorInfoRegistry.isCollapsed(this._dashboardItemId, this._viewletId);
		this._updateCollapsedState();
		var args = {node: this._middleSection, duration: 200};
		this.viewletExpandAnimation = dojo.fx.wipeIn(args);
		this.viewletCollapseAnimation = dojo.fx.wipeOut(args);
		this.connect(this._header,'ondblclick',this.toggleCollapsed);
		this.connect(this._headerButtons,'ondblclick',dojo.stopEvent);
		
		this.connect(this._toggleCollapse, 'onclick', this.toggleCollapsed);
		this.connect(this.viewletCollapseAnimation, "onEnd", this._onCollapseAnimationEnd);
		this.connect(this.viewletExpandAnimation, "onEnd", this._onCollapseAnimationEnd);
		
		this.connect(this._toggleMenu,'onclick',this.openDropDown);
		this.connect(this._zoom,'onclick',this.toggleZoom);
		
		this.truncateTimeout = window.setTimeout(dojo.hitch(this,this.truncate), 1000);
		this._windowResizeEvent = ResizeNotifier.register({context: this, callback: this.queueTruncate});
		
		//Register event handlers to show zoom link if viewlet is zoomable
		if (this._viewletDefinition.zoomable) {
			this.connect(this._middleSection,'onmouseover',this._showZoom);
			this.connect(this._middleSection,'onmouseout',this._hideZoom);
		}
		
		//set display: none dynamically to allow wipe-in/out effect
		this._settingsContainer.style.display = "none";
		
		this.setEditMode(this.isEditing);
		
		//setup auto-refresh
		if (this._viewletDefinition.refreshInterval > 0) {
			this.connect(this._viewlet, "refresh", "onRefresh");
		}
		//allow viewlets to initialize themselves before the first refresh
		this._viewlet.init();
		// yield thread
		this._yieldTimeout = setTimeout(dojo.hitch(this, this.postCreate2), 0);
	},
	postCreate2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			//inform viewlet that it is active
			this._viewlet.setActive(true);
			//trigger initial refresh
			this.refreshViewlet();
		}
	},
	destroy: function() {
		this._isDestroyed = true;
		ResizeNotifier.unregister(this._windowResizeEvent);
		// kill auto-refresh
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		// kill any remaining timeouts
		if (this._yieldTimeout) {
			// cancel yield
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this.truncateTimeout) {
			// cancel deferred title truncation
			clearTimeout(this.truncateTimeout);
			delete this.truncateTimeout;
		}
		if (this._settingsWidget && this._settingsWidget.destroy) {
			this._settingsWidget.destroy();
		}
		if (this._viewlet) {
			this._viewlet.destroy();
		}
		if (this.zoomDiv && this.zoomDiv.parentNode) {
			this.zoomDiv.parentNode.removeChild(this.zoomDiv);
		}
		if (this._zoomWindowResize) {
			ResizeNotifier.unregister(this._zoomWindowResize);
			this._zoomWindowResize = null;	
		}
		if (this._menu) {
			this._menu.destroy();
			delete this._menu;
		}
		this._edit = null;
		this._close = null;
		this.inherited(arguments);
	},
	/*
	 * Called whenever the viewlet's refresh method is called (via
	 * this.connect). Resets the auto-refresh timer.
	 */
	onRefresh: function() {
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		// recall the last refresh time so that we can properly suspend and resume
		// auto-refresh without disrupting the normal schedule
		this._lastRefreshTime = new Date().getTime();
		this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
	},
	/*
	 * Event handler for onresize truncate call.
	 * Use yielding so that UI is not hung in IE6/7 during truncate's while loop.
	 */ 
	queueTruncate: function(event) {
		if (!this.truncateTimeout) {
			this.truncateTimeout = setTimeout(dojo.hitch(this,this.truncate),0);
		}
	},
	_toggleHeader: function(event, kill) {
		if (event)
			event.preventDefault();
			
		if (this._headerVisible || kill) {
			dojo.removeClass(this.domNode,"header");
			dojo.removeClass(this.domNode,this.trimClass+"-header");
			this._headerLink.innerHTML = messages.showHeader;
			this._headerLink.title = messages.showHeader;
			if (kill) {
				this.disconnect(this._headerLinkOnMouseOver);
				this.disconnect(this._headerLinkOnMouseOut);			
			} else {
				this._headerLinkOnMouseOver = this.connect(this.domNode,'onmouseover', this._showHeaderLink);
				this._headerLinkOnMouseOut = this.connect(this.domNode,'onmouseout', this._hideHeaderLink);	
			}
			this._headerVisible = false;
		} else {
			dojo.addClass(this.domNode,"header");
			dojo.addClass(this.domNode,this.trimClass+"-header");
			this._headerLink.innerHTML = messages.hideHeader;
			this._headerLink.title = messages.hideHeader;
			this.disconnect(this._headerLinkOnMouseOver);
			this.disconnect(this._headerLinkOnMouseOut);
			this._headerVisible = true;
		}
		this._headerLink.blur();
	},
	_showHeaderLink: function(event) {
		clearTimeout(this._headerLinkTimeout);
		this._headerLinkTimeout = null;
		this._headerLink.style.display = "block";
		if (this.isZoomed && this._contentWrapper.scrollHeight > this._contentWrapper.clientHeight) this._headerLink.style.right = "20px";
		else this._headerLink.style.right = "";
	},
	_hideHeaderLink: function (event) {
		if (!this._headerLinkTimeout)
			this._headerLinkTimeout = setTimeout(dojo.hitch(this,this._hideHeaderLinkDelayed),20);
	},
	_hideHeaderLinkDelayed: function () {
		this._headerLinkTimeout = null;
		this._headerLink.style.display = "none";
	},
	addHoverClass: function(event, force){
		if (this.trim == 7) return;
		this.hovering = true;
		if (!this.isEditing){
			dojo.addClass(this.domNode,"hover-not-editing");
			dojo.addClass(this.domNode,this.trimClass + "-hover-not-editing");
			return;
		}
		dojo.addClass(this.domNode,"hover");
		dojo.addClass(this.domNode,this.trimClass+"-hover");
	},
	mouseMoveHandler: function(event) {
		var coords = this.trimlessCoords;
		if (!coords) return;
		if ((event && event.pageX > coords.x && event.pageX < coords.x + coords.w && event.pageY > coords.y - 25 && event.pageY < coords.y + 50)) {
			return;
		} else {
			this.disconnect(this.trimlessMouseMove);
			this.trimlessMouseMove = null;
			this.trimlessCoords = null;
			setTimeout(dojo.hitch(this,this.removeHoverHandler),0);
		}
	},
	removeHoverHandler: function(event) {
		if(this._menuCloseOnBody || this._viewlet.isEditing() || this._settingsOpen || this.isZoomed || !this.hovering || (this.trim == 7 && this._collapsed)) return;
		if (this.trim == 7) {
			if (event) {
				var coords = dojo.coords(this.domNode,true);
				if (dojo.isDescendant(event.target, this._header) == true || (event.pageX > coords.x && event.pageX < coords.x + coords.w && event.pageY > coords.y - 20 && event.pageY < coords.y + 30)) {
					this.trimlessCoords = coords;
					if (!this.trimlessMouseMove) {
						this.trimlessMouseMove = this.connect(document.body,"onmousemove",this.mouseMoveHandler);	
					}
					return;
				}	
			}
		}
		this.hovering = false;
		if (dojo.isIE) {
			setTimeout(dojo.hitch(this,this.removeHoverClass),50); // remove flicker in IE
		} else {
			this.removeHoverClass();	
		}
	},
	removeHoverClass: function() {
		if (!this.hovering){
			if (!this.domNode || !this.trimClass) {
				return;
			}
			if (!this.isEditing) {
				dojo.removeClass(this.domNode, "hover-not-editing");
				dojo.removeClass(this.domNode,this.trimClass + "-hover-not-editing");
			} else {
				dojo.removeClass(this.domNode, this.trimClass + "-hover");
				dojo.removeClass(this.domNode, "hover");
			}	
		}
	},
	showBackground: function(showBackground) {
		if (showBackground) {
			dojo.addClass(this._contentWrapper,"showBackground");
		} else {
			dojo.removeClass(this._contentWrapper,"showBackground");
		}
	},
	setTrim: function(trim) {
		if (this.trimClass) {
			dojo.removeClass(this.domNode,this.trimClass);
			if (dojo.hasClass(this.domNode, this.trimClass+"-hover")) {
				dojo.removeClass(this.domNode,this.trimClass+"-hover");	
				var addHovClass = true;
			}
		}
		
		var trimClass;
		if (trim == 1) trimClass = "gray";
		else if (trim == 2) trimClass = "gold";
		else if (trim == 3) trimClass = "green";
		else if (trim == 4) trimClass = "teal";
		else if (trim == 5) trimClass = "purple";
		else if (trim == 6) trimClass = "orange";
		else if (trim == 7) trimClass = "none";
		else trimClass = "blue";
		 
		dojo.addClass(this.domNode,trimClass);
		
		if ((this.trim == 7 || !this.trim) && trim != 7) {
			this.disconnect(this._headerLinkOnMouseOver);
			this.disconnect(this._headerLinkOnMouseOut);
			this.disconnect(this._headerLinkHandle);
			this._addHoverHandle = this.connect(this._header,'onmouseover',this.addHoverClass);
			this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this.removeHoverHandler);
			this._toggleHeader(null, true);
			this._hideHeaderLink();
		} else if (this.trim != 7 && trim == 7) {
			this._headerLinkOnMouseOver = this.connect(this.domNode,'onmouseover', this._showHeaderLink);
			this._headerLinkOnMouseOut = this.connect(this.domNode,'onmouseout', this._hideHeaderLink);
			this._headerLinkHandle = this.connect(this._headerLink,'onclick', this._toggleHeader);
			this.disconnect(this._addHoverHandle);
			this.disconnect(this._removeHoverHandle);
		}
		
		this.trim = trim;
		this.trimClass = trimClass;
		
		if (addHovClass) {
			this.addHoverClass();	
		}
	},
	truncate: function(force) {
		this.truncateTimeout = null;
		if (this._viewletDefinition.titleAsHyperlink) {
			var node = this._headerPrimary.firstChild;
		} else {
			var node = this._headerPrimary;
		}
		// force is used when the title has been changed and the truncate algorithm needs to be run
		// even if the clientWidth is the same
		// force == true is used since this function is called from many places, some of which
		// pass in parameters
		if (force == true || (this._titleIsTruncated && this._header.clientWidth > this._prevWidth)) {
			node.innerHTML = HTMLUtility.escapeHTML(this._viewletHelper.getTitle());
			this._titleIsTruncated = false;
			node.title = "";
		}
		if (this._headerText.clientHeight > this._header.clientHeight + 5) { // 5px buffer for browser em height calculation anomalies
			var title = node.innerHTML;
			while (this._headerText.clientHeight > this._header.clientHeight + 5) { // 5px buffer for browser em height calculation anomalies
				title = title.substring(0,title.length-1);
				while (title.lastIndexOf(" ") == title.length-1)
					title = title.substring(0,title.length-1);
				node.innerHTML = title + "...";
				if (node.innerHTML.length <= 6) break;
			}
			node.title = this._viewletHelper.getTitle();
			this._titleIsTruncated = true;
			this._prevWidth = this._header.clientWidth;
		}
	},
	setEditMode: function(editing) {
		this.isEditing = editing;
		this._viewlet.setDashboardEditing(editing);
		if(editing) {
			// add listeners
			if (!this.editModeListeners) {
				this.editModeListeners = new Array();
			}
			// create buttons
			if (!this._edit && this._viewletDefinition.editable) {
				this._edit = document.createElement("a");
				this._edit.href="#";
				this._edit.title = messages.editContents;
				this._edit.className = "edit-contents";	
				var beforeNode = this._headerButtons.lastChild;
				if (beforeNode.nodeType == 3) beforeNode = beforeNode.previousSibling;
				this._headerButtons.insertBefore(this._edit, beforeNode);
				this.editModeListeners.push(this.connect(this._edit, 'onclick', this.startEditing));
			}
			if (!this._close) {
				this._close = document.createElement("a");
				this._close.href="#";
				this._close.title = messages.close;
				this._close.className = "close";	
				this._headerButtons.appendChild(this._close);
				this.editModeListeners.push(this.connect(this._close, 'onclick', this.close));
			}
		} else {
			while (this.editModeListeners && this.editModeListeners.length >0){
				this.disconnect(this.editModeListeners.pop());
			}
			if (this._edit) {
				this._edit.parentNode.removeChild(this._edit);
				this._edit = null;
			}
			if(this._close) {
				this._close.parentNode.removeChild(this._close);	
				this._edit = null;
			}
			if (this._settingsOpen) {
				this.toggleSettings();
			}
			if (this._viewlet.isEditing()) {
				this.stopEditing();
			}
		}
	},
	/*
	 * Creates a copy of the viewlet, with a new id, and updates the
	 * dashboard state.
	 */
	cloneViewlet: function(event) {
		if (this.isZoomed) this.toggleZoom (false, true);
		
		var jsonMemento = null;
		var memento = this.viewletHelper.getMemento();
		if (memento) {
			jsonMemento = memento.serialize();
		}
		var columnHelper = this._columns[this._columnIndex]._columnHelper;
		var viewlets = columnHelper.getViewlets();
		for (var i=0;i<viewlets.length;i++) {
			if (viewlets[i].getId() == this._viewletId) {
				this.viewletPage.addViewlet((this._viewletHelper.isUserTitle() || !this._viewletDefinition.dynamicTitle ? dojo.string.substitute(messages.copyOf, [this._viewletHelper.getTitle()]) : this._viewletHelper.getTitle()), this._viewletHelper.getDefinitionId(), this._viewletHelper.getIcon(), this._viewletHelper.getPreferences(), this._viewletHelper.getScope(), this._viewletHelper.getScopeItem() ? this._viewletHelper.getScopeItem().itemId : null, this._viewletHelper.getScopeSubteams(), this._viewletHelper.isShowBackground(), this._viewletHelper.getTrim(), jsonMemento, this._columnIndex, i, this._viewletHelper.isUserTitle());
				break;
			}
		}
	},
	/*
	 * Called when the user clicks on the viewlet container's close button.
	 */
	close: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (confirm(messages.closeViewlet)) {
			if (this.isZoomed) this.toggleZoom();
			var args = {node: this.domNode, duration: 250};
			var fadeOut = dojo.fadeOut(args);
			var wipeOut = dojo.fx.wipeOut(args);
			this.connect(wipeOut, "onEnd", this._onCloseAnimationEnd);
			dojo.fx.chain([fadeOut,wipeOut]).play();
		}
	},
	_onCloseAnimationEnd: function() {
		this._columns[this._columnIndex].remove(this.index);
	},
	closeDropDown: function(event) {
		this.disconnect(this._menuCloseOnBody);
		this.disconnect(this._menuCloseOnViewlet);
		this._menuCloseOnBody = null;
		this._menuCloseOnViewlet = null;
		dojo.removeClass(this._toggleMenu,Constants.ViewletContainer.OPEN_MENU_CLASS);
		dijit.popup.close(this._menu);
		this.removeHoverHandler();
	},
	getHeaderLink: function(event) {
		var url = this._viewlet.getTitleHyperlinkURL();
		if (url) {
			this._headerPrimary.firstChild.href=url;
		}
	},
	isVisible: function() {
		return this.viewletPage.isVisible();
	},
	setTitle: function(/* string */ title) {
		if(this._viewletDefinition.titleAsHyperlink) {
			this._headerPrimary.firstChild.replaceChild(document.createTextNode(title),this._headerPrimary.firstChild.firstChild);
		} else {
			this._headerPrimary.replaceChild(document.createTextNode(title),this._headerPrimary.firstChild);
		}
		this.truncate(true);
	},
	setSecondaryTitle: function(/* string */ title) {
		this._headerSecondary.innerHTML = title;
		this.truncate(true);
	},
	getSecondaryTitle: function() {
		return this._headerSecondary.innerHTML;
	},
	openDropDown: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (this.viewletPage.killPopup){
			this.viewletPage.killPopup();
		}
		//calculate the position of the drop down menu
		var pos = dojo.coords(this._toggleMenu);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		
		if (this.domNode.parentNode == this._columns[this._columns.length-1].node) {
			pos.x += Constants.ViewletContainer.LAST_COL_MENU_OFFSET;
		}
		//Display menu
		this._createDropDown();
		dijit.popup.open({ popup: this._menu, parent: this, x: pos.x, y: pos.y + Constants.ViewletContainer.LAST_COL_MENU_OFFSET, onExecute: dojo.hitch(this,this.closeDropDown),orient: 'TL' });
		
		//register event handlers that cause menu to close
		this._menuCloseOnBody = this.connect(this,"_onBlur",this.closeDropDown);
		this._menuCloseOnViewlet = this.connect(this._viewletWrapper,"onmouseup",this.closeDropDown);
		
		//style menu icon to indicate open state
		dojo.addClass(this._toggleMenu,Constants.ViewletContainer.OPEN_MENU_CLASS);
		this.viewletPage.killPopup = dojo.hitch(this,this.closeDropDown);
		this.addHoverClass(null, true);
	},
	setAutoRefresh: function(/* boolean */ value) {
		if (this._viewletDefinition.refreshInterval > 0) {
			if (value && !this._refreshTimeout) {
				var timeSinceLastRefresh = new Date().getTime() - this._lastRefreshTime;
				var timeUntilNextRefresh = (this._viewletDefinition.refreshInterval * 1000) - timeSinceLastRefresh;
				// always wait at least 5 sec before an auto-refresh to avoid disruptions
				if (timeUntilNextRefresh < 5000) {
					timeUntilNextRefresh = 5000;
				}
				// resume auto-refresh
				this._refreshTimeout = setTimeout(dojo.hitch(this._viewlet, "refresh"), timeUntilNextRefresh);
			}
			else if (!value && this._refreshTimeout) {
				// suspend auto-refresh
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					delete this._refreshTimeout;
				}
			}
		}
	},
	startEditing: function(event) {
		if (event) dojo.stopEvent(event);
		if (!this._viewlet.isEditing()) {
			var height = this._contentWrapper.clientHeight;
			this.setAutoRefresh(false);
			if (this._collapsed) {
				this.toggleCollapsed();
			}
			if(!this.editingButtons) this.editingButtons = [];

			if(!this.isZoomed)
				this._contentWrapper.style.height = this._contentWrapper.clientHeight + "px";
			
			this._editButtonArea.style.display = "block";
			this.editingButtons.push(this.connect(this._saveEdits,"onclick",this.stopEditing));
			this.editingButtons.push(this.connect(this._cancelEdits,"onclick",this.stopEditing));
			
			dojo.addClass(this.domNode, "com-ibm-team-dashboard-viewlet-editing");
			
			this._viewlet.getViewletState().startEditing();
			this._viewlet.setEditing(true);
			if (!this.isZoomed) {
				this._contentWrapper.style.height = height + "px";
				var anim = dojo.animateProperty({node: this._contentWrapper, duration: 250, 
					properties: 
						{height: {start: height, end: this._bodyContainer.clientHeight + this._editButtonArea.clientHeight + this._settingsContainer.clientHeight, unit: "px"}
					}});	
				anim.play();
				this.connect(anim,"onEnd",function() {
					this._contentWrapper.style.height = "";
					dojo.publish("com.ibm.team.dashboard.web.onresize");
				});	
			}
		}
	},
	stopEditing: function(e) {
		var height = this._contentWrapper.clientHeight;
		//Collapsing viewlet while in edit mode calls this method
		var saving = false;
		//if the event handler exists, then a button was explicitly pressed
		if (!e) {
			if(!confirm(dojo.string.substitute(messages.stopEditingConfirm, [this._viewletHelper.getTitle()]))){
				return false;
			}
		} else {
			var saving = e.target == this._saveEdits;
		}
		if (saving) {
			// "Save" button was clicked
			// Validate edited content
			var validContent = this._viewlet.validateEditedContent();
			// Check if the content is valid
			if (validContent == false) {
				// Content is not valid.  Stay in edit mode
				return;
			}
			// Content is valid.  Leave edit mode
			// Notify viewlet that edited contents is about to be saved
			this._viewlet.preSaveState();
		}
		this._editButtonArea.style.display = "none";
		dojo.removeClass(this.domNode, "com-ibm-team-dashboard-viewlet-editing");
		this._viewlet.getViewletState().stopEditing(saving);
		this._viewlet.setEditing(false);
		while(this.editingButtons.length > 0) {
			this.disconnect(this.editingButtons.pop());
		}
		this.setAutoRefresh(true);
		
		if (!this.isZoomed) {
			this._contentWrapper.style.height = height + "px";
			var anim = dojo.animateProperty({node: this._contentWrapper, duration: 250, 
				properties: 
					{height: {start: height, end: this._bodyContainer.clientHeight + this._settingsContainer.clientHeight, unit: "px"}
				}});	
			anim.play();
			this.connect(anim,"onEnd",function() {
				this._contentWrapper.style.height = "";
				dojo.publish("com.ibm.team.dashboard.web.onresize");
			});	
		}
		if (!e) return true;
	},
	toggleCollapsed: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (this._viewlet.isEditing()) {
			if(!this.stopEditing()) {
				return;
			}
		}
		if (this.isZoomed) {
			this.toggleZoom();
		}
		if (this._settingsOpen) {
			this.toggleSettings();
		}
		if (this._collapsed) {
			dojo.removeClass(this._middleSection,Constants.ViewletContainer.MIDDLE_SECTION_COLLAPSED_CLASS);
			dojo.removeClass(this._header,Constants.ViewletContainer.HEADER_COLLAPSED_CLASS);
			this._middleSection.style.height = "1px";
			this.viewletExpandAnimation.play();	
		} else {
			this.viewletCollapseAnimation.play();
		}		
	},
	_handleAutoRefresh: function() {
		// skip refresh unless viewlet is showing
		if (this.isVisible()) {
			this.refreshViewlet();
		}
		else {
			// try again next interval
			this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
		}
	},
	refreshViewlet: function(/* boolean */ force) {
		if (!force && this.isRefreshing)
			return;
		if (!force)
			this.isRefreshing = 1;
		else
			this.isRefreshing++;
		this._viewlet.refresh();
	},
	_onCollapseAnimationEnd: function(){
		this._collapsed = !this._collapsed;
		ContributorInfoRegistry.setCollapsed(this._dashboardItemId, this._viewletId, this._collapsed);
		this._updateCollapsedState();
		this._middleSection.style.overflow = "";
		dojo.publish("com.ibm.team.dashboard.web.onresize");
	},
	_updateCollapsedState: function(){
		if (this._collapsed) {
			dojo.addClass(this._middleSection,Constants.ViewletContainer.MIDDLE_SECTION_COLLAPSED_CLASS);
			dojo.addClass(this._header,Constants.ViewletContainer.HEADER_COLLAPSED_CLASS);
			if (this._toggleCollapse) {
				this._toggleCollapse.className = Constants.ViewletContainer.EXPAND_CLASS;
				this._toggleCollapse.title = Constants.ViewletContainer.EXPAND_TITLE;	
			}
			if (this.trim == 7 && !this._headerVisible)
				this._toggleHeader();
		} else {
			if (this._toggleCollapse) {
				this._toggleCollapse.className = Constants.ViewletContainer.COLLAPSE_CLASS;
				this._toggleCollapse.title = Constants.ViewletContainer.COLLAPSE_TITLE;
			}
			if (this.trim == 7 && this._headerVisible)
				this._showHeaderLink();
		}
	},
	toggleSettings: function(event) {
		if(!this.settingsWipeInAnimation || !this.settingsWipeOutAnimation) {
			var args = {node: this._settingsContainer, duration: 300};
			this.settingsWipeInAnimation = dojo.fx.wipeIn(args);
			this.connect(this.settingsWipeInAnimation,"onEnd",this._afterExpandSettings);	
			this.settingsWipeOutAnimation = dojo.fx.wipeOut(args);
			this.connect(this.settingsWipeOutAnimation,"onEnd",this._afterCollapseSettings);	
		}
		this.setAutoRefresh(this._settingsOpen);
		if (this._settingsOpen) {
			this._hideZoom();
			this.settingsWipeOutAnimation.play();
		} else {
			if (this._collapsed) {
				this.toggleCollapsed();
			}
			if (this._settingsContainer.childNodes.length > 0) {
				while (this._settingsContainer.firstChild) {
					this._settingsContainer.removeChild(this._settingsContainer.firstChild);
				}
			}
			this._settingsWidget = new SettingsArea({viewlet: this._viewlet, viewletHelper: this._viewletHelper, viewletDefinition: this._viewletDefinition, self: this, parseOnLoad: true});
			this._settingsContainer.appendChild(this._settingsWidget.domNode);
			this._settingsContainer.style.display = "block";
			this._settingsContainer.style.height = "1px";
			this._settingsWidget.init();
		
			this.settingsWipeInAnimation.play();

			if (this.trim == 7 && !this._headerVisible)
				this._toggleHeader();
			this.addHoverClass();
		}
		this._settingsOpen = !this._settingsOpen;
		// Notify the viewlet that it's settings area was toggled
		this._viewlet.settingsAreaToggled(this._settingsOpen);
	},
	_afterCollapseSettings: function() {
		this._settingsWidget.destroy();
		this._settingsWidget = null;	
	},
	_afterExpandSettings: function() {
		dojo.publish("com.ibm.team.dashboard.web.onresize");
	},
	toggleZoom: function(event, noAnimation) {
		if (event){
			if (this.zoomDiv && this.zoomDiv.style.display == "none") return;
			if (this.isZoomed && dojo.isDescendant(event.target, this.domNode) && event.target != this._zoom) return;
			dojo.stopEvent(event);
		}
		//do not toggle zoom when dashboard is not in focus
		if (!dojo.hasClass(document.body, "com-ibm-team-dashboard-body")) {
			return;
		}
		
		// Notify viewlet that a zoom operation is about to occur
		this._viewlet.preZoom();	

		if (!this.isZoomed) {
			this._zooming = true;
			this._calcZoomVariables();

			dojo.addClass(this.viewletPage.dashboardContainer.domNode,"zoomed-page");
			
			//create a placeholder for viewlet
			this.zoomPlaceholder = document.createElement("div");
			
			var offset = 0;
			if (this.trim == 7) {
				offset = 3;
			} 
			
			if (this.index != undefined && this.index == 0) {
				this.zoomPlaceholder.style.height = this.initialDimensions.h - offset + "px";
			} else {
				this.zoomPlaceholder.style.height = this.initialDimensions.h - offset - dojo.style(this.domNode,"marginTop") + "px";
			}
			this.domNode.parentNode.replaceChild(this.zoomPlaceholder, this.domNode);
			
			//create an absolutely positionned div and append viewlet
			this.zoomDiv = document.createElement("div");
			this.zoomDiv.className = "com-ibm-team-dashboard-zoom-container com-ibm-team-dashboard-content com-ibm-team-dashboard-web-ui-internal-DashboardPage";
			this.zoomDiv.style.position = "absolute";
			this.zoomDiv.appendChild(this.domNode);
			
			this.viewletPage.dashboardContainer.zoomDiv = this.zoomDiv;
			this.viewletPage.dashboardContainer._updateZoomMargins = dojo.hitch(this,this._updateZoomMargins);
			
			//create and position viewlet at same place as it started, except absolutely positioned 
			dojo.marginBox(this.zoomDiv, {l: this.initialDimensions.l, t: this.initialDimensions.t, w: this.initialDimensions.w});
			document.body.appendChild(this.zoomDiv);
			
			//animate position and width
			if (!noAnimation) {
				if (this._columns.length == 1) {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{height: {start: this.initialDimensions.h, end: this.endDimensions.h, unit: "px"},
						top: {end: this.endDimensions.t, unit: "px"}
						}});
				} else {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{width: {start: this.initialDimensions.w, end: this.endDimensions.w, unit: "px"}, 
						height: {start: this.initialDimensions.h, end: this.endDimensions.h, unit: "px"},
						top: {end: this.endDimensions.t, unit: "px"},
						left: {end: this.endDimensions.l, unit: "px"}
						}});	
				}
				//animate height
				var anim2 = dojo.animateProperty({node: this._contentWrapper, duration: 250, properties: {height: {end: this.endDimensions.h - 24, unit: "px"}}});
					
				//run animations
				anim.play();
				anim2.play();	
			} else {
				this.zoomDiv.style.height = this.endDimensions.h + "px";
				this.zoomDiv.style.top = this.endDimensions.t + "px";
				if (this._columns.length != 1) {
					this.zoomDiv.style.width = this.endDimensions.w + "px";
					this.zoomDiv.style.left = this.endDimensions.l + "px";
				}
				this._contentWrapper.style.height = this.endDimensions.h - 24 + "px";
			}
			
			//change viewlet zoom icon
			dojo.removeClass(this._zoom, "zoom-in");
			dojo.addClass(this._zoom, "zoom-out");
			
			this._contentWrapper.style.overflow = "auto"; // Must define this programatically to fix scrollbar bug in IE
			this._zoomWindowResize = ResizeNotifier.register({context: this, callback: this._updateZoomMargins});
			
			this.isZoomed = true;
			
			this.addHoverClass();
			
			if (!noAnimation) {
				this.connect(anim, "onEnd", this._onExpandEnd);	
			} else {
				this._onExpandEnd();
			}
			
			this._showZoom();
		} else {
			this._zooming = true;
			if (!noAnimation) {
				//animate position and width
				if (this._columns.length == 1) {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{height: {end: this.initialDimensions.h, unit: "px"},
						top: {end: this.initialDimensions.t, unit: "px"}
						}});
				} else {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{width: {end: this.initialDimensions.w, unit: "px"}, 
						height: {end: this.initialDimensions.h, unit: "px"},
						top: {end: this.initialDimensions.t, unit: "px"},
						left: {end: this.initialDimensions.l, unit: "px"}
						}});	
				}
				//animate height
				var anim2 = dojo.animateProperty({node: this._contentWrapper, duration: 250, properties: {height: {end: this.initialDimensions.h - 24, unit: "px"}}});
	
				anim.play();
				anim2.play();				
			} else {
				this.zoomDiv.style.height = this.initialDimensions.h + "px";
				this.zoomDiv.style.top = this.initialDimensions.t + "px";
				if (this._columns.length != 1) {
					this.zoomDiv.style.width = this.initialDimensions.w + "px";
					this.zoomDiv.style.left = this.initialDimensions.l + "px";
				}
				this._contentWrapper.style.height = this.initialDimensions.h - 24 + "px";
			}

			//make sure zoom icon isn't adjusted for scroll position
			this._zoom.style.bottom = "";
			
			//kill events
			this.disconnect(this._unZoomEvent);
			this.disconnect(this._onViewletClickEvent);
			this.disconnect(this.onViewletScroll);
			ResizeNotifier.unregister(this._zoomWindowResize);
			this._unZoomEvent = null;
			this._zoomWindowResize = null;
			this.onViewletScroll = null;
			
			this.isZoomed = false;
			dojo.removeClass(this.viewletPage.dashboardContainer.domNode,"zoomed-page");
			
			if (!noAnimation) {
				this.connect(anim, "onEnd", this._onCollapseEnd);
				this.connect(anim2, "onEnd", function(){
					//make sure these get called always
					this._contentWrapper.style.overflow = "hidden"; // Must define this programatically to fix scrollbar bug in IE
					this._contentWrapper.style.height = "";
					this.domNode.style.height = "";
				});	
			} else {
				this._onCollapseEnd();
			}
		}
		//tell viewlet that it should/shouldn't show zoomed content
	},
	_onExpandEnd: function(){
		this._zooming = false;
		// Notify viewlet that a zoom operation has completed
		this._viewlet.postZoom();	
		// Set zoom state and update viewlet content
		this._viewlet.setZoomed(true);
		this._viewlet.getSite().setLoading(false);
		this._unZoomEvent = this.connect(dojo.byId("net-jazz-ajax-WorkbenchRoot"),"onclick",this.toggleZoom);
		dojo.publish("com.ibm.team.dashboard.web.onresize");
		this._updateScrollPosition();
		if (this.trim == 7)
			setTimeout(dojo.hitch(this,this._showHeaderLink),150);
	},
	_onCollapseEnd: function(){
		this._zooming = false;
		// Notify viewlet that a zoom operation has completed
		this._viewlet.postZoom();	
		// Set zoom state and update viewlet content
		this._viewlet.setZoomed(false);	
		this._viewlet.getSite().setLoading(false);
		
		this._contentWrapper.style.overflow = "hidden"; // Must define this programatically to fix scrollbar bug in IE
		
		document.body.removeChild(this.zoomDiv);
		this.zoomPlaceholder.parentNode.replaceChild(this.domNode, this.zoomPlaceholder);
		this.zoomPlaceholder = null;
		this._contentWrapper.style.height = "";
		this.domNode.style.height = "";
	
		this.viewletPage.dashboardContainer.zoomDiv = null;
		this.viewletPage.dashboardContainer._updateZoomMargins = null;
		this.zoomDiv = null;
	
		dojo.removeClass(this._zoom, "zoom-out");
		dojo.addClass(this._zoom, "zoom-in");
	
		dojo.publish("com.ibm.team.dashboard.web.onresize");
		this.removeHoverHandler(); //show hover trim while zoomed
		if (this.trim == 7)
			this._showHeaderLink();
	},
	_addViewletIcon: function() {
		var icon = this._viewletHelper.getIcon();
		if (!icon) {
			icon = this._viewletDefinition.icon;
		}
		var iconImage = document.createElement("img");
		if (icon.indexOf("://") == -1) {
			var index = icon.indexOf("/");
			var segment1 = icon.substring(0, index);
			if (segment1 == "resources") {
				// 60810; support existing 1.0 viewlets until migration
				segment1 = "com.ibm.team.dashboard.viewlets.web";
			}
			var path = icon.substring(index + 1);
			iconImage.src = dojo.moduleUrl(segment1, path);
		}
		else {
			iconImage.src = icon;
		}
		iconImage.className = "icon-image";
		this._headerIcon.appendChild(iconImage);
	},
	addMenuItem: function(/* string */ label, /* string */ iconClass, /* function */ callback) {
		if (!this._contributedMenuItems) {
			this._contributedMenuItems = new Array();
		}
		var entry = {label: label, onClick: callback};
		if (iconClass) {
			entry.iconClass = iconClass;
		}
		this._contributedMenuItems.push(entry);
		// if the menu has already been created, insert it
		if (this._menu) {
			if (this._contributedMenuItems.length == 1) {
				// first contribution; add a separator
				this._menu.addChild(new dijit.MenuSeparator(), this._menuContribIndex);
			}
			this._menu.addChild(new dijit.MenuItem(entry), this._menuContribIndex++);
		}
	},
	_createDropDown: function (){
		if (this._menu) {
			this._menu.destroy();
		}
		// index to add viewlet menu contributions
		this._menuContribIndex = 0;
		var menu = new dijit.Menu();
		if (this.isEditing) {
			menu.addChild(new dijit.MenuItem({label: messages.editSettings, iconClass: 'prefsIcon', onClick: dojo.hitch(this,this.toggleSettings)}));
			this._menuContribIndex++
			if (this._viewletDefinition.editable) {
				menu.addChild(new dijit.MenuItem({label: messages.editContents, iconClass: 'editIcon', onClick: dojo.hitch(this,this.startEditing)}));
				this._menuContribIndex++
			}
			menu.addChild(new dijit.MenuSeparator());
			this._menuContribIndex++
		}
		
		// add known menu contributions
		if (this._contributedMenuItems) {
			for (var i=0;i<this._contributedMenuItems.length;i++) {
				menu.addChild(new dijit.MenuItem(this._contributedMenuItems[i]));
			}
			this._menuContribIndex += this._contributedMenuItems.length;
			menu.addChild(new dijit.MenuSeparator());
		}
		
		// read-only viewlets are still refreshable
		menu.addChild(new dijit.MenuItem({label: messages.refresh, iconClass: 'refreshIcon', onClick: dojo.hitch(this,this.refreshViewlet)}));
		
		if (this.isEditing) {
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({
				label: messages.duplicateViewlet,
				iconClass: 'cloneIcon',
				onClick: dojo.hitch(this, this.cloneViewlet)
			}));
		}
		this._menu = menu;
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
	},
	_hideZoom: function(e) {
		this._zoom.style.display = 'none';
	},
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("ViewletContainer.js remote error: " + errorObj.message);
	},
	_showZoom: function(e) {
		if (this._viewlet.hasZoomableContent())
			this._zoom.style.display = 'block';
		else return;
		
		if (this.isZoomed) {
			if (!(dojo.isIE < 7 && dojo.isIE > 0)) {
				this._zoom.style.bottom = 1 - this._contentWrapper.scrollTop + "px";	
			}
			if (!this.onViewletScroll) {
				this.onViewletScroll = this.connect(this._contentWrapper,"onscroll",this._showZoom);	
			}
		}
	},	
	/*
	 * Workaround for bug 31776: REST service not mapping EBoolean to
	 * Javascript boolean.
	 * @param Object
	 * @param String: Object property
	 */
	_toBoolean: function(obj, val) {
		if (typeof(obj[val]) == "string") {
			obj[val] = (obj[val] == "true");
		}
	},
	_calcZoomVariables: function() {
		//get coordinates for various elements of the page
		if (this.zoomPlaceholder){
			var coords = dojo.coords(this.zoomPlaceholder, true);
			var viewletPageCoords = dojo.coords(this.zoomPlaceholder.parentNode.parentNode, true);
		} else {
			var coords = dojo.coords(this.domNode, true);
			var viewletPageCoords = dojo.coords(this.domNode.parentNode.parentNode, true);
		}
		var workbenchCoords = dojo.coords(dojo.byId("net-jazz-ajax-WorkbenchRoot"),true);
		
		//calculate bounds
		var minTop = viewletPageCoords.y;
		var minLeft = viewletPageCoords.x + 6;

		var marginLeft = 4;
		var marginTop = 8;
		
		//calculate initial dimensions
		if (this._columns.length == 1) {
			var initWidth = coords.w - 6;	 
		} else {
			var initWidth = coords.w;
		}
		if (this.zoomPlaceholder) {
			this.initialDimensions = {
				l: coords.x,
				t: coords.y,
				w: initWidth,
				h: coords.h - marginTop
			}
		} else {
			this.initialDimensions = {
				l: coords.x - marginLeft,
				t: coords.y - marginTop,
				w: initWidth,
				h: coords.h - marginTop
			}
		} 
		//calculate zoom height/width
		try {
			var viewportWidth = document.body.clientWidth;
			var viewportHeight = window.innerHeight;
			if (!viewportHeight) viewportHeight = document.documentElement.clientHeight;	
		} catch(e){
			viewportWidth = 640;
			viewportHeight = 480;
		}
		if (this._columns.length != 1) {
			var endWidth = Math.round(viewportWidth * 0.60);
		} else {
			var endWidth = initWidth;
		}
		var endHeight = Math.round(viewportHeight * 0.70);
		
		//calculate zoom positions
		var endTop = this.initialDimensions.t - (endHeight - this.initialDimensions.h) / 2;
		var endLeft = this.initialDimensions.l - (endWidth - this.initialDimensions.w) / 2;
			
		//if zoom will cause the viewlet to expand outside bounds of page, constrain it to bounds
		if (endTop < minTop) endTop = minTop;
		if (endLeft < minLeft) endLeft = minLeft;
		if (endTop + endHeight > workbenchCoords.h - 30 - marginTop) endTop = workbenchCoords.h - 30 - endHeight;
		if (endLeft + endWidth > minLeft + viewletPageCoords.w) endLeft = minLeft + viewletPageCoords.w - endWidth - 8;
		
		this.endDimensions = {
			l: endLeft,
			t: endTop,
			w: endWidth,
			h: endHeight
		}
	},
	_updateZoomMargins: function() {
		if (this._zooming) {
			setTimeout(dojo.hitch(this, this._updateZoomMargins), 200);
			return;
		}
		if (!this.zoomDiv) return;
		
		this._calcZoomVariables();
		
		this.zoomDiv.style.top = this.endDimensions.t + "px";
		this.zoomDiv.style.height = this.endDimensions.h + "px";
		this._contentWrapper.style.height = this.endDimensions.h + "px";
		if (this._columns.length > 1) {
			this.zoomDiv.style.left = this.endDimensions.l + "px";
			this.zoomDiv.style.width = this.endDimensions.w + "px";
		}
		this._updateScrollPosition();
	},
	_updateScrollPosition: function() {
		var coords = dojo.coords(this.zoomDiv);
		var dy = coords.y + this.endDimensions.h - document.documentElement.clientHeight + 30;
		if (dy > 0) {
			window.scrollBy(0,dy);
		} else if (coords.y < 0) {
			window.scrollBy(0,coords.y-20);
		}
	}
});

})();
