/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletPage");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletHelper");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("dojo.fx");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");


(function() {

//
// Aliases
//
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ViewletChooserBlock = com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock;
var ViewletHelper = com.ibm.team.dashboard.web.client.internal.ViewletHelper;
var ViewletColumn = com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletContainer = com.ibm.team.dashboard.web.ui.internal.ViewletContainer;
var ViewletRegistry = com.ibm.team.dashboard.web.client.internal.ViewletRegistry;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

/**
 * A page of viewlets (contents of one dashboard tab).
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletPage",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletPage.html"),
	
	constructor: function(args) {
		this._viewletPageHelper = args.viewletPageHelper;
		this._viewletPageIndex = args.tabIndex;
		this.dashboardContainer = args.dashboardContainer;
		this.numColumns = this._viewletPageHelper.getColumns().length;
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		
		this.noViewlets = messages.noViewlets;
		this.loadingMessage = messages.loading;
	},
	postCreate: function() {
		this._viewletChooser.style.display = "none";
		this.emptyTabAddViewletLink = I18nUtil.addMessage(this._showInEditMode, dojo.string.substitute(messages.noViewletsAdd, [ messages.addViewletText ]))[0];
		
		// check if there are any viewlets
		var isEmpty = true;
		var cols = this._viewletPageHelper.getColumns();
		for (var i=0;i<cols.length;i++) {
			var col = cols[i];
			if(col.getViewlets().length != 0) {
				// show "Loading..."
				this._loadingText.style.display = "block";
				isEmpty = false;
				break;
			}
		}
		this._setEmptyTabMessage(isEmpty);
		if (this.dashboardContainer.editMode != Constants.READ_ONLY) {
			this.isEditing = true;
		} else {
			this.isEditing = false;
			this._showInEditMode.style.display = "none";
		}
	},
	init: function() {
		this._yieldTimeout = setTimeout(dojo.hitch(this, this.init2), 0);
	},
	init2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			this._createLayout();
			this.connectAddViewletButton();
		}
	},
	isVisible: function() {
		return (this.dashboardContainer.getCurrentPage() == this) && this.dashboardContainer.isVisible();
	},
	connectAddViewletButton: function() {
		this.dashboardContainer.metaNav.connectAddViewletButton(this.expandViewletChooser,this);
		this.connect(this.emptyTabAddViewletLink,"onclick",this.expandViewletChooser);
		if (this._addViewletAreaExpanded) {
			this.dashboardContainer.metaNav.hideAddViewletButton();
		} else {
			this.dashboardContainer.metaNav.showAddViewletButton();
		}
	},
	_createLayout: function() {
		this.columns = new Array();
		this.loading = true;
		
		var content = this._tabArea;
		var columnHelpers = this._viewletPageHelper.getColumns();
		
		this._loadingText.parentNode.removeChild(this._loadingText);
		for (var i=0;i<columnHelpers.length;++i) {
			var columnWidget = this._createColumnWidget(content, {cols: columnHelpers.length, id: "column" + (i+1), columnIndex: i, columnHelper: columnHelpers[i]});
			this.columns.push(columnWidget);
			this._populateColumnViewlets(columnWidget, columnHelpers[i]);
		}
		this.loading = false;
	},
	
	/*
	 *  Create one column
	 */
	_createColumnWidget: function(contentArea, params) {
		var column = document.createElement("div");
		if (params.cols == 1) {
			column.className = "one-col";
		} else if (params.cols == 2) {
			column.className = "two-col";
		} else {
			column.className = "three-col";
		}
		column.id = params.id;
		contentArea.appendChild(column);
		var permissions = this._viewletPageHelper.getDashboardItem().getPermissions();
		return new ViewletColumn(column, {accept: ["viewlet"], columnIndex: params.columnIndex, columnHelper: params.columnHelper, viewletPage: this, disabled: !this.isEditing});
	},
	setEditMode: function(editing) {
		this.isEditing = editing;
		for (var i = 0; i < this.columns.length; i++) {
			this.columns[i].setEditMode(editing);
		}
		if (editing) {
			this._showInEditMode.style.display = "";
			if (this == this.dashboardContainer.currentPage) {
				this.connectAddViewletButton();			
			}
		} else {
			this._showInEditMode.style.display = "none";
			this.closeViewletChooser();
		}
	},
	setActive: function(/* boolean */ active) {
		for (var j = 0; this.columns && j < this.columns.length; j++) {
			var widgets = this.columns[j].widgets;
			for (var k = 0; widgets && k < widgets.length; k++) {
				var widget = widgets[k];
				if (widget && widget._viewlet) {
					widget._viewlet.setActive(active);
					if (widget._settingsOpen && widget._settingsWidget)
						widget._settingsWidget._resize();
				}
			} 	
		}
	},
	_populateColumnViewlets: function(columnWidget, /* ColumnHelper */ columnHelper) {
		// add viewlets
		var columnIndex = columnWidget.index;
		var viewlets = columnHelper.getViewlets();
		for (var i=0;i<viewlets.length;i++) {
			this._addViewlet(viewlets[i], columnIndex, -1);
		}
		this.checkEmpty();
	},
	/*
	 * Adds a new viewlet to the dashboard, assigns it a new ID (fetched
	 * from the server), and updates the server state, asynchronously.
	 */
	addViewlet: function(/* string */ title, /* string */ definitionId, /* string */ icon, /* object */ preferences, /* number */ scope, /* string */ scopeItemId, /* boolean */ scopeSubteams, /* boolean */ showBackground, /* number */ trim, /* string */ jsonMemento, /* number */ destColumnIndex, /* number */ destViewletIndex, /* boolean */ userTitle) {
		// first get a UUID
		var self = this;
		var serviceObject = {
			success: function(uuid) {
				// add the viewlet at top of first column
				var rawHelper = {
					id: uuid,
					title: title,
					definitionId: definitionId,
					icon: icon,
					memento: jsonMemento,
					preferences: Util.toEntryArray(preferences),
					collapsed: false,
					scope: scope,
					scopeItem: {itemId: scopeItemId },
					scopeSubteams: scopeSubteams,
					showBackground: showBackground,
					userTitle: userTitle,
					viewletTrim: trim
				};
				var viewletHelper = new ViewletHelper(rawHelper, self._viewletPageHelper.getDashboardItem());
				self._addViewlet(viewletHelper, destColumnIndex, destViewletIndex);
				self._viewletPageHelper.addViewlet(viewletHelper, destColumnIndex, destViewletIndex);
			},
			failure: function(errorObject) {
				errorObject.message = messages.addViewletRemoteError + errorObject.message;
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObject]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardClient.generateUUID(srh);
	},
	/*
	 * Adds a viewlet widget to the dashboard either at the top or bottom
	 * of a specific column (does not update server state). A viewlet index
	 * of -1 or undefined will place the viewlet at the bottom of the
	 * column.
	 */
	_addViewlet: function(/* ViewletHelper */ viewletHelper, /* number */ columnIndex, /* number */ viewletIndex) {
		var serviceObj = {
			_columnIndex: columnIndex,
			_viewletHelper: viewletHelper,
			_viewletIndex: viewletIndex,
			_viewletPage: this,
			success: function(/* ViewletDefinitionDTO[] */ viewletDefinitionDTOs) {
				// set up params
				var args = {
					viewletHelper: this._viewletHelper,
					viewletDefinition: viewletDefinitionDTOs[0],
					viewletPage: this._viewletPage,
					columns: this._viewletPage.columns,
					columnIndex: this._columnIndex
				};
				// add viewlet to destination column
				var column = this._viewletPage.columns[this._columnIndex];
				var viewletContainer = new ViewletContainer(args);
				
				if (this._viewletIndex != -1 && column.widgets.length >= this._viewletIndex) {
					// insert at given index
					column.add(viewletContainer, {index: this._viewletIndex, handle: viewletContainer._header, type: "viewlet", animate: true});
				} else {
					// append to the end of the column
					column.add(viewletContainer, {handle: viewletContainer._header, type: "viewlet", animate: false});
				}
				this._viewletPage._setEmptyTabMessage(false);
			},
			failure: function(/* Error */ error) {
				error.message = messages.viewletNotFetched + error.message;
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ViewletRegistry.getDefinitions([viewletHelper.getDefinitionId()], srh, this.dashboardContainer.getServiceTracker());
	},

	checkEmpty: function() {
		var isEmpty = true;
		for (var i = 0; i < this.columns.length; i++) {
			if(this.columns[i].widgets.length != 0) {
				isEmpty = false;
				break;
			}
		}
		this._setEmptyTabMessage(isEmpty);
	},
	_setEmptyTabMessage: function(isVisible) {
		if (isVisible) {
			this._emptyTabMessage.style.display = "block";	
		} else {
			this._emptyTabMessage.style.display = "none";
		}	
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._isDestroyed = true;
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		while(this.columns && this.columns.length > 0) {
			this.columns.pop().destroy();
		}
		this.columns = null;
		if (this._viewletChooserBlock) {
			this._viewletChooserBlock.destroy();
		}
		this.inherited(arguments);
	},
	refreshPage: function() {
		this.dashboardContainer._refreshCurrentTab();
	},
	refreshViewlets: function() {
		for (var i=0;i<this.columns.length;i++) {
			var viewlets = this.columns[i].widgets;
			for (var j=0;j<viewlets.length;j++) {
				viewlets[j].refreshViewlet();
			}
		}
	},
	/*
	 * Event handler for close button on Add Viewlet panel
	 */
	closeViewletChooser: function(event) {
		if (event){
			dojo.stopEvent(event);
		}
		if (!this._addViewletAreaExpanded) {
			return;
		}
		this._addViewletAreaExpanded = false;
		this.chooserWipeOutAnimation.play();
		this.connect(this.chooserWipeOutAnimation,"onEnd",function() {
			this.dashboardContainer.metaNav.showAddViewletButton();
			dojo.publish("com.ibm.team.dashboard.web.onresize");
		});
	},
	/*
	 * Event handler for Open Add Viewlet Area button
	 */
	expandViewletChooser: function(event,noAnim) {
		if (event){
			dojo.stopEvent(event);
		}
		if (this._addViewletAreaExpanded && this._viewletChooser.style.display != "none") {
			return;
		}
		this._populateViewletChooser();
		this._addViewletAreaExpanded = true;
		if (noAnim) {
			this._viewletChooser.style.display = "block";
			this.dashboardContainer.metaNav.hideAddViewletButton();
		} else {
			this.dashboardContainer.metaNav.hideAddViewletButton();
			this.chooserWipeInAnimation.play();
		}
		dojo.publish("com.ibm.team.dashboard.web.onresize");
	},
	
	/*
 	 * Method
 	 * 
 	 * Populates the chooser tree.
 	 * Called after add viewlet panel is expanded.
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_afterChooserWipeInAnimation: function() {
		setTimeout(dojo.hitch(this._viewletChooserBlock, this._viewletChooserBlock.startup), 10);
	},
	
	/*
 	 * Method
 	 * 
 	 * Creates the content of the Add Viewlet panel.  
 	 * Called when add viewlet panel is expanded.
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_populateViewletChooser: function() {
		// Get the attach point for the viewlet chooser
		var node = this._addViewletContent;
		// If the point has children already, then the viewlet chooser was
		// already created 
		if (node.hasChildNodes()) {
			return;
		} 
		// Create the new viewlet chooser
		var dashboardItem = this._viewletPageHelper.getDashboardItem();
		var scope = dashboardItem.getScope();
		var scopeItemId = dashboardItem.getScopeItem().itemId;
		this._viewletChooserBlock = new ViewletChooserBlock({self: this, scope: scope, scopeItemId: scopeItemId}); 
		// Add the new child node
		node.appendChild(this._viewletChooserBlock.domNode);
		
		var args = {node: this._viewletChooser, duration: 200};
		
		this.chooserWipeInAnimation = dojo.fx.wipeIn(args);
		this.chooserWipeOutAnimation = dojo.fx.wipeOut(args);
		this.connect(this.chooserWipeInAnimation,"onEnd",this._afterChooserWipeInAnimation);	
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
	},	
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("DashboardContainer.js remote error: " + errorObj.message);
	}
});
})();
