/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");

(function() {

var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo",
	[dijit._Widget, dijit._Templated], {
	
	// template contains nested divs to support including it in tooltips
	templateString: "<div><div class='com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo' dojoAttachPoint='_content'></div></div>",
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");
		this._dto = args.memberInfoDto;
		this._linkTeams = args.linkTeams;
		this._linkProjects = args.linkProjects;
	},
	
	postCreate: function() {
		this._populateContent();
	},
	
	_populateContent: function() {
		this._content.innerHTML = "";
		this._doPersonalArea();
		if (this._dto.processAreas && this._dto.processAreas.length > 0)
			this._doProcessAreas();
	},
	
	_doPersonalArea: function() {
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		
		var nameRow = document.createElement("tr");
		if (this._dto.picture) {
			var photoCell = document.createElement("td");
			dojo.addClass(photoCell, "photo");
			photoCell.rowSpan = 3;
			var photo = document.createElement("img");
			photo.src = this._dto.picture;
			photo.width = "48";
			photo.height = "48";
			photoCell.appendChild(photo);
			nameRow.appendChild(photoCell);
		}
		var nameLabelCell = document.createElement("td");
		dojo.addClass(nameLabelCell, "label");
		nameLabelCell.appendChild(document.createTextNode(messages.name));
		nameRow.appendChild(nameLabelCell);
		var nameCell = document.createElement("td");
		nameCell.appendChild(document.createTextNode(this._dto.name));
		nameRow.appendChild(nameCell);
		
		var idRow = document.createElement("tr");
		var idLabelCell = document.createElement("td");
		dojo.addClass(idLabelCell, "label");
		idLabelCell.appendChild(document.createTextNode(messages.userId));
		idRow.appendChild(idLabelCell);
		var idCell = document.createElement("td");
		idCell.appendChild(document.createTextNode(this._dto.userId));
		idRow.appendChild(idCell);
		
		var emailRow = document.createElement("tr");
		var emailLabelCell = document.createElement("td");
		dojo.addClass(emailLabelCell, "label");
		emailLabelCell.appendChild(document.createTextNode(messages.email));
		emailRow.appendChild(emailLabelCell);
		var emailCell = document.createElement("td");
		var emailLink = document.createElement("a");
		emailLink.href = "mailto:" + this._dto.email;
		emailLink.appendChild(document.createTextNode(this._dto.email));
		emailCell.appendChild(emailLink);
		emailRow.appendChild(emailCell);
		
		tbody.appendChild(nameRow);
		tbody.appendChild(idRow);
		tbody.appendChild(emailRow);
		
		table.appendChild(tbody);
		
		this._content.appendChild(table);
	},
	
	_doProcessAreas: function() {
		var ul = document.createElement("ul");
		dojo.addClass(ul, "processList");
		
		var title = document.createElement("li");
		dojo.addClass(title, "processTitle");
		title.appendChild(document.createTextNode(messages.process));
		ul.appendChild(title);
		
		this._projectPaths = new Object();
		for (var i in this._dto.processAreas) {
			this._doProcessArea(ul, this._dto.processAreas[i], 0, this._linkProjects);
		}
		
		this._content.appendChild(ul);
	},
	
	_doProcessArea: function(ul, area, indent, link) {
		var li = document.createElement("li");
		dojo.addClass(li, "processListEntry");
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		
		var td = document.createElement("td");
		td.style.paddingLeft = indent + "em";
		this._storeProjectPaths(area);
		
		if (link) {
			var name = document.createElement("a");
			
			var projectPath = this._projectPaths[area.itemId];
			var href = (projectPath ? projectPath : "") + "#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD;
			if (area.path) {
				href += "&" + DashboardConstants.URL_PARAM_TEAM + "=" + area.path;
			}
			name.href = href;
		} else {
			var name = document.createElement("span");
		}
		name.appendChild(document.createTextNode(area.name));
		td.appendChild(name);
		tr.appendChild(td);
		
		if (area.roles && area.roles.length > 0) {
			td = document.createElement("td");
			dojo.addClass(td, "roles");
			for (var i in area.roles) {
				if (i != 0)
					td.appendChild(document.createTextNode(", "));
				var span = document.createElement("span");
				span.appendChild(document.createTextNode(area.roles[i]));
				var tipSpan = document.createElement("span");
				var innerSpan = document.createElement("span");
				innerSpan.innerHTML = messages.loading;
				innerSpan.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo-TooltipLoading";
				tipSpan.appendChild(innerSpan);
				new HoverPopup({
					connectNode: [span],
					callback: dojo.hitch(this,this._populateTooltip),
					callbackArg: {tipSpan: innerSpan, roleId: area.roles[i],
						processAreaScope: indent == 0 ? DashboardConstants.SCOPE_PROJECT_AREA : DashboardConstants.SCOPE_TEAM_AREA,
						processAreaItemId: area.itemId},
					repopulateDelay: -1}, tipSpan);
				td.appendChild(span);
			}
			tr.appendChild(td);
		}
		
		tbody.appendChild(tr);
		table.appendChild(tbody);
		li.appendChild(table);
		
		ul.appendChild(li);
		
		if (area.children) {
			for (var i in area.children) {
				this._doProcessArea(ul, area.children[i], indent + 1, this._linkTeams);
			}
		}
	},
	
	_storeProjectPaths: function (area) {
		if (area.path)
			return;
		var pathname = window.location.pathname;
		var params = window.location.search;
		var projectPath = pathname.substring(0, pathname.lastIndexOf('/') + 1) + encodeURIComponent(area.name) + (params ? params : "");
		this._projectPaths[area.itemId] = projectPath;
		if (area.children)
			this._storeChildrenPaths(area.children, projectPath);
	},
	
	_storeChildrenPaths: function (array, path) {
		for (var i = 0; i < array.length; i++) {
			var area = array[i];
			this._projectPaths[area.itemId] = path;
			if (area.children)
				this._storeChildrenPaths(area.children, path);
		}
	},
	
	_populateTooltip: function(args) {
		var self = this;
		var serviceObj = {
			success: function(/* string */ description) {
				args.tipSpan.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo-Tooltip";
				if (description)
					args.tipSpan.innerHTML = HTMLUtility.escapeHTML(description);
				else
					args.tipSpan.innerHTML = messages.noDescription;
				args.tooltip.refresh();
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getRoleDescription(args.roleId, args.processAreaScope, args.processAreaItemId, srh);
	}
});

})();