/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("net.jazz.ajax.ui.internal.Dialog");

dojo.require("dijit._Widget");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectArea");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MultiSelectMessages");

/*
 * Aliases 
 */
var MultiSelectArea = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectArea;
var Dialog = net.jazz.ajax.ui.internal.Dialog;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * TODOs 
 */

(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectDialog",
	dijit._Widget, {

	/*
	 * Properties
	 */	
	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return MultiSelectDialog
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MultiSelectMessages");
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->
		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		this._callBackDialogExit = null;
		this._doIt = false;
		
		this._dialog = null;
		
		// Create the swap list area
		this._multiSelectArea = new MultiSelectArea({
			context: this,
			unselectedListTitleText: this.messages.msgAllParameters,
			selectedListTitleText: this.messages.msgSelectedParameters,
			sortField: this.sortField
		}); 		
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	show: function() {
		var self = this;
		// Create the dialog
		this._dialog = new Dialog({
			contentNode: this._multiSelectArea.domNode,
			primaryTitle: this.messages.msgSelectParameters,
			width: "670px",
			isClosable: true,
			onClose: function() {
				self._handleEventXImageClicked();
			}
		});

		// Debug
		// this._debugDialog();		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	hide: function() {
		// Ensure the dialog is open
		if (this._isObjectDefined(this._dialog) === false) {
			return;
		}
		// Close dialog
		this._dialog.close();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_handleEventXImageClicked: function() {
		// Fire dialog exit event
		this._fireEventDialogExit();
		// Destroy widgets
		this._multiSelectArea.destroy();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_fireEventDialogExit: function() {
		// Ensure a call back was defined
		if (this._callBackDialogExit === null) {
			return;
		}
		// Create the event
		var event = new Object();
		event.dialog = this;
		event.doIt = this._doIt;
	 	
		// Execute the defined call back
		this._callBackDialogExit.handleEventMultiSelectDialogExit(event);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object args
 	 * 			. Object self
 	 * 				. Function handleEventMultiSelectDialogExit(Object event)
 	 * @return boolean
 	 */	
	setCallBackDialogExit: function(args) {
		// Set the listener
		this._callBackDialogExit = args.self;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @return boolean
 	 */	
	_isObjectDefined: function(variable) {
	    if (typeof(variable) != 'object') {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_handleEventOKButtonClicked: function() {
		// Set positive flag
		this._doIt = true;
		// Hide the dialog
		this.hide();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_handleEventCancelButtonClicked: function() {
		// Hide the dialog
		this.hide();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectDialog";
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_debugDialog: function() {
		// Create sample data
	 	var unselectedEntries = new Array();
		for (var x = 0; x < 20; x++) {
			var entry1 = new Object();
			entry1.label = "Entry " + x;
			entry1.value = "value";
			entry1.id = x;
			entry1.type = 0;
			unselectedEntries.push(entry1);
		}
	 	var selectedEntries = new Array();
		for (var y = 20; y < 25; y++) {
			var entry2 = new Object();
			entry2.label = "Entry " + y;
			entry2.value = "value";
			entry2.id = y;
			entry2.type = 1;
			selectedEntries.push(entry2);
		}
		// Create sample icons
		var iconMap = new Array();
		iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/team-dash-icon.gif";
		iconMap[1] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/compont_obj.gif";

		// API calls
		this.setIconMap(iconMap);
		this.setUnselectedEntries(unselectedEntries);
		this.setSelectedEntries(selectedEntries);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setSelectedEntries: function(entries) {
		this._multiSelectArea.setSelectedEntries(entries);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		return this._multiSelectArea.getSelectedEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	setUnselectedEntries: function(entries) {
		this._multiSelectArea.setUnselectedEntries(entries);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getUnselectedEntries: function() {
		return this._multiSelectArea.getUnselectedEntries();
	},

	/*
 	 * Public Method
 	 * 
 	 * Type is the key (Number)
 	 * Icon path is the value (String)
 	 * 
 	 * @param Array map : [Number]=String
 	 * @return void
 	 */	
	setIconMap: function(map) {
		this._multiSelectArea.setIconMap(map);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		this._multiSelectArea.showLoading(show);
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Call super
		this.inherited(arguments);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
