/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.UUIDUtility");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {

var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemQueryChooser = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser;
var UUIDUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.UUIDUtility;

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider",
	CustomPreferenceProvider, {
		
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");
		this._labelMap = new Array();
		this._fetchingMap = new Array();
		this._prefId = args.prefId;
	},
	
	initDefault: function(queryDTO) {
		this._cacheLabel(queryDTO);
		this._defaultValue = queryDTO.itemId;
	},
		
	getLabel: function(value){
		var label = this.getTextLabel(value);
		if (label != "")
			return this._buildLabel(label);
		this._fetchLabel(value);
		return null;
	},
	
	getTextLabel: function(value) {
		if (UUIDUtility.isValidUUID(value) == false) {
			if (this._defaultValue)
				value = this._defaultValue;
			else
				return messages.providerNoQuery;
		}
		if (value in this._labelMap)
			return this._labelMap[value];
		return "";
	},
	
	getLoadingLabel: function() {
		return this._buildLabel(messages.loading);
	},
	
	_buildLabel: function (text) {
		return '<input type="text" readonly="readonly" value="' + text.replace(/"/g, "&quot;") + '"/>';
	},

	createChooser: function(onOk, settings, viewlet) {
		// Get scope
		var scope = settings.scope || settings.scope == 0 ? settings.scope : viewlet.getScope();
		// Get scope item ID
		var scopeItemId = settings.scopeItemId ? settings.scopeItemId : viewlet.getScopeItem().itemId;
		// Get current value
		var currentValue = settings.preferences[this._prefId];
		if ((UUIDUtility.isValidUUID(currentValue) == false) && this._defaultValue)
			currentValue = this._defaultValue;
		
		var params = {
			scope: scope,
			scopeItemId: scopeItemId,
			onOk: dojo.hitch(this,function(queryDTO) {
				this._cacheLabel(queryDTO);
				onOk(queryDTO.itemId, this._prefId);
			}),
			multiSelect: false,
			defaultValue: currentValue
		};
		new WorkItemQueryChooser(params);
	},
	
	_fetchLabel: function (itemId) {
		if (itemId in this._fetchingMap)
			return;
		this._fetchingMap[itemId] = true;
		var self = this;
		var serviceObject = {
			_successCallback: function(/*array*/ queryDTO) {
				if (queryDTO == null)
					self._cacheLabel({itemId: itemId, name: messages.providerNoQuery});
				else
					self._cacheLabel(queryDTO);
			},
			_errorCallback: function(error) {
				console.error(error);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		QueryClient.getQuery(responseHandler, itemId, false);
	},
	
	_cacheLabel: function(queryDTO) {
		this._labelMap[queryDTO.itemId] = queryDTO.name;
	}
});

})();
