/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("dojox.date.posix");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");

// editor parts
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout");
dojo.require("com.ibm.team.workitem.web.client.internal.editor.SubscriptionManager");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var EditorUtil            = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;

var SubscriptionManager = com.ibm.team.workitem.web.client.internal.editor.SubscriptionManager;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;

var DEFAULT_TAB             = "overview"; // $NON-NLS-1$
var DATE_FORMAT_NO_TIME     = "%b %e, %Y";
		/*
		 * The KNOWN_ATTRIBUTES describe which work item properties to return.
		 * If false, skip the property.
		 * For properties that are returned, the map identifes the attribute of the property
		 * If "true" then just return the property itself, otherwise return the named attribute.
		 * 
		 * Example: duration returns the value, foundIn returns workItem.foundIn.id, creationDate is skipped
		 * 
		 * IE, all attributes other than these are added to form elements
		 * This matches some login in the REST service
		 */
		
		var KNOWN_ATTRIBUTES = { //$NON-NLS this list does not need to be translated
			category: "id",
			correctedEstimate: "label",
			creationDate: false,  			// immutable (id)
			creator: false,					// immutable
			description: "content",
			dueDate:"id",
			duration: "label",
			foundIn:"id",
			id: false, 						// immutable
			internalComments: false, 		// special case
			internalPriority: "id",
			internalResolution: "id",
			internalSeverity: "id",
			internalState: "id",
			internalSubscriptions: false,   // special case
			modified: false,				// immutable
			modifiedBy: false,				// immutable
			owner: "itemId",
			projectArea: "id",
			resolutionDate: false,			// immutable
			resolver: false,				// immutable	
			startDate: "id",				// immutable
			summary: "content",
			tags: "content",
			target: "id",
			teamArea: "id",
			timeSpent: "label",
			workItemType: false				// special case
		};

		/*
		 * DERIVED_ATTRIBUTES (always true) show attributes of work items
		 * that are derived and do not need to be added to form elements
		 */
		var DERIVED_ATTRIBUTES = {
			stateId:  true,
//			comment: true,
			intervalItemId:true
//			type: true,
		};
		
		 // Key map translates some key changes
		var ATTRIBUTE_KEY_MAP = {
//			severity: "internalSeverity",
//			priority: "internalPriority",
//			resolution: "internalResolution",
//			comment: "internalComments"
		};



/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditor.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemEditor.css"),
		
		//
		// Widget initialization
		//
		
		/**
		 * Dojo widget lifecycle method; called once per web UI instance;
		 * responsible for creating header widget and subtab widgets.
		 */
		postCreate: function() {
			checkNotNull(this.parentController);
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages"); 
			
			// TODO: Manage all updates in managers like subscription to handle multiple same sections
			// e.g., if there are multiple "approvals" sections, they should all use the same data		
			this._updateManagers = [];
			this.isWorkItemEditor = true;
			this._subscriptionManager = new SubscriptionManager(this);
			this._subscriptionManager.addUpdateListener(this);
			this._updateManagers.push(this._subscriptionManager); 
			this.defaultTab = DEFAULT_TAB;

			// initialize editor parts map
			this._editorParts= {};
			this._linkManager = null;
			this.editorUtil= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
			this.pageController = this.parentController; // workItemPageController
			
			// each of the parameters in 'createArgs' is available to each
			// editor part via 'this' (e.g. this.parentController).
			var createArgs= {
				parentController: this,
				editorUtil: this.editorUtil
			};
			
			// create the editor header
			this._editorParts["header"]= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader(
				createArgs, this._headerPlaceholder);
			
		},
		getDefaultTabName: function() {
			if(!this._editorParts || this._editorParts[this.defaultTab]) {
				return this.defaultTab;
			}
			for(var key in this._editorParts) {
				if(key != "header") {
					return key;
				}
			}
			return this.defaultTab ;
		},
		/* 
		 * DestroyWidget should be called whenever the page is destroyed and should
		 * propagate to all widgets.
		 */
		destroyWidget: function()  {
			this._destroyChildWidgets();
			if(this._subscriptionManager) {
				this._subscriptionManager.removeUpdateListener(this);
			}
			this.destroy();
		},
		_destroyChildWidgets: function(skipHeader) {
			for (var partName in this._editorParts) {
				if(skipHeader && partName == "header") continue;
				var editorPart= this._editorParts[partName];
				if(editorPart.destroyWidget) {
					editorPart.destroyWidget();	
				} else {
					if(editorPart.destroy) {
						editorPart.destroy();
					}
				}
				delete this._editorParts[partName];
			}
			this._linkManager = null;
		},

		/**
		 * Create a subtab for the work item editor.
		 * 
		 * @param {String} logicalName this name to identify the editor part via
		 * 	this._editorParts and also the name to specify which widget to show
		 *  in the viewWorkItem URL.
		 * @param {widget} widget is a widget to associate
		 *  with the reference from this._editorParts[logicalName].
		 * @param {Object} createArgs arguments with which to initialize the widget
		 * @param {String} tabLinkLabel the label of the hyperlink that allows users
		 *  to view the particular tab.
		 */
		_createTabPart: function(logicalName, widget,  tabLinkLabel) {
			if(logicalName) logicalName = escape(logicalName).replace(/\%/g, "_"); // NLS tab names
			this._editorParts[logicalName]= widget;
			// create hyperlink for each tab
			
			try {
			if (this._tabLinkContainer) {
				for (var i = 0; i < this._tabLinkContainer.childNodes.length; i++) {
					if (this._tabLinkContainer.childNodes[i] && this._tabLinkContainer.childNodes[i].__jazzTabName == logicalName) {
						return; // already exists
					}
				}
			}
			} catch(e) {console.debug(dojo.string.substitute(this.messages.errorCreatingTab, [ logicalName ])) };
			
			var link = document.createElement("a");
			dojo.addClass(link,"tab");
			link.href = "#";
			link.__jazzTabName= logicalName;
			
			
			var innerWrapper = document.createElement("div");
			innerWrapper.className = "innerWrapper";
			
			var topWrapper = document.createElement("div");
			topWrapper.className = "topWrapper";
			
			var topBorderDiv = document.createElement("div");
			topBorderDiv.className = "topBorder";
			topWrapper.appendChild(topBorderDiv);
			
			var cornerDiv = document.createElement("div");
			cornerDiv.className = "corners";
			topWrapper.appendChild(cornerDiv);
									
			link.appendChild(topWrapper);

			
			var titleDiv = document.createElement("div");
			titleDiv.className = "title";
			titleDiv.innerHTML = tabLinkLabel;
			innerWrapper.appendChild(titleDiv);

			link.appendChild(innerWrapper);
						 
			this._tabLinkContainer.appendChild(link);
			
		
		},
		getAttachmentInfo: function(number ) {
			if(!number) return;
			if(!isNaN(number)) {
				try{
					number = parseInt(number);
				} catch(e) { return };
			}
			if(!this._workItem || !this._workItem.linkTypes ||  this._workItem.linkTypes.length == 0) return;
			for(var i=0;i<this._workItem.linkTypes.length;i++) {
				if(this._workItem.linkTypes[i].id=="com.ibm.team.workitem.linktype.attachment") {
					if(this._workItem.linkTypes[i].linkDTOs) {
						for(j=0;j< this._workItem.linkTypes[i].linkDTOs.length;j++) {
							if(this._workItem.linkTypes[i].linkDTOs[j] 
								&& this._workItem.linkTypes[i].linkDTOs[j].target 
								&&  this._workItem.linkTypes[i].linkDTOs[j].target.id === number) {
								return this._workItem.linkTypes[i].linkDTOs[j].target;
							}
						}
					}					
				}
			}
			
		},

		/**
		 * Displays the new work item in the work item editor, open initially to the
		 * tab specified by <code>initialTabl</code> or to the default tab if the
		 * caller doesn't specify an initial tab.
		 * 
		 * @param {Object} workItem the work item DTO to display
		 * @param {String} (optional) initialTab the name of the initial tab to show
		 */
		setWorkItem: function(workItem, initialTab, newTypeId, op) {
			var isTypeChange = newTypeId != null;
			this.changedTypeId = newTypeId;
			var retainedState = { };
			/*
			 * If its a type change, the save all the retained state on all tabs
			 * 
			 * Otherwise it is a refresh and only save the new comment text
			 */
			if (isTypeChange) {
				retainedState = this._getFormElements(/* includeUnchangedAttributes */ false);
			}
			if (this._editorParts.overview && this._editorParts.overview.hasRetainedState()) {
				retainedState.unsavedCommentText= this._editorParts.overview.getNewCommentText();
			}
			// initialize dirty state to false
			this._isDirty= false;
			
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header._workItem = workItem;
			this._editorParts.header.markBusy(this.messages.busyLoading);

			if(workItem.id < 1) { 
				this._editorParts.header.summary.focus();
			}

			// store a reference to the current work item
//			this._destroyChildWidgets(/* skipHeader */ true);
			this._editorParts = { header: this._editorParts.header};
			
			this._editorParts.header.setHeaderToolbarState(workItem.id < 1);
			
			if(this._workItem  && this._workItem._newIdLabel) {
				workItem._newIdLabel = this._workItem._newIdLabel;
			}
			this._workItem= workItem;
			this._subscriptionManager.setWorkItem(workItem);
			this._editorParts.header.updateRefreshTimestamp(this.isNewWorkItem());
			
			// parallel requests
			var params = {workItem: workItem, initialTab: initialTab, retainedState: retainedState, isTypeChange: isTypeChange, newTypeId: newTypeId, op: op};
			this._fetchEditableProperties(params);
			this._fetchPresentationProperties(params);
			this._fetchTime(params);
			
			return retainedState.unsavedCommentText == null ? false : retainedState.unsavedCommentText.length> 0;
		},
		focus: function () {
			if(this._editorParts && this._editorParts.header) this._editorParts.header.focus();
		},
		_fetchTime: function(params) {
			if(WorkItemUtil.serverTimeDelta && params.presentationProps && params.editProps) {
				self._populateWorkItem(params);
			}
			if(WorkItemUtil.serverTimeDelta) return; // already loaded;
			var self = this;
			var serviceObj= {
				success: function(timeNow) {
					WorkItemUtil.setServerTimeDelta(timeNow)
					if (params.presentationProps && params.editProps) {
						// only populate when we have both responses
						self._populateWorkItem(params);
					}
				},
				failure: function(errorObj) {
					self.handleLoadingError(errorObj, self.messages.errorLoadingTimeFromServer);
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			WorkItemClient.getTime(srh, { });
		},
		_fetchEditableProperties: function(params) {
			var self = this;
			var serviceObj= {
				success: function(editProps) {
					params.editProps = editProps;
					if (params.presentationProps && WorkItemUtil.serverTimeDelta) {
						// only populate when we have both responses
						self._populateWorkItem(params);
					}
				},
				failure: function(errorObj) {
					self.handleLoadingError(errorObj, self.messages.errorLoadingProperties);
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			var args= this.isNewWorkItem() ? {newWorkItem: true, typeId: params.workItem.attributes.workItemType.id} : {itemId: params.workItem.itemId};
			if(params.isTypeChange) {
				args.typeId= params.newTypeId;
			}
			WorkItemClient.getWorkItemEditableProperties(srh, args);
		},
		
		_fetchPresentationProperties: function(params) {
			var self = this;
			var serviceObj= {
				success: function(presentationProps) {
					params.presentationProps = presentationProps;
					if (params.editProps && WorkItemUtil.serverTimeDelta) {
						// only populate when we have both responses
						self._populateWorkItem(params);
					}
				},
				failure: function(errorObj) {
					self.handleLoadingError(errorObj, self.messages.errorLoadingPresentation);
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			var args = {}
			var typeId =  params.isTypeChange ? params.newTypeId : (params.workItem.attributes.workItemType && params.workItem.attributes.workItemType.id != null && params.workItem.attributes.workItemType.id != "" ? params.workItem.attributes.workItemType.id : null);
			if(typeId) 
			    args.typeId = typeId;
				
			if (!params.isTypeChange) {
				args.itemId= params.workItem.itemId;
			}
			WorkItemClient.getEditorPresentation(srh, args);
		},
		
		_populateWorkItem: function(params) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._workItem.displayedAttributes = {};
			this._workItem.editProps = params.editProps;
			this._setEditableProperties(params.workItem, params.editProps, params.presentationProps, params.initialTab, params.retainedState, params.isTypeChange, params.newTypeId);

			if(params.isTypeChange || params.retainedState.unsavedCommentText || this._workItem._draftId) { // TODO: Generalized Retained state, e.g., this.hasRetainedState();
				this.markDirty();
				if (params.op) { this.parentController.finishOperation(params.op); }
				this._editorParts.header.markFullyLoaded();
			}
			
			if(this._workItem && this._workItem.attributes && this._workItem.attributes.projectArea && this._workItem.attributes.projectArea.isArchived) {
					var msg = {
						message: this.messages.msgProjectAreaIsArchived, 
						action: "ALERT",
						isSticky: false,
						isError: false
					}
					this._editorParts.header.setHeaderMessage(msg);
			}
			
			// Handle the tab changing while loading
			var parms = WorkItemUtil._getHashRequestParams(this.getURL());
			var tab;
			for(var j=0;j<parms.length;j++) {
				if(parms[j].name == "tab") {
					tab = parms[j].value;
					break;
				}
			}
			if(tab) {
				this.showTab(tab)
			} else {
				this.showTab(this.getDefaultTabName())
			}
			
		},
		
		tabSpecs: {
				builtInOverviewLayout: {
						isDefault: true,
//						title: "Overview",
						renderer: "_renderOverview",
						knownTab: "overview"
						},
				builtInLinksLayout: {
//						title: "Links",
						renderer: "_renderTabContent",
						knownTab: "links"
						},
				builtInApprovalsLayout: {
						renderer: "_renderTabContent",
						knownTab: "approvals"
						},
				builtInCustomAttributesLayout: {
//						title: "Custom",
						renderer: "_renderCustomAttributes"
						},
				builtInHistoryLayout: {
						renderer: "_renderTabContent",
						knownTab: "history"
						}
		},
		knownTabs: {
			links: false,
			approvals: false,
			overview: false
		},
		
		// Set the editable properties in each editor part 
		// this is a continuation of 'setWorkItem'.
		_setEditableProperties: function(workItem, editProps, presentationProps, initialTab,  retainedState, isTypeChange, newTypeId) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel			
//			if(!isTypeChange) {
				dojox.data.dom.removeChildren(this._tabLinkContainer);
				dojox.data.dom.removeChildren(this._currentTabContainer);
				this._destroyChildWidgets(/* skipHeader */ true);
//			}
			
			presentationProps = presentationProps || {}; 
			presentationProps.pages = presentationProps.pages || [];
			
			
			for (var i=0;i< presentationProps.pages.length; i++) {
				var tab = presentationProps.pages[i];
				var spec = this.tabSpecs[tab.layout];
				this.defaultTab = DEFAULT_TAB;
				if (spec && spec.renderer && this[spec.renderer]) {
					var tabId = this[spec.renderer](editProps, presentationProps.pages[i], retainedState, isTypeChange);
					if (spec.isDefault) {
						this.defaultTab = tabId;
					}
					if(spec.knownTab) {
						this.knownTabs[spec.knownTab] = tabId;
					}
				}
			}

			this._editorParts.header.reset(isTypeChange);
			var isCompatibleTypeChange = this._editorParts.header.bindWorkItem(workItem, retainedState, isTypeChange, newTypeId);

			this.showTab(initialTab || this.getDefaultTabName());
			if (!isTypeChange) {
				this._isDirty = false;	
			}

			
			this._editorParts.header.markFullyLoaded();
			this._editorParts.header.updateSubscribeSelf();
			
			// convert the array of attribute values to a map keyed by the property name
			var allValuesMap= {};
			for (var i=0; i<editProps.allValues.length; i++) {
				var cur= editProps.allValues[i];
				allValuesMap[cur.attributeName]= cur.uiItems;
			}
			
			// for each editor part, call a set of optional functions, if the
			// function exists on each part
			//
			// As of M4 this is only called for the header
			for (var tabName in this._editorParts) {
				var editorPart= this._editorParts[tabName];
				if (dojo.isFunction(editorPart.bindOptionalValues)) {
					editorPart.bindOptionalValues(allValuesMap, retainedState, isTypeChange, isCompatibleTypeChange);
				}
				if (dojo.isFunction(editorPart.bindRequiredProperties)) {
					editorPart.bindRequiredProperties(editProps.requiredProperties || [], retainedState, isTypeChange);
				}
			}
			
			// update the tab URLs for the new work item
			var tabLinks= this._tabLinkContainer.getElementsByTagName("a");
			var self = this;
			dojo.forEach(tabLinks, function(link) {
				tabName= link.__jazzTabName !== self.getDefaultTabName() ? link.__jazzTabName : null;
				link.href= self.editorUtil._getURIForTab(tabName, this._workItem);
			});

			if(this._editorParts.overview) this._editorParts.overview.markFullyLoaded();
			
		},

		// Hard coded Tab
		_renderCustomAttributes: function(editProps, pageProps, retainedState, isTypeChange) {
			// Convert the page props left and right to details and discussion
			var overviewSections = [];			
			if(pageProps && pageProps.sections) {
				var sections = pageProps.sections;
				for(var i=0;i<sections.length;i++){
					var isAutocollect = false;
					if(sections[i].properties) {
						for(var k=0;k<sections[i].properties.length;k++) {
							if(sections[i].properties[k] && sections[i].properties[k].key == "autocollect" && sections[i].properties[k].value == "true") {
								isAutocollect = true;
								break;
							}
						}
					}
					if (!isAutocollect) {
						if (sections[i].slot == "left") {
							sections[i].slot = "details";
						}
						else 
							if (sections[i].slot == "right") {
							   if(sections[i].presentations && sections[i].presentations[0] && sections[i].presentations[0].partId=="com.ibm.team.workitem.presentation.discussion") {
									sections[i].slot = "discussion";
							   } else {
									sections[i].slot = "description";
							   }
							}
						overviewSections.push(sections[i]);
					}
				}
			}
			if(overviewSections.length == 0) return; // skip tab			
			pageProps.sections = overviewSections;
			
			return this._renderOverview(editProps, pageProps, retainedState, isTypeChange, /* isCustomAttribute */ true);
		},
		_renderOverview: function(editProps, pageProps, retainedState, isTypeChangem, isCustomAttributeLayout) {

			var createArgs= {
				workItem: this._workItem,
				parentController: this,
				editorUtil: this.editorUtil,
				editProps: editProps,
				pageProps: pageProps,
				retainedState: retainedState,
				isCustomAttributeLayout: isCustomAttributeLayout
				
			};
			
			var _overview = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview( createArgs);

			var title = this.messages.defaultOverviewTabTitle; // "Overview"
			if (pageProps && pageProps.properties) {
				var props = pageProps.properties;
				for (var i = 0; i < props.length; i++) {
					if (props[i].key == "title") {
						title = props[i].value;
						break;
					}
				}
			}

			var tabName = title.toLowerCase().replace(/ /g,"");
			this._createTabPart(tabName, _overview, title);

			return tabName;
		},
		
		// Generic tab renderer	
		_renderTabContent: function(editProps, presentationProps, retainedState, isTypeChange) {

			var title = dojo.string.substitute(this.messages.defaultGenericTabTitlePrefix, [ "" + (this._tabLinkContainer.childNodes.length + 1)] );
			var props = presentationProps.properties;
			for(var i=0;i<props.length;i++){
				if(props[i].key == "title") {
					title = props[i].value;
					break;
				}
			}

			var tabId = title.toLowerCase().replace(/ /g,"_");
			tabId = escape(tabId).replace(/\%/g, "_"); // NLS tab names
			if(isTypeChange && this._editorParts.tabId) {
				return tabId;
			}
			
			var createArgs= {
				workItem: this._workItem,
				parentController: this,
				editProps: editProps,
				presentationProps: presentationProps,
				editorUtil: this.editorUtil,
				retainedState: retainedState,
				tabId: tabId
			};
			
			
			var _widget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout( createArgs);
			this._createTabPart(tabId, _widget, title);

			return tabId;			
		},	
		/**
		 * Returns <code>true</code> if the work item editor is displaying a new
		 * work item, <code>false</code> if the work item editor is displaying an
		 * existing work item. 
		 */
		isNewWorkItem: function() {
			if(!this._workItem) return false;
			return this._workItem.id < 0;
		},
		
		disableSave: function(isDisabled) {
				for(var partName in this._editorParts) {
					if(this._editorParts[partName] && this._editorParts[partName].disableBottomSaveCommand) {
						this._editorParts[partName].disableBottomSaveCommand(isDisabled)
					}
				}
		},
		
		/**
		 * Return the id of work item currently bound to the editor.
		 */
		getId: function() {
			return this._workItem.id;
		},
		
		/*
		 * Get the full version of the label, e.g. "Defect 1235: Something broke".
		 */
		getSummaryLabel: function() {
			return this.editorUtil.getSummaryLabel(this._workItem);
		},
		/*
		 * Get the short version of the label, e.g. "Defect 1235".
		 */
		getShortLabel: function() {
			if(!this._editorParts || !this._editorParts.header) return null; // timing issue on cancel
			if(!this._workItem){
				return null;
			}
			var id = this._workItem.id;
			
			var typeName  = this.messages.defaultTypeLabel;
			try {
				typeName =this._workItem.attributes.workItemType.label;
				if (this._editorParts.header._updatedTypeName) {
				typeName = this._editorParts.header._updatedTypeName;
				}
			} catch(e) {}
			
			if(this._workItem.id < 0)
				return typeName + " " + EditorUtil.getNewWorkItemTimestampLabel(this._workItem);

			return typeName + " " + id;
		},
		getURL: function() {
			return this.url;
			
		},
		getIconURL: function() {
			var url = null; 
			try {
				if (this._editorParts && this._editorParts.header && this._editorParts.header._updatedTypeIconSrc) {
					url = this._editorParts.header._updatedTypeIconSrc;
				}
				else {
					if (this._workItem && this._workItem.attributes && this._workItem.attributes.workItemType) {
						url = this._workItem.attributes.workItemType.iconUrl;
					}
				}
			} 
			catch (e) {	}; // ignore error		
			return url;
		},
		
		/**
		 * Displays the tab specified by <code>tabName</code>.
		 * 
		 * @param {String} tabName the logical name of the tab to display.
		 */
		showTab: function(tabName) {

			// If the tab name is in the window url, override the method call.
			var urlTab = WorkItemUtil.getURLParam(window.location.href, "tab");
			if(urlTab) {
				tabName = urlTab;
			} else {
				tabName = tabName || this.getDefaultTabName();
			}
			
			if(!this._editorParts[tabName]) {
				if(this.knownTabs[tabName] ){
					tabName = this.knownTabs[tabName];
				} else {
	 				tabName = this.getDefaultTabName();
				}

			}
			
			if(!this._editorParts[tabName]) return; // click during refresh
			if(this._editorParts[tabName] && this._editorParts[tabName].showWidget) {
				this._editorParts[tabName].showWidget();
			}
			
			// swap in contents
			if (this._currentTabContainer.firstChild) {
				this._currentTabContainer.replaceChild(this._editorParts[tabName].domNode, this._currentTabContainer.firstChild);
			}
			else {
				this._currentTabContainer.appendChild(this._editorParts[tabName].domNode);
			}
			
			// update tab selection
			var links= this._tabLinkContainer.childNodes;
			dojo.forEach(links, function(link) {
				if (link.__jazzTabName === tabName)
					dojo.addClass(link, "Selected");
				else {
					dojo.removeClass(link, "Selected");
				}
			});

			if (this.url) {
				var url = this.url;
				var ix = url.indexOf("&tab=");
				if(ix == -1){
					this.url = this.url + "&tab=" + tabName;
				} else {
					url = (url + "&").replace(/&&/g, "&");
					this.url = url.replace(/&tab=.+&/, "") + "&tab=" + tabName;
				}
			}
			if(tabName != this._currentTab) {
				this._sendChangedEvent("Tab Changed");
			}
			
			// store a reference to the current tab
			this._currentTab= tabName;
		},
		
		//
		// Controller methods *only* intended to be called by editor parts via
		// this.parentController.
		//
		
		/**
		 * Add the subscribers contained in <code>subscriberArray</code> to the
		 * editor.
		 * 
		 * @param {Array} subscriberArray
		 */
		addSubscribers: function(subscriberArray) {
			this._subscriptionManager.addSubscribers(subscriberArray);
		},

		/**
		 * Removes the subscribers contained in <code>subscriberArray</code> from the
		 * editor.
		 * 
		 * @param {Array} subscriberArray
		 */
		deleteSubscribers: function(subscriberArray) {
			this._subscriptionManager.deleteSubscribers(subscriberArray);
		},
		
		/**
		 * Returns whether or not the given contributor is a subscriber to the current
		 * work item.
		 * 
		 * @param {Contributor} contributor
		 */
		isSubscriber: function(contributor) {
			return this._subscriptionManager.isSubscriber(contributor);
		},
		
		/**
		 * Mark the editor dirty only if the editor is in the correct state.
		 * <p>
		 * Background: there's a tough edge case caused by supporting the Ctrl-S
		 * save method.  If you make changes in a text input or textarea and
		 * then hit Ctrl-S, the DOM 'onchange' event will fire as soon as you
		 * set focus anywhere else on the page.  This method attempts to detect
		 * this state, and if it detects it, skip the 'mark dirty' action and
		 * clear the flag so that future 'onchange' events result in correct
		 * form dirtying. 
		 * </p>
		 * 
		 * @param {Object} e a DOM event representing a potential change.
		 */
		conditionalMarkDirty: function(e) {
			var target= e.target || e.srcElement;
			if (target.tagName === "SELECT" || !this._ctrlSWasPressed) {
				target.__jazzChanged= this.markDirty();
			}
			this._ctrlSWasPressed= false;
		},
		
		/**
		 * Return <code>true</code> if the user wishes to abandon the current
		 * editor or <code>false</code> if the user doesn't wish to abandon the
		 * current editor. 
		 */
		confirmAbandon: function() {
			var hasUnsavedCommentText = false;
			if(this._editorParts.overview) {
				hasUnsavedCommentText = this._editorParts.overview.getNewCommentText().length > 0;
			}
			var msg;
			if(this.isNewWorkItem()){
				var msg= this.messages.promptResetContinue;
				if(hasUnsavedCommentText) {
					msg = this.messages.promptResetLostChangesContinue; ;
				}
			}else{
				var msg= this.messages.promptRefreshContinue;
				if(hasUnsavedCommentText) {
					msg = this.messages.promptRefreshLostChangesContinue;
				}
			}
			return confirm(msg);
		},
		
		
		/**
		 * Respond to an error condition by forwarding the error object to the
		 * parent controller.
		 * 
		 * @param {Object} errorObj an error object created by a remote method.
		 * @param {Object} message an optional human-friendly error message
		 *  describing what the system was trying to do when the error occurred.
		 */
		handleLoadingError: function(errorObj, message) {
			this.parentController.handleError(errorObj, message);
		},
		checkLogin: function() {
			if (isGuest()) {
				if(this._isGuestRefreshing) return; // Prevent double questions
				this._isGuestRefreshing = true;
				this.parentController.pageController.checkLogin();
				this.refresh();
				this._isGuestRefreshing = false;
				return false;
			}
			return true;
		},
		/**
		 * Mark the editor as dirty and update the internal dirty state
		 */
		markDirty: function() {
			if(!this.checkLogin()) return;
			// Always enable save
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.markDirty();
			if (!this._isDirty) {
				this._isDirty= true;
				if(this._editorParts.overview) {
					this._editorParts.overview.markDirty();
				}
				this.parentController.setIsDirty(this, true);
				this._sendChangedEvent("Save Enabled");
			}
			return true;
		},
		_sendChangedEvent: function(reason) {
			// Notify listeners (Recently Viewed) that the state has changed
			// isDirty
			// Tab changed
			dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, [this._workItem]);
		},
	    isDirty: function() {
			return this._isDirty;
		},
		/**
		 * Refresh the current work item by fetching and displaying its most
		 * recent state from the server.
		 * 
		 * @param {Object} skipConfirmation optional flag to suppress a
		 *  confirmation dialog.
		 */
		refresh: function(suppressConfirmation) {
			// do nothing if the editor isn't dirty or if the user cancels
			if (!suppressConfirmation && this._isDirty && !this.confirmAbandon()) {
				return;
			}
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.setHeaderToolbarState(this._workItem.id < 0);
			// If its a new work item editor, then reset it
			if(this._workItem.id < 0) {
				var type;
				if(this._workItem.attributes && this._workItem.attributes.workItemType) {
					type = this._workItem.attributes.workItemType.id;
				}			
				this.parentController.newWorkItem({ doReset: true, type: type, id: this._workItem.id});
				return;
			}
			this._editorParts.header.updateRefreshTimestamp(this.isNewWorkItem());
			
			// call viewWorkItem directly since the URL isn't going to change
			var obj= { id: this._workItem.id };
			if (this._getCurrentTab()) {
				obj.tab= this._getCurrentTab();
			}
			this.parentController.viewWorkItem(obj, /* isRefresh */ true);
		},
		askToSave: function() {
			if(!this._isDirty) return;
			if(confirm(dojo.string.substitute(this.messages.promptSave, [ this.getShortLabel()]))) {
				this.parentController._clearSaveParameter();
				this._editorParts.header._handleSaveClick();
			} else {
				this.parentController._clearSaveParameter();
			}
		},
		
		/**
		 * Save the changes to work item currently displayed in the editor.
		 */
		save: function() {

			var attachmentPart = this.findObjectWithProperty({
					property: "isUploadInProgress"
				});
			
		    if(attachmentPart && attachmentPart.isUploadInProgress()) {
				if(confirm(this.messages.promptAttachmentUploadInProgress)) {
					attachmentPart.cancelUploadInProgress();
				} else {
					var msg = {
						message: this.messages.msgSaveCancelledDueToUpload,
						details: this.messages.msgSaveCancelledDueToUploadDetails, 
						action: "ALERT",
						isSticky: false,
						isError: false
					}
					this._editorParts.header.setHeaderMessage(msg);
					return;
				}
			}
			// this gets run in the context of work item application controller
	        var op= this.parentController.createOperation(this.messages.operationSavingWorkItem, "saveWorkitem");
	    	this.parentController.startOperation(op);
	    	this._editorParts.header.clearValidationMessage(true);
	    	this._editorParts.header.markBusy(this.messages.busySaving);
			this._editorParts.header.disableSave(true);
			
	    	var self= this;
	    	var serviceObj= {
	    		success: function(workItem) {
					self._clearRetainedState();
	    			self._saveCallback(workItem.attributes.id.id, op);
// Save is always followed by refresh so just let it spin
//	    			self._editorParts.header.markFullyLoaded();
	    		},
	    		failure: function(operationReport) {
	    			self._saveErrorCallback(operationReport, op);
					if(!self._editorParts || !self._editorParts.header) return; // timing issue on cancel
					self._editorParts.header.markFullyLoaded();
	    		}
	    	};
			var responseHandler= new ServiceResponseHandler(
				serviceObj,
				"success",
				"failure");

			var appArgs = this._getSaveAppArgs();
			
			WorkItemClient.postWorkItem(responseHandler, appArgs);
		},
		_getSaveAppArgs: function() {
			var formElements= this._getFormElements(/* includeUnchangedAttributes */ true);

			// If current user made a comment, subscribe the current user
			//TODO: This only works for comments on overview page
			if(!isGuest() && formElements.comment && formElements.comment.length > 0 ) {
				var currentUser = getAuthenticatedContributor();
				if(currentUser.itemId != this._workItem.attributes.owner.itemId && 
					currentUser.itemId != this._workItem.attributes.creator.itemId 
				) {
					this._subscriptionManager.addSubscriber(currentUser);
				}	
			}
				
			// TODO: Remove target mapping
			if(formElements.target) {
				formElements.intervalItemId = formElements.target;
			}

			// If there are any null or missing values for known attributes, set them from the work item
			for(var attrKey in KNOWN_ATTRIBUTES) {
				if (KNOWN_ATTRIBUTES[attrKey] && formElements[attrKey] == null) {
					formElements[attrKey] = this._getWorkItemAttributeValue(attrKey);
				}
			}		
	
			var appArgs = {} ;
			
			// Find all the custom attributes and put them in a the attribute and value arrays
			var attributeIdentifiers = [];
			var attributeValues = [];
			var action = null;
			for(var key in formElements) {
				// If there is a value, and its different than the saved work item, add it to be saved
				if(formElements[key] != null ) {
					if(!DERIVED_ATTRIBUTES[key] && (KNOWN_ATTRIBUTES[key] != false) && (this.isNewWorkItem() 
					 ||!this._isAttributeEqual(this._getWorkItemAttributeValue(key), formElements[key], key) )) {
						var newKey = ATTRIBUTE_KEY_MAP[key] ? ATTRIBUTE_KEY_MAP[key] : key;
						if(newKey == "type") {
							newKey = "workItemType"; // Type is a special case
						}
						if (key == "internalState") {
							action = formElements[key]
						}
						else {
							if(newKey != "comments") { // Comments are a special case (so skip them) 
							    // If a widget sends the atttribute wrapped like: { type: "date", value: aValue } e.g. Timestamp 
								// then use the .value
							    var value = formElements[key].type ? formElements[key].value : formElements[key];  
								attributeIdentifiers.push(newKey);
								attributeValues.push(value);
							}
						}
					} 
				}
			}

			// FIXME: this is a hack to send back an custom attribute of a draft work item without an presentation 
			if (this.isNewWorkItem() && this._workItem.attributes["com.ibm.rqm.defect"]) {
				attributeIdentifiers.push("com.ibm.rqm.defect");
				attributeValues.push(this._workItem.attributes["com.ibm.rqm.defect"].content);
			}

			// Comment is a special case
			if(formElements.comment) {
				attributeIdentifiers.push("internalComments");
				attributeValues.push(formElements.comment);
			}

			
			appArgs.attributeIdentifiers = attributeIdentifiers;
			appArgs.attributeValues = attributeValues;
						
			// Action is required
			if(action) {
				appArgs.action = action; 
			}
			
			// ItemId and type are required
			appArgs.itemId = formElements.itemId;
			appArgs.type = formElements.workItemType ? formElements.workItemType : formElements.type;
			
			// For new work items, we always need to send back the category if it was changed
			if(this._workItem.id < 0 && !this._hasValue(attributeIdentifiers, "category")) {
						attributeIdentifiers.push("category");
						attributeValues.push(formElements.category);
				}
			
			// WorkItemType is a special case
			// Three cases below:
			// 1) Work item type is provided and changed in an enumeration
			// 2) The type was changed in an existing type, and the new type does not have a wi type enumeration
			// 3) The work item is new.
			var workItemTypeFound= this._hasValue(attributeIdentifiers, "workItemType");
			if (!workItemTypeFound) {
				if (formElements.workItemType && this._workItem.attributes.workItemType.id && (formElements.workItemType != this._workItem.attributes.workItemType.id)) {
					attributeIdentifiers.push("workItemType");
					attributeValues.push(formElements.workItemType);
			  		workItemTypeFound = true;
				}
			}

			
			// if the presentation doesn't include these form elements, use the defaults
			appArgs.type = appArgs.type ? appArgs.type : this.changedTypeId;
			appArgs.type = appArgs.type ? appArgs.type : this._workItem.attributes.workItemType.id;
			appArgs.type = appArgs.type ? appArgs.type : this.parentController.pageController.defaultType;
			appArgs.itemId = appArgs.itemId ? appArgs.itemId : this._workItem.itemId;
			if (this._workItem.stateId) {
				appArgs.stateId = appArgs.stateId ? appArgs.stateId : this._workItem.stateId;
	/*
	 * If there was a type change in an existing type, then need to include the new type
	 */
				if (!workItemTypeFound && this.changedTypeId) {
					workItemTypeFound = true;
					attributeIdentifiers.push("workItemType");
					attributeValues.push(this.changedTypeId);
				}
			}
			else { // New work item need a type
				if (!workItemTypeFound) {
					workItemTypeFound = true;
					attributeIdentifiers.push("workItemType");
					attributeValues.push(this.changedTypeId || appArgs.type);
				}
			}
			
			/*
			 * Get the commands from the elements
			 * Each command is: { appArgsId: [ commands ] }
			 */
			var saveCommands = this._getSaveCommands();
			for(var key in saveCommands) {
				var cmdArray = saveCommands[key];
					if(cmdArray.length > 0) {
					var updateArray = [];
					for(var i=0;i<cmdArray.length; i++) {
						updateArray.push(dojo.toJson(cmdArray[i]));
					}
					appArgs[key] = updateArray;					
				}
			}
			return appArgs;
	
		},
		_hasValue: function(array, value){
			for(var i=0; i< array.length;i++) {
				if(array[i] == value) {
					return true;
				}
			}
			return false;
		},
		_isAttributeEqual:function(attr, other, key) {
			// attr comes from work item
			// other comes from form elements;
			if(attr == other) return true;
			if(attr == null && other == "") return true;
			if(other == null && attr == "") return true;

// If the attr is an iso string, then convert it to time
			try {
				if(attr != null && attr != "") {
					attrDate = WorkItemUtil.fromISOString(attr)
					if(attrDate) {
						var time = attrDate.getTime();
						if(time == other) return true;
					}
				}
			} catch( e) {} // ignore

			if(other && other.type == "date") {
				other = other.value;
				var attrDate, otherDate, attrStr, otherStr;
				// If the attr is an iso string, then convert it to time
				try {
					if(other != null && other != "") {
						otherDate = WorkItemUtil.fromISOString(other)
						if(otherDate) {
							var otherTime = otherDate.getTime();
							if(time == otherTime) return true;
						}
					}
				} catch( e) {} // ignore

				// test for null conditions. dates can be -1 to indicate unset.
				if(attr == null && other == -1) return true;
				if(attr == -1 && other == null) return true;
				
				// If the dates are the same, but the hours different, then the dates are equal
				// This time is always local, and this normalizes distant time zones
				try {
					if(other && !otherDate) {
						otherDate = new Date(other);
					}
					if (attrDate && otherDate) {
						attrStr = dojox.date.posix.strftime(attrDate, DATE_FORMAT_NO_TIME);
						otherStr = dojox.date.posix.strftime(otherDate, DATE_FORMAT_NO_TIME);
						if (attrStr == otherStr) 
							return true;
					}
				} catch(e) {} // ignore, not dates
			}

			
			if(other && other.toLowerCase) {
				other = other.toLowerCase();
			}
			if(attr && attr.toLowerCase) {
				attr = attr.toLowerCase();
			}
			
			if(attr == null && other == "unassigned") return true;
			if(other == null && attr == "unassigned") return true;
			if(attr == null && other == "undefined") return true;
			if(other == null && attr == "undefined") return true;

//			if(attr == null && other == -1000) return true;
//			if(other == null && attr == -1000) return true;
			return false;
		},
		_getMailto: function(address) {
			if(this._workItem.id > 0 ) {
				var summaryLabel = this.getSummaryLabel();
				if(!summaryLabel ) summaryLabel = "";
//				return "mailto:" + address + "?subject=" + escape(summaryLabel);
				return "mailto:" + address + "?subject=" + summaryLabel;
			}
			return "mailto:" + address;
		}, 
		_getWorkItemAttributeValue:function(attr) {
			
			// Type is a special case
//			if(attr == "type") return this._workItem.workItemType;
			
			var result;
					var key = KNOWN_ATTRIBUTES[attr];
					if (key && this._workItem.attributes[attr] != null) {
						if (key == true) {
							result = this._workItem.attributes[attr]
						}
						else {
							if (this._workItem.attributes[attr][key] != null) {
								result = this._workItem.attributes[attr][key];
							}
						}
					} else {
						// Unknown attribute
						if(this._workItem.attributes[attr]) {
							if(this._workItem.attributes[attr].id)
								result = this._workItem.attributes[attr].id;
							else if(this._workItem.attributes[attr].itemId) 
								result = this._workItem.attributes[attr].itemId;
							else if(this._workItem.attributes[attr].label)
								result = this._workItem.attributes[attr].label;
							else
								result = this._workItem.attributes[attr].content;
						} else {	
						if (key == null) {
							for (var key in this._workItem.attributes[attr]) {
								if (key == attr) {
										return this._workItem.attributes[attr].id ? this._workItem.attributes[attr].id : null;
									}
								}
							}
						}
					}
			return result;
		},
		_getSaveCommands: function() {
			var allCommands= {};
			for (var partName in this._editorParts) {
				if (this._editorParts[partName].getSaveCommands) {
					this._editorParts[partName].getSaveCommands(allCommands);
				}
			}
			
			for(var i=0; i< this._updateManagers.length; i++ ) {
				if(this._updateManagers[i].getSaveCommands) {
					this._updateManagers[i].getSaveCommands(allCommands);
				}
			}
			
			return allCommands;
		},
		
		_clearRetainedState: function() {
			if(this._editorParts.overview) this._editorParts.overview.clearRetainedState();
		},
		/*
		 * Get the Approvals data
		 */
		fetchSimpleInfo: function(fetchArgs) {
//			var fetchArgs = {
//				serviceSelector: "getApprovalTypes",
//				responseObject: this,
//				responseSelector: "refreshApprovalTypes",
//				progressText: "Loading Approval Types"
//			};
	        var op= this.parentController.createOperation(fetchArgs.progressText, "fetchSimpleInfo");
	    	this.parentController.startOperation(op);

			var self=this;
			// fetch the values required to edit the work item
			var serviceObj= {
				success: function(simpleInfo) {
					if(fetchArgs.responseObject[fetchArgs.responseSelector]) {
						fetchArgs.responseObject[fetchArgs.responseSelector](simpleInfo);
					}					
					if (op) { self.parentController.finishOperation(op); }
				},
				failure: function(errorObj) {
					self.handleLoadingError(errorObj, dojo.string.substitute(self.messages.errorLoadingSimpleInfo, [ fetchArgs.serviceSelector ]));
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");

			if(WorkItemClient[fetchArgs.serviceSelector]) {
				WorkItemClient[fetchArgs.serviceSelector](srh);
			} else {
				console.debug(dojo.string.substitute(this.messages.errorInvalidFetchSimpleInputSelector,[serviceSelector]));
			}
			
		},		
		handleSubscriptionChange: function() {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.updateSubscribeSelf();
		},
		
		/**
		 * Update the label and icon of the work item type information displayed
		 * in the work item editor header.
		 * 
		 * @param {String} typeName the name of the type (e.g. 'Defect').
		 * @param {Object} imgSrc a URL for the icon for the type.
		 * @param {boolean} is this initialization or not (optional)
		 */
		updateType: function(value, typeName, imgSrc, isInitialization) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on close
			this._editorParts.header.updateType(typeName, imgSrc, isInitialization);
			
			if (!isInitialization) {
		        var op= this.parentController.createOperation(this.messages.operationLoadingTypeDefinition, "loadingType");
		    	this.parentController.startOperation(op);
		    	this._editorParts.header.markBusy(this.messages.busyLoading);
				this._editorParts.header.disableSave(true);
				
				this.setWorkItem(this._workItem, this._currentTab, value.id, op);

			}	
		},
				
		/**
		 * Display the work item with the id specified by 'id'.
		 * 
		 * @param {Object} id the id (e.g. '1234') used to look up the work
		 *  item.
		 * @param {Object} (optional) initialTab the initial tab (e.g. 'Links')
		 *  to display when the editor displays the work item.
		 */		
		viewWorkItem: function(id, initialTab) {
			checkNotNull(id);
			this.parentController._internalViewWorkItem(id, initialTab);
		},
		
		//
		// Non-controller methods (should only be called within this module).
		//
		
		/*
		 * Return the logical name of the current tab, or 'null' if the editor
		 * is displaying the default tab.
		 */
		_getCurrentTab: function() {
			return this._currentTab !== this.getDefaultTabName() ? this._currentTab : null;
		},
		/*
		 * Return an object containing the names and values of all of the form
		 * elements in the editor parts.
		 */
		_getFormElements: function(isReturningUnchangedAttributes) {
			var allFormElements= {};
			var allProperties =  { }; // Handle the case where a value is null
			for (var partName in this._editorParts) {
				var editorPartReqParms= this._editorParts[partName].getFormElements(isReturningUnchangedAttributes);
				for (var prop in editorPartReqParms) {
					allProperties[prop] = true;
					allFormElements[prop]= editorPartReqParms[prop];
				}
			}
			
			// Defect 38252: Return all unchanged attributes
			// If is returning unchanged values, include 
			// non-skipped unchanged attributes from the work item
			if(isReturningUnchangedAttributes) {
				for(var attr in this._workItem) {
					// If it is not already in the form element (e.g., was changed) 
					// and it is not on the exclusion list
					if(!allProperties[attr] && KNOWN_ATTRIBUTES[attr]){
						if(KNOWN_ATTRIBUTES[attr] == true) {
							allFormElements[attr] = this._workItem.attributes[attr];							
						} else  {
							if(this._workItem.attributes[attr][KNOWN_ATTRIBUTES[attr]]) {
								allFormElements[attr] = this._workItem.attributes[attr][KNOWN_ATTRIBUTES[attr]];
							}
						}
					}	
				}
			}
			
			return allFormElements;
		},
		
		/*
		 * Callback for a successful work item save.
		 */
		_saveCallback: function(savedWorkItemId, op) {
			if (op) { this.parentController.finishOperation(op); }
			this.parentController.setIsDirty(this, false); 
			if (this.isNewWorkItem()) {
				this.parentController.removeEditorTab(this._workItem.id);
				this.viewWorkItem(savedWorkItemId, this._getCurrentTab());
			} else {
				if(this._editorParts.overview) {
					this._editorParts.overview.clearNewCommentText();
				}
				this.refresh(/*skipConfirmation*/true);
			}
			this._isDirty = false;
		},

		/*
		 * Callback for an unsuccessful work item save.
		 */
		_saveErrorCallback: function(report, op) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.disableSave(false);

			if (op) { this.parentController.finishOperation(op); }
			
			if (report.infos && report.infos.length > 0) {
				var action, info= report.infos[0];
				if(info.description == "Specify duplicate") { // message must match value from server %NON-NLS-1%
					action = "ADD_DUPLICATE_OF_LINK_ACTION"
				} else {
					action = "ALERT"
				}
				var msg = {
						message: info.summary,
						details:  info.description,
						action: action,
						isSticky: false,
						isError: true
					}
				this._editorParts.header.setHeaderMessage(msg);
			} else {
				var msg = {
						message: report.message,
						action: "ALERT",
						isSticky: false,
						isError: true
					}
				this._editorParts.header.setHeaderMessage(msg);
			}
			this._editorParts.header.endOperation();
//			if (e.status == 500) {
//				var msg= "Unable to save the work item.\n\n";
//				alert(msg + "Details: " + e.message.replace(/\n/g, " "));
//			} else {
//				if(e.status==409 && e.message== "Conflict") {
//					var msg= "Unable to save the work item because the work item did not have the most recent state. This will happen when someone else changed it while you were editing it. Note your changes, refresh, make additional changes, then save again.\n\n";
//					alert(msg + "Details: " + e.message.replace(/\n/g, " "));
//				} else {
//					this.handleError(e, "Error saving work item");
//				}
//			}
		},
		getManagerSelector: function(attributeId) {
			if(!attributeId) return null;
			 return "is" + attributeId + "Manager";
		},
		getAttributeValue: function(attributeId) {
			if(!attributeId) return null;
			if(attributeId == "summary") {
				try {
					return this._editorParts.header.summary.value;
				} catch(e) {
					return ""; // not initialized
				}
				
			}
			try {
				var manager = this.findObjectWithProperty({
					property: this.getManagerSelector(attributeId)
				});
				if (manager && manager.getValue) {
					return manager.getValue();
				}
			} 
			catch (e) {
				console.debug(dojo.string.substitue(this.messages.errorGettingAttributeValue, [ attributeId ]));
			}
			return null;
		},
		getLinkManager: function() {
			if (this._linkManager == null) {
				this._linkManager = this.findObjectWithProperty({
					property: "isLinkManager"
				});
			}
			if(!this._linkManager == null) this._linkManager = false;
			return this._linkManager;
		},
		showLinksTab: function() {
			if(this.getLinkManager()) {
				return this._linkManager.showLinksTab();
			}	
		},
		hasDuplicateOfLink: function() {
			if(this.getLinkManager()) {
				return this._linkManager.hasDuplicateOfLink();
			}	
			return false;
		},
		notifyLinkManagerToAddDuplicateLink: function() {
			// find a tab that can handle it (probably links
			if(this.getLinkManager()) {
				this._linkManager.addDuplicateLink()
			}	
		},
		notifyLinkManagerToRemoveDuplicateLink: function() {
			// find a tab that can handle it (probably links
			if(this.getLinkManager()) {
				this._linkManager.removeDuplicateLink()
			}	
		},
		updateHeaderValidation: function() {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.updateValidationMessage();
		},
		findObjectWithProperty: function(/* property, path */ parms) {
			if(this[parms.property]) return this;
			return EditorUtil.findObjectWithPropertyInObject({
				object: this._editorParts,
				path: parms.path,
				property: parms.property
			}, this);
		},
		/*
		 * Respond to any 'keydown' event triggered by a child form element. 
		 */
		_handleGenericKeydown: function(e) {
			this._handleGenericKeypress(e, "keydown");
		},
		
		_handleSaveClick: function(e) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header._handleSaveClick(e);
		},
		
		/*
		 * Respond to any 'keyup' event triggered by a child form element. 
		 */
		_handleGenericKeyup: function(e) {
			this._handleGenericKeypress(e, "keyup");
		},
		
		/*
		 * Logic for handling keydown and keyup events; critical logic for both
		 * marking the form dirty and for processing Ctrl-S keystrokes.
		 */
		_handleGenericKeypress: function(e, type) {
			
			// ensure that the keystroke is coming from either a textarea or
			// input control, and that the control hasn't explicitly marked
			// itself to be ignored
			var target= this.editorUtil._getOriginalTarget(e);
			var inputElement= target.tagName === "TEXTAREA" || target.tagName === "INPUT" ? target : null;
			
			// on Ctrl-S keydown, save work item if in a dirty state
			// (note: 83 is apparently the magic key code for 's')
			if (type === "keydown" && e.ctrlKey && e.keyCode === 83 && !e.altKey) {
				
				if (this._isDirty) {
					if (inputElement) {	this._ctrlSWasPressed= true; }
					
					// save work item
					this.save();
				}
				
				// stop propagation
				if (e.stopPropagation) e.stopPropagation();
				else e.cancelBubble= true;
				
				// prevent default action (e.g. "Save Web Page" dialog)
				if (e.preventDefault) e.preventDefault();
				else e.returnValue= false
				
				// old DOM model
				return false;
			}
			
			// if the target's not an input or the input has explicitly opted
			// out of being considered a change (e.g. the duplicate work item
			// search dialog box's text search field) then return
			if (!inputElement || inputElement.__jazzIgnoreKeystroke) { return; }
			
			// if the editor's already dirty and the element's already marked as
			// modified, then we don't need to test anything
			if (this._isDirty && inputElement.__jazzChanged) { return; }
			
			// In order to test whether or not a value has changed as the result
			// of any keystroke, we bind to both keydown and keyup.  On keydown,
			// the value of the control will not yet have changed, so we cache
			// it.  On keyup, the value may have changed, so we compare the
			// current value with what we cached onkeydown and mark the editor
			// dirty if they differ.
			//
			// This is basically the same logic as onchange but immediately
			// responsive to keystrokes rather than requiring the control to
			// lose focus.
			if (type === "keydown") {
				// on keydown, store the current (yet unmodified) value of the
				// control
				this._originalInputValue= inputElement.value;
				this._originalInputElement= inputElement;
			} else if (type === "keyup") {
				// on keyup, mark the editor dirty if the control's value has
				// changed
				if (inputElement === this._originalInputElement &&
						inputElement.value !== this._originalInputValue) {
					this.markDirty();
					inputElement.__jazzChanged= true;
				}
			} else {
				throw new Error(dojo.string.substitute(this.messages.errorUnknownKeypressType, [type]));
			}
		}
	}
);

})();
