/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems"); //$NON-NLS-1$

dojo.require("dojo.behavior"); //$NON-NLS-1$
 
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupModes"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.SortModes"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Filters"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.BarModes"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TreeNode"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TableTree"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.Tab"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var PlanningClient		= com.ibm.team.apt.web.client.internal.PlanningClient;
var PlanningUtil		= com.ibm.team.apt.web.client.util;

var Filters 			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.Filters;
var GroupModes 			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupModes;
var SortModes 			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.SortModes;
var BarModes 			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.BarModes;
var Tab					= com.ibm.team.apt.web.ui.internal.view.plan.Tab;
var TableTree			= com.ibm.team.apt.web.ui.internal.view.common.TableTree;
var ResponseHandler		= com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler;

var DEFAULT_FILTERS		= ["resolved"]; //$NON-NLS-1$
var DEFAULT_GROUPMODE	= "toplevel"; //$NON-NLS-1$

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.view.plan.Tab, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	
	
	_planId: null,
	_myFolders: null,
	_workItems: null,
	_teamMemberWorkLoad: null,
	_referencedItems: null,
	
	_optionsSection: null,
	_optionsTable: null,
	_plannedItemsTable: null,

	_labelFilters: MSGS.PLANNEDITEMS_LABEL_FILTERS,
	_labelFiltersResolved: MSGS.FILTERS_RESOLVED,
	_labelGroupMode: MSGS.PLANNEDITEMS_LABEL_GROUPBY,
	_labelGroupModeTopLevel: MSGS.GROUPMODE_TOPLEVEL,
	_labelGroupModeOwner: MSGS.GROUPMODE_OWNER,
	_labelSortMode: MSGS.PLANNEDITEMS_LABEL_SORTBY,
	_labelSortModeMyOrder: MSGS.SORTMODE_MYSORTMODE,
	_labelSortModePriority: MSGS.SORTMODE_PRIORITY,
	_labelSortModeEstimate: MSGS.SORTMODE_ESTIMATE,
	_labelBarMode: MSGS.PLANNEDITEMS_LABEL_BARS,
	_labelBarModeLoad: MSGS.BARMODE_WORKLOAD,
	_labelBarModeProgress: MSGS.BARMODE_PROGRESS,

	constructor: function() {
		this._notYetShown= true;
		this._plannedItemsWidget= null;
		this._currentViewOptions= {};

		this._filters= null;
		this._groupMode= null;
		this._workItemSortFn= null;
		this._groupSortFn= null;
		this._barMode= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	setViewOptions: function(viewOptions) {
		this._currentViewOptions= viewOptions;
		
		var filterSelector= {};
		dojo.forEach(DEFAULT_FILTERS, function(filterId) {
			filterSelector[filterId]= true;
		});
		
		var hasExcludeOptionSpecified= false;
		for (var optionName in viewOptions) {
			if (dojo.isFunction(Filters[optionName])){
			 	filterSelector[optionName]= viewOptions[optionName] == "true"; //$NON-NLS-1$
			}
		}
		
		var filters= [];
		for (filterId in filterSelector) {
			if (filterSelector[filterId]) {
				filters.push(filterId);
			}
		}

		var groupMode= viewOptions["groupMode"]; //$NON-NLS-1$
		if (!dojo.isFunction(GroupModes[groupMode])) {
			groupMode= DEFAULT_GROUPMODE;
		}

		var sortMode= viewOptions["sortMode"]; //$NON-NLS-1$
		if (!dojo.isFunction(SortModes[sortMode])) {
			sortMode= null;
		}
		
		var barMode= viewOptions["barMode"]; //$NON-NLS-1$
		if (!dojo.isObject(BarModes[barMode])) {
			barMode= null;
		}

		this._applyOptions(filters, groupMode, sortMode, barMode);

	},

	getViewOptions: function() {
		
	},
	
	getColumnDefinitions: function() {
		return this._groupMode.getColumns();
	},

	getTeamMembers: function() {
		var result= [];
		for (var contributorId in this._teamMemberWorkLoad) {
			result.push(contributorId);
		}
		return result;
	},
	
	getMyFolders: function() {
		return this._myFolders;
	},

	getWorkLoad: function(contributor) {
		return this._teamMemberWorkLoad[contributor];
	},
	
	getBarMode: function() {
		return this._barMode;
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/templates/TabPlannedItems.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		var constructorArgs= TableTree.constructorArgs();
		this._plannedItemsWidget= new TableTree(constructorArgs, this._plannedItemsTable);

		this._plannedItemsWidget.setFilter(dojo.hitch(this, this._nodeFilterFn));
		this._plannedItemsWidget.setSorter(dojo.hitch(this, this._nodeSortFn));

		this._applyOptions(DEFAULT_FILTERS, DEFAULT_GROUPMODE, null, null);


		var _self= this;
		dojo.query("input[type=\"radio\"]", this._optionsSection). forEach(function(inputNode) { //$NON-NLS-1$ //$NON-NLS-2$
			_self.connect(inputNode, "onclick", "_clickOptionsRadioButton"); //$NON-NLS-1$ //$NON-NLS-2$ 
		});

		var _self= this;
		dojo.query(".filter input", this._optionsSection). forEach(function(inputNode) { //$NON-NLS-1$
			_self.connect(inputNode, "onclick", "_clickOptionsFilter"); //$NON-NLS-1$ //$NON-NLS-2$
		});

		dojo.style(this._optionsSection, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
	},


	destroy: function() {
		if (this._plannedItemsWidget != null) {
			this._plannedItemsWidget.destroy();
			this._plannedItemsWidget= null;
		}

		this.inherited(arguments);
	},

	// ---- Tab.js implementation ------------------------------------------------------------------------------------------

	widgetAboutToShow: function() {
		if (this._notYetShown) {
			if (this._rootGroup == null) {
				var serviceCallController= this._getServiceCallController(); 
				
				var _self= this;
				var serviceCallback= {
					_success: function(result) {
						_self._myFolders= result.myFolders;
						_self._teamMemberWorkLoad= _createWorkLoadObject(result.teamMemberWorkLoad);
						_self._referencedItems= result._referencedItemStore;
						_self._displayPlannedItems(result.workItems);
					},
					_error: function(error) {
						serviceCallController.remoteError(error);
					}
				}
				
				var responseHandler= new ResponseHandler(serviceCallback, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$
				responseHandler.setLabel(MSGS.MONITOR_LOAD_PLANNEDITEMS, serviceCallController);		
				PlanningClient.fetchPlannedWorkItems(responseHandler, this._planId);

			} else {
				this._displayPlannedItems(this._workItems);
			}

			this._notYetShown= false;
		}
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_clickOptionsRadioButton: function(evt) {
		evt.stopPropagation();
		this._currentViewOptions[evt.target.getAttribute("name")]= evt.target.getAttribute("value"); //$NON-NLS-1$ //$NON-NLS-2$
		window.location.href= PlanningUtil.getPlanViewURI(this._planId, this._currentViewOptions);
		this.setViewOptions(this._currentViewOptions);
	},
			
	_clickOptionsFilter: function(evt) {
		evt.stopPropagation();
		this._currentViewOptions[evt.target.getAttribute("value")]= evt.target.checked ? "true" : "false"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
		window.location.href= PlanningUtil.getPlanViewURI(this._planId, this._currentViewOptions);
		this.setViewOptions(this._currentViewOptions);
	},

	_applyOptions: function(filters, groupMode, sortMode, barMode) {
		// create filter list
		
		var oldFilters= this._filters;
		this._filters= dojo.map(filters, function(filterId) {
			return Filters[filterId];
		});
		
		var needsRefresh= false;
		var needsRebuild= false;
		var needsBarUpdate= false;
		if (oldFilters != null) {
			needsRefresh= this._filters.length != oldFilters.length || dojo.some(this._filters, function(filter) { return dojo.indexOf(oldFilters, filter) == -1; }); 
		}

		if (this._groupMode == null || groupMode != this._groupMode.getGroupModeId()) {
			this._groupMode = new GroupModes[groupMode](this);
			this._groupSortFn = this._groupMode.getGroupSorter() || this._defaultGroupSortFn;
			needsRebuild= true;
		}

		var availableSortModes= this._groupMode.getSortModes();
		if (sortMode == null || !dojo.some(availableSortModes, function(sorter) { return sorter.id == sortMode; }))
			sortMode= availableSortModes[0].id;

		if (this._workItemSortFn == null || this._workItemSortFn.id != sortMode) {
			this._workItemSortFn = SortModes[sortMode] || this._defaultWorkItemSortFn;
			needsRefresh= true;
		}

		var availableBarModes= this._groupMode.getBarModes();
		if (barMode == null || !dojo.some(availableBarModes, function(barModeDescriptor) { return barModeDescriptor.id == barMode; }))
			barMode= availableBarModes[0].id;

		if (this._barMode == null || this._barMode.id != barMode) {
			this._barMode = BarModes[barMode];
			needsBarUpdate= true;
		}

		if (needsRebuild) {
			if (this._workItems != null) {
				var rootNode= this._groupMode.buildNodeTree(this._workItems);
				this._plannedItemsWidget.setContent(rootNode);
			}
			needsRefresh= false;
			needsBarUpdate= false;
		}

		if (needsBarUpdate) {
			var rootNode= this._plannedItemsWidget.getContent();
			rootNode.accept(function(node){
				if (node.getNodeType() == "group") { //$NON-NLS-1$
					var createdWidget= node.getCreatedWidget();
					if (createdWidget)
						createdWidget.updateBar();
				}
				return true;
			});
		}

		if (needsRefresh) {
			this._plannedItemsWidget.refresh();
		}

		this._updateOptionsLabels();
	},
	
	_updateOptionsLabels: function() {
		var _self= this;
		dojo.query(".filter input", this._optionsTable).forEach(function(checkboxElement) { //$NON-NLS-1$
			var isSelected= dojo.indexOf(_self._filters, Filters[checkboxElement.getAttribute("value")]) != -1; //$NON-NLS-1$
			checkboxElement.checked= checkboxElement.defaultChecked= isSelected;
		});

		dojo.query(".groupBy input", this._optionsTable).forEach(function(radioElement) { //$NON-NLS-1$
			var isSelected= _self._groupMode.getGroupModeId() == radioElement.getAttribute("value"); //$NON-NLS-1$
			radioElement.checked= radioElement.defaultChecked= isSelected;
		});

		
		var availableSortModes= this._groupMode.getSortModes();
		var currentSorterId= _self._workItemSortFn.id;
		dojo.query(".sortBy input", this._optionsTable).forEach(function(radioElement) { //$NON-NLS-1$
			var thisSorterId= radioElement.getAttribute("value"); //$NON-NLS-1$
			
			var isAvailable= dojo.some(availableSortModes, function(sorter) { return sorter.id == thisSorterId; });
			dojo.toggleClass(radioElement.parentNode, "invisible", !isAvailable); //$NON-NLS-1$
			dojo.toggleClass(radioElement.parentNode, "visible", isAvailable); //$NON-NLS-1$

			if (isAvailable) {
				var isSelected= currentSorterId == thisSorterId;
				radioElement.checked= radioElement.defaultChecked= isSelected;
			}
		});

		var availableBarModes= this._groupMode.getBarModes();
		dojo.query(".barMode", this._optionsTable).forEach(function(node){ //$NON-NLS-1$
			dojo.toggleClass(node, "invisible", availableBarModes.length <= 1) //$NON-NLS-1$
		});
		
		if (availableBarModes.length > 1) {
			var currentBarModeId= _self._barMode.id;
			dojo.query(".barMode input", this._optionsTable).forEach(function(radioElement) { //$NON-NLS-1$
				var thisBarModeId= radioElement.getAttribute("value"); //$NON-NLS-1$
				
				var isAvailable= dojo.some(availableBarModes, function(barMode) { return barMode.id == thisBarModeId; });
				dojo.toggleClass(radioElement.parentNode, "invisible", !isAvailable); //$NON-NLS-1$
				dojo.toggleClass(radioElement.parentNode, "visible", isAvailable); //$NON-NLS-1$
	
				if (isAvailable) {
					var isSelected= currentBarModeId == thisBarModeId;
					radioElement.checked= radioElement.defaultChecked= isSelected;
				}
			});
		}
	},
	
	_getItem: function(type, id) {
		return this._referencedItems.getItem(type, id);
	},

	_displayPlannedItems: function(workItems) {
		this._workItems= workItems || [];

		var rootNode= this._groupMode.buildNodeTree(this._workItems);
		this._plannedItemsWidget.setContent(rootNode);

		dojo.style(this._optionsSection, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	// ---- tree filter implementation -------------------------------------------------------------------------------------
	
	_nodeFilterFn: function(node) {
		if (node.getNodeType() != "workItem") //$NON-NLS-1$
			return false;
		
		return dojo.some(this._filters, function(filter) {
			return filter(node);
		});
	},
	
	// ---- tree sort implementation ---------------------------------------------------------------------------------------
	
	_nodeSortFn: function(treeNodeA, treeNodeB) {
		var categoryA= this._nodeCategory(treeNodeA);
		var categoryB= this._nodeCategory(treeNodeB);
		
		if (categoryA != categoryB)
			return categoryA - categoryB;
			
		switch (categoryA) {
			case 1: 
				return this._workItemSortFn(treeNodeA, treeNodeB);
			case 2: 
				return this._groupSortFn(treeNodeA, treeNodeB);
			default: 
				return 0;
		}
	},
	
	_nodeCategory: function(treeNode) {
		switch (treeNode.getNodeType()) {
			case "workItem": //$NON-NLS-1$
				return 1;
			case "group": //$NON-NLS-1$
				return 2;
			default:
				return 100;
		}
	},
	
	_defaultGroupSortFn: function(treeNodeA, treeNodeB) {
		var labelA= treeNodeA.getNodeValue().name;
		var labelB= treeNodeB.getNodeValue().name;
		
		if (labelA == labelB)
			return 0;
		return labelA < labelB ? -1 : 1;
	},

	_defaultWorkItemSortFn: function(treeNodeA, treeNodeB) {
		return treeNodeB.getNodeValue().workItemId -  treeNodeA.getNodeValue().workItemId;
	},
	
	__sentinel: null // terminates this class definition
});

function _createWorkLoadObject(teamMemberWorkLoad) {
	result= {};
	dojo.forEach(teamMemberWorkLoad, function(workLoadDTO){
		result[workLoadDTO.contributorId]= workLoadDTO;
	});
	
	return result;	
}

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems.constructorArgs= function(iterationPlanPage, plannedItemsPageData, serviceCallController) {
	var result= {
		_planId: iterationPlanPage.getPlanId(),
		_teamMemberWorkLoad: {},
		_referencedItems: null
	};
	
	if (plannedItemsPageData != null) {
		result._myFolders= plannedItemsPageData.myFolders;
		result._workItems= plannedItemsPageData.workItems;
		result._teamMemberWorkLoad= _createWorkLoadObject(plannedItemsPageData.teamMemberWorkLoad);
		result._referencedItems= plannedItemsPageData._referencedItemStore;
	}

	return dojo.mixin(result, Tab.constructorArgs(iterationPlanPage, "workItems", MSGS.PLANNEDITEMS_TAB_NAME, serviceCallController)); //$NON-NLS-1$
};

})();
