/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative 	= net.jazz.ajax.toolbox.lang.checkNonNegative;
var checkNotEmpty 		= net.jazz.ajax.toolbox.lang.checkNotEmpty;

var ImageClient			= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemUtil 		= com.ibm.team.workitem.web.client.util;

/*
 * Constants
 */
var PAGE_SIZE_INTERVALS = [10, 25, 50, 75, 100, 250, 500];

/**
 * A meta-area widget that allows the user to manipulate query results by
 * navigating between query result pages and by performing textual filters on a
 * query page.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/templates/QueryResultPageControl.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/query/templates/Query.css"),
		
		// populate the template with data, connect events, and show initial
		// sub-tab
		postCreate: function() {
			checkNotNull(this.query, "this.query");
			checkNotNull(this.queryResultsSummaryPage, "this.queryResultsSummaryPage");
			checkNotNull(this.queryResultsWidget, "this.queryResultsWidget");
			checkNotNull(this.parentView, "this.parentView");
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
			
			if(this.isBottomControl) {
				this.domNode.className = "queryResultPageControlBottom";
				this.topRow.style.display = "none";
			}

			var links= this.titleContainer.getElementsByTagName("a");
			var name= this.query.name ? this.query.name : this.messages.titleNewQuery;
			name = WorkItemUtil.plainToHTML(name)
			var shortName = (name.length> 55) ? name.substring(0,52) + "..." : name;
			this.titleContainer.innerHTML = shortName;
			this.titleContainer.title =name;
			
			if (this._getResultCount() > 0) {
				this._showPagingControls();
			}
			
			// only enable editing for saved queries
			var element = document.createElement("a");
			element.title = this.messages.titleEditCurrentQuery;
			dojo.addClass(element,"editLink");
			if (this.query.itemId && !this.query.isNew && this.query.isEditable && window.location.href.indexOf("action=newQuery")== -1) {
				element.href= WorkItemUtil.getEditQueryURI(this.query.itemId);
			} else {
				dojo.addClass(element,"disabledIcon");
			}
			this.editQueryArea.appendChild(element);

			var exportLink= this._createLink(this.query.csvExportLink, "exportLink", this.messages.titleExportCSV);
			this.connect(exportLink, "onclick", function(event) {
				try{ 
					event.cancelBubble = true;
					} catch(e) { }
				});
			this.editQueryArea.appendChild(exportLink);			
			
			if(this.queryUrl && (this.queryUrl.indexOf("action=com.ibm.team.workitem.similarityQuery") != -1)) {
				var refreshUrl = WorkItemUtil.reuniquifyURL(this.queryUrl);
			} else {
				var refreshUrl = WorkItemUtil.reuniquifyURL(WorkItemUtil.getQueryURI(this.query), true);
			}
			
			var refreshLink= this._createLink(refreshUrl, "refreshLink", this.messages.titleReRunQuery);
			this.editQueryArea.appendChild(refreshLink);
			
			// separator
			this.editQueryArea.appendChild(this._createSeparator());

			var printLink= this._createLink(this.query.htmlExportLink, "printLink", this.messages.titleViewFullReport);
			printLink.target= "_blank";
			this.editQueryArea.appendChild(printLink);
			this._resetFilter();
		},
		
		_createLink: function(url, style, tooltip) {
			var link= document.createElement("a");
			link.title= tooltip;
			dojo.addClass(link, style);
			if (url) {
				link.href= url;
			} else {
				dojo.addClass(link, "disabledIcon");
			}
			return link;
		},
		
		_createSeparator: function() {
			var div= document.createElement("div");
			dojo.addClass(div, "separator");
			return div;
		},

		//
		// event handlers related to the filter control
		//
		getQueryResultsWidget: function() {
			return this.parentView._getQueryResultsWidget();
		}, 
		_onSetFilterValue: function(e) {
				this.getQueryResultsWidget().filter(this.filterControl.value);
		},
		
		_onFilterFocus: function(e) {
			if (this.filterControl.__jazzIsReset) {
				this.filterControl.value= "";
				dojo.removeClass(this.filterControlWrapper,"disabledText");
			}
			this.filterControl.__jazzIsReset= false;
		},
		
		_onFilterBlur: function(e) {
			var filterVal= dojo.trim(this.filterControl.value);
			if (filterVal.length === 0) {
				this._resetFilter();
			} else {
				this.filterControl.value= filterVal;
			}
		},
		setFilter: function(filterVal) {
				this.filterControl.value= filterVal;
				if(filterVal == "") {
					this.resetFilter();
				}
		},		
		_onClearFilterClick: function(e) {
			e.preventDefault();
			this._resetFilter();
		},
			
		_resetFilterControl: function() {
			if(this.filterControl.value =="") {
				this.getQueryResultsWidget().filter("");
				dojo.addClass(this.filterControlWrapper,"disabledText");
				this.filterControl.value= this.messages.labelTypeFilterText;
				this.filterControl.__jazzIsReset= true;
			}
		},	
		_resetFilter: function() {
			// unfilter results
			this.getQueryResultsWidget().filter("");
			dojo.addClass(this.filterControlWrapper,"disabledText");
			this.filterControl.value= this.messages.labelTypeFilterText;
			this.filterControl.__jazzIsReset= true;
		},
		
		//
		// logic for displaying the paging controls
		//
		
		_showPagingControls: function() {
			
			// show items per page drop-down
			if (!this.isBottomControl) {
				this.itemsPerPageDiv.appendChild(this._getItemsPerPageElement());
			}
			
			// show previous page text or link
			this.pageNumberDiv.appendChild(this._getPreviousElement());
			
			// show count info
			var pageInfoSpan= document.createElement("span");
			
			pageInfoSpan.className= "pageInfoSpan";
			var endIndex = Math.min(this._getMaxPageIndex(), this._getResultCount());
			var actual = this._getActualCountInfo();
			var resultCount = this._getResultCount() + "";
			if(endIndex == 1) {
				if(actual) {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPage1OfXWithActual, 
						[ resultCount, (actual + "") ]);
				} else {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPage1OfX, 
						[ resultCount ]);
				}
			} else {
				var startIndex = this._getStartIndex() + "";
				var pageIndex = Math.min(this._getMaxPageIndex(), resultCount)  + "";
				if(actual) {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPageXToYOfZWithActual, 
						[ startIndex, pageIndex, resultCount, (actual+"") ]);
				} else {
					pageInfoSpan.innerHTML = dojo.string.substitute(this.messages.labelPageXToYOfZ, 
						[ startIndex, pageIndex, resultCount ]);
				}
			}
			this.pageNumberDiv.appendChild(pageInfoSpan);
			
			// show next page link or text
			this.pageNumberDiv.appendChild(this._getNextElement());
			
			// filter only appears at top
			if (this.isBottomControl) {
				this.filterControlWrapper.style.display = "none";
			}
		},
		
		_getActualCountInfo: function() {
			var count = this._getResultCount();
			var actual = this._getTotal();
			if (count < actual) {
				return actual;
			}
			return null;
		},
		
		/*
		 * Creates and returns the dom node of the page size drop down with
		 * label div.
		 */
		_getItemsPerPageElement: function() {
			var div = document.createElement("div");
			div.className = "itemsPerPage";
			
			var select = document.createElement("select");
			var count = this._getResultCount();
			
			// populate the drop-down
			var optionIndex = 0;
			var defaultSize = WorkItemUtil._getDefaultQueryPageSize();
			var currentSize = this.queryItemsPerPage;
			if (!currentSize) {
				currentSize = count > defaultSize ? defaultSize : "all";
			}
			for (var i=0;i<PAGE_SIZE_INTERVALS.length;i++) {
				var pageSize = PAGE_SIZE_INTERVALS[i];
				if (count > pageSize) {
					var option = new Option("" + pageSize, "" + pageSize);
					select.options[optionIndex++] = option;
					option.selected = (pageSize == currentSize);
				}
				else break;
			}
			// show all not supported for large queries
			if (count <= PAGE_SIZE_INTERVALS[PAGE_SIZE_INTERVALS.length - 1]) {
				var option = new Option(this.messages.labelShowAll, "all");
				select.options[optionIndex] = option;
				option.selected = (currentSize == "all");
			}
			
			// disable drop-down if only one choice
			if (count <= PAGE_SIZE_INTERVALS[0]) {
				select.disabled = true;
				dojo.addClass(select, "disabled");
			}
			
			// register event handler
			select.onchange = dojo.hitch(this, function() {
				var itemsPerPage = select.options[select.selectedIndex].value;
				location.href = WorkItemUtil.getQueryURI(this.query, 1, this.queryOrderBy, this.queryOrderDirection, itemsPerPage, true);
			});
			
			div.appendChild(select);
			var labelSpan = document.createElement("span");
			labelSpan.className = "itemsPerPageText";
			labelSpan.innerHTML = this.messages.labelItemsPerPage;
			div.appendChild(labelSpan);
			return div;
		},
		
		_getPreviousElement: function() {
			return (this._getStartIndex() > 1) ?
				this._getLinkElement(WorkItemUtil.getQueryURI(this.query, Math.max(this._getStartIndex() - this.getPageSize()), this.queryOrderBy, this.queryOrderDirection, this.queryItemsPerPage, true), this.messages.labelPrevious) :
				this._getDisabledTextElement(this.messages.labelPrevious);
		},
		
		_getNextElement: function() {
			return (this._getMaxPageIndex() < this._getResultCount()) ?
				this._getLinkElement(WorkItemUtil.getQueryURI(this.query, this._getMaxPageIndex() + 1,  this.queryOrderBy, this.queryOrderDirection, this.queryItemsPerPage, true), this.messages.labelNext) :
				this._getDisabledTextElement(this.messages.labelNext);
		},
		
		_getDisabledTextElement: function(str) {
			var span= document.createElement("span");
			span.className= "disabledText";
			span.appendChild(document.createTextNode(str));
			return span;
		},
		
		_getLinkElement: function(href, text) {
			checkNotEmpty(text, "text");
			checkNotEmpty(href, "href");
			
			var linkElement = document.createElement("a");
			linkElement.href = href;
			linkElement.appendChild(document.createTextNode(text));
			
			return linkElement;
		},
		
		_getStartIndex: function() {
			checkNotNull(this.queryResultsSummaryPage.startIndex, "this.queryResultsSummaryPage.startIndex");
			
			return Number(this.queryResultsSummaryPage.startIndex) + 1;
		},
		
		_getResultCount: function() {
			checkNotNull(this.queryResultsSummaryPage.limit, "this.queryResultsSummaryPage.limit");
			
			return Number(this.queryResultsSummaryPage.limit);
		},
		
		_getTotal: function() {
			var total = new Number(this.queryResultsSummaryPage.totalCount);
			var estimatedTotal = new Number(this.queryResultsSummaryPage.estimatedTotal);
			
			if (estimatedTotal > total)
				return estimatedTotal;
			
			return total;
		},
		
		_getMaxPageIndex: function() {
			if (this.queryItemsPerPage == "all") {
				return this._getResultCount();
			}
			return this._getStartIndex() + this.getPageSize() - 1;
		},
		
		getPageSize: function() {
			if (this.queryItemsPerPage && this.queryItemsPerPage != "all") {
				return Number(this.queryItemsPerPage);
			}
			return WorkItemUtil._getDefaultQueryPageSize();
		}
	});

})();