/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var SelectAttributeDialog= com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectAttributeDialog;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderTableEntry", [dijit._Widget, dijit._Templated], {

	templateString: '<tr dojoAttachEvent="onclick: onSelect">' +
					'<td dojoAttachPoint="_label"></td>' +
					'<td><select dojoAttachPoint="_direction" dojoAttachEvent="onchange: onDirectionChange">' +
					'<option value="asc">${textAscending}</option>' +
					'<option value="desc">${textDescending}</option>' +
					'</select></td>' +
					'<td class="action"><div class="deleteButton" dojoAttachEvent="onclick: onDelete"></div></td>' +
					'</tr>',
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textAscending = this.messages.buttonAscending;
		this.textDescending = this.messages.buttonDescending;
		this.item= args.item;
		this.section= args.section;
	},
	
	postCreate: function() {
		var label= this.item.label;
		dojox.data.dom.replaceChildren(this._label, document.createTextNode(label));
		this._direction.options.selectedIndex= this.item.ascending ? 0 : 1;
	},
	
	getItem: function() {
		return this.item;
	},
	
	onDelete: function(event) {
		this.section.deleteEntry(this);
	},
	
	onSelect: function(event) {
		this.section.selectEntry(this);
	},
	
	onDirectionChange: function(event) {
		var ascending= event.target.value == "asc";
		this.item.ascending= ascending;
		this.section._setChanged(true);
	}

});
	
var EditableTableSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection;
var OrderTableEntry= com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderTableEntry;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection", EditableTableSection, {

	onAdd: function(event) {
		event.preventDefault();
		new SelectAttributeDialog({ 
			onOk: dojo.hitch(this, this._addAttributes),
			filterAttribute: "isSortable",
			attributeTree: this._attributes
			});
	},
	
	setCachedAttributeTree: function(attributes) {
		this._attributes= attributes;
	},
	
	setInput: function(columns) {
		this.deleteAllEntries();
		if (columns)
			this._addColumns(columns);
		this._setChanged(false);
	},
	
	_addAttributes: function(attributes) {
		for (var i= 0; i < attributes.length; i++) {
			var item= dojo.clone(attributes[i]);
			item.ascending= true;
			this._addItem(item);
		}
	},
	
	_addColumns: function(columns) {
		for (var i= 0; i < columns.length; i++) {
			this._addItem(columns[i]);
		}
	},
	
	_addItem: function(item) {
		var entry= new OrderTableEntry({ item: item, section: this});
		this.addEntry(entry);
	}

});
	
})();
