/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TabController");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorSection");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.UserSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TeamAreaSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ResultColumnsSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){

var checkNotNull= net.jazz.ajax.toolbox.lang.checkNotNull;
var PlatformUI= net.jazz.ajax.ui.PlatformUI;

var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImagePool= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var ErrorSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorSection;

var TabController= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TabController;
var SectionContainer= com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer;
var TermSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection;
var ConditionSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection;

var EditableTableSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection;
var UserSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.UserSection;
var TeamAreaSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TeamAreaSection;
var ResultColumnsSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ResultColumnsSection;
var OrderColumnsSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection;
var TextAreaSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection;

var isGuest= com.ibm.team.repository.web.client.session.isGuest;


dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/QueryEditor.html"),
	
	/////////////////////////////////////////////////////////////////////////////
	// Widget creation
	
	constructor: function(args) {
		this.parentController= args.parentController;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textRun = this.messages.buttonRun;
		this.textSave = this.messages.buttonSave;
		this.textSaveCopy = this.messages.buttonSaveCopy;
		this.textLabelName = this.messages.labelName;
		this.textTabTitleConditions = this.messages.tabTitleConditions;
		this.textTabTitleDetails = this.messages.tabTitleDetails;
		this.textTabTitleResultLayout = this.messages.tabTitleResultLayout;
		this.textSectionTitleDetails = this.messages.sectionTitleDetails;
		this.textLabelCreator = this.messages.labelCreator;
	},
	
	postCreate: function(){
		
		this._createTabs();
		this._createControls();
		
		this.typeIcon.src= ImagePool.WORKITEM_QUERY;
		this._setTitle(this.messages.sectionTitleWorkItemQuery);
		
		this._errorSection= new ErrorSection();
		this._errorDiv.appendChild(this._errorSection.domNode);
		
		this._dirtySubscription= dojo.subscribe(EventTopics.QUERY_EDITOR_CHANGED, this, this._setDirty);
		
		var srh= new ServiceResponseHandler(this, "_setAttributeTree", "_handleAttributeError");
		QueryClient.getAttributeTree(srh);
	},
	
	destroy: function() {
		try {
			dojo.unsubscribe(this._dirtySubscription);
			this._tabController.destroy();
			if(this._descriptionSection) this._descriptionSection.destroy();
			if(this._teamAreaAssociations) this._teamAreaAssociations.destroy();
			if(this._userAssociations) this._userAssociations.destroy();
			if(this._resultColumns) this._resultColumns.destroy();
			if(this._orderColumns) this._orderColumns.destroy();
			if(this._container) this._container.destroy();
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	
	_createTabs: function(){
		
		var tabStyleName= "Selected";
		var tabController= new TabController();
		this._tabController= tabController;
		
		tabController.registerTab(this._tabConditions, this._tabConditionsContent, tabStyleName);
		tabController.registerTab(this._tabDetails, this._tabDetailsContent, tabStyleName);
		tabController.registerTab(this._tabResultLayout, this._tabResultLayoutContent, tabStyleName);
		
		tabController.showTab(this._tabConditions);
	},
	
	_createControls: function() {
		
		var descriptionSection= new TextAreaSection({ title: this.messages.sectionTitleDescription, editActionName: this.messages.buttonEditDescription, emptyMessage: this.messages.labelEmptyDescription });
		this._descriptionSectionDiv.appendChild(descriptionSection.domNode);
		this._descriptionSection= descriptionSection;

		var teamAreaAssociations= new TeamAreaSection({ title: this.messages.sectionTitleTeamAreaSharing, addActionName: this.messages.buttonAddTeamArea, emptyMessage: this.messages.labelEmptyTeamAreaSharing});
		this._teamAreaSharingColumn.appendChild(teamAreaAssociations.domNode);
		this._teamAreaAssociations= teamAreaAssociations;

		var userAssociations= new UserSection({ title: this.messages.sectionTitleUserSharing, addActionName: this.messages.buttonAddUser, emptyMessage: this.messages.labelEmptyUserSharing});
		this._userSharingColumn.appendChild(userAssociations.domNode);
		this._userAssociations= userAssociations;

		var resultColumns= new ResultColumnsSection({ title: this.messages.sectionTitleResultColumns, addActionName: this.messages.buttonAddColumn, emptyMessage: this.messages.labelEmptyResultColumns, supportOrdering: true });
		this._resultsColumn.appendChild(resultColumns.domNode);
		this._resultColumns= resultColumns;

		var orderColumns= new OrderColumnsSection({ title: this.messages.sectionTitleSortOrder, addActionName: this.messages.buttonAddSortColumn, emptyMessage: this.messages.labelEmptyOrderColumns, supportOrdering: true });
		this._orderColumn.appendChild(orderColumns.domNode);
		this._orderColumns= orderColumns;
	},
	
	_createTermPart: function(/* TermDTO */ term){
		
		if (this._container)
			this._container.destroy();
			
		var termSection= new TermSection({ attributes: this._attributeTree, term: term });
		var container= new SectionContainer({
			sectionWidget: termSection
		});
		this._termSection= termSection;
		this._tabConditionsContent.appendChild(container.domNode);
		
		this._container= container;
	},
	
	/////////////////////////////////////////////////////////////////////////////
	// API
	
	showQuery: function(query) {
		this._isDirty= (query.stateId ? false : true);
		this._errorSection.clearError();
		this._internalShowQuery(query);
		if (!query.stateId) {
			this._nameInput.focus();
		}
	},
	
	_internalShowQuery: function(query, keepName) {
		this._query= query;
		
		this._suspendDirtyEvents= true;
		
		this._showName(query.name, keepName);
		this._showCreator(query.creator);
		this._descriptionSection.setText(query.description);
		this._teamAreaAssociations.setInput(query.teamAreaAssociations);
		this._userAssociations.setInput(query.userAssociations);
		this._resultColumns.setInput(query.columns);
		this._orderColumns.setInput(query.sorting);
		
		if (this._attributeTree) {
			this._createTermPart(query.expression)
		}
		this._tabController.showTab(this._tabConditions);
		
		this._suspendDirtyEvents= false;
		this._updateDirtyState();
	},
	
	_setTitle: function(title) {
		dojox.data.dom.replaceChildren(this._titleTextSpan, document.createTextNode(title));
	},
	
	/////////////////////////////////////////////////////////////////////////////
	
	// Values
	clearName: function() {
		this._isClear = true;
		this._nameInput.value= "";
	},
	
	_showName: function(name, keepName) {
		if (!((keepName || this._isClear) && name == "" && this._nameInput.value != "" )) {
			this._nameInput.value= name;
		}
		this._isClear = false;

	},
	
	_getName: function() {
		return this._nameInput.value;
	},
	
	_hasNameChanged: function() {
		return this._query.name != this._getName();
	},
	
	_showCreator: function(creator) {
		if (creator)
			dojox.data.dom.replaceChildren(this._queryCreator, document.createTextNode(creator.label));
	},
	
	/////////////////////////////////////////////////////////////////////////////
	// Callbacks
	
	_setDirty: function() {
		if (!this._isDirty && !this._suspendDirtyEvents) {
			this._isDirty= true;
			this._updateDirtyState();			
		}
	},
	
	_updateDirtyState: function() {
		this._updateSaveButtonEnablement();
		this._setTitle(this.messages.sectionTitleWorkItemQuery + (this._isDirty ? "*" : ""));
	},
	
	_updateSaveButtonEnablement: function() {
		var value= this._getName();
		var isInvalid= isGuest() || !(value && value.length > 0);
		this._saveCmd.disabled= isInvalid || !this._isDirty;
		if (this._query.stateId) {
			this._saveCopyCmd.disabled= isInvalid;
		} else {
			this._saveCopyCmd.disabled= true;
		}
	},
	
	_handleNameChange: function() {
		if (this._hasNameChanged())
			dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
		this._updateSaveButtonEnablement();
	},
	
	_handleRunClick: function() {
		if (!this._termSection)
			return;
		
		var expression= this._termSection.getTerm();
		
		var query= new Object();
		query.name= this._getName();
		query.itemId= "";
		query.internalExpression= expression;
		
		var sortOrder= this._orderColumns.getValues();
		var resultColumns= this._resultColumns.getValues();
		var columnArgs= { sortOrder: sortOrder, columnIdentifiers: this._getIds(resultColumns)};
		
		this.parentController.runQuery(query, columnArgs);
	},
	
	_handleSaveClick: function() {
		this._handleSave(false);
	},
	
	_handleSaveCopyClick: function() {
		this._handleSave(true);
	},
	
	_handleSave: function(createCopy) {

		this._startOperation(dojo.string.substitute(this.messages.busySavingQuery, [ this._getName() ]), "operation.query.save");
		
		var args= this._collectSaveArgument(createCopy);
		var srh= new ServiceResponseHandler(this, "_querySaved", "_handleSaveError");
		QueryClient.saveQuery(srh, args);
	},
	
	_collectSaveArgument: function(createCopy) {
		
		var args= new Object();
		
		if (this._query && !createCopy) {
			if (this._query.stateId) {
				args.itemId= this._query.itemId;
				args.stateId= this._query.stateId;
			}
		}
		if (this._hasNameChanged() || createCopy) {
			args.name= this._getName();
		}
		if (this._descriptionSection.hasChanged()) {
			args.description= this._descriptionSection.getText();
		}
		if (this._teamAreaAssociations.hasChanged()) {
			var teamAreas= this._teamAreaAssociations.getValues();
			args.teamAreaAssociations= this._getIds(teamAreas);
		}
		if (this._userAssociations.hasChanged()) {
			var users= this._userAssociations.getValues();
			args.contributorAssociations= this._getIds(users);
		}
		if (this._resultColumns.hasChanged()) {
			var resultColumns= this._resultColumns.getValues();
			args.resultColumns= this._getIds(resultColumns);
		}
		if (this._orderColumns.hasChanged()) {
			
			var orderColumns= this._orderColumns.getValues();
			args.sortColumns= this._getIds(orderColumns);
			
			var directions= new Array();
			for (var i= 0; i < orderColumns.length; i++) {
				directions.push(orderColumns[i].ascending);
			}
			args.sortDirections= directions;
		}
			
		args.expression= this._termSection.getTerm();
		
		return args;
	},
	
	_getIds: function(uiItems) {
		var ids= new Array();
		for (var i= 0; i < uiItems.length; i++) {
			ids.push(uiItems[i].id);
		}
		return ids;
	},
	
	_querySaved: function(operationReport) {
		if (operationReport.query) {
			this._isDirty= false;
			this._internalShowQuery(operationReport.query);			
		}
		this._errorSection.showReport(operationReport);
		
		this._stopOperation();
		this.parentController._setQueryPageTitle(this._getName(), /* isEditing */ true);
		dojo.publish(EventTopics.BUILDER_POST_SAVE, []);
	},

	_setAttributeTree: function(/* TreeDTO */ tree) {
		this._attributeTree= tree;
		this._resultColumns.setCachedAttributeTree(tree);
		this._orderColumns.setCachedAttributeTree(tree);
		if (this._query) {
			this._internalShowQuery(this._query, /* keepName */ true);
		}
	},
	
	_startOperation: function(name, id) {
		this._currentOperation= this.parentController.createOperation(name, id);
		this.parentController.startOperation(this._currentOperation);
		this._errorSection.clearError();
	},
	
	_stopOperation: function() {
		this.parentController.finishOperation(this._currentOperation);
	},
	
	_handleAttributeError: function(e) {
		this._stopOperation();
		this._errorSection.showError(this.messages.errorFetchingAttributes, e);
	},
	
	_handleSaveError: function(operationReport) {
		this._stopOperation();
		this._errorSection.showReport(operationReport);
	}
});
	
})();
