/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Class 
 */
com.ibm.team.dashboard.web.util.internal.HTMLUtility = {

	isMessagesLoaded: false,

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_loadMessages: function() {
		if (this.isMessagesLoaded) {
			return;
		}
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.isMessagesLoaded = true;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String html
 	 * @return boolean
 	 */	
	isSecureHTMLContent: function(html) {
		// Load externalized strings
		this._loadMessages();
		
		// Check for insecure content
		if (html.match(/<script/i)) {
			// Insecure content found
			alert(this.messages.msgHTMLErrorScriptTags);
			return false;
		} else if (html.match(/javascript:/i)) {
			// Insecure content found
			alert(this.messages.msgHTMLErrorQualifiers);
			return false;
		} else if (html.match(/on[a-z]+[\s]*=/i)) {
			// Insecure content found
			alert(this.messages.msgHTMLErrorEventHandlers);
			return false;
		}		

		// No insecure content found
		return true;
	},
	
	/*
	 * Returns a new string with all special HTML characters escaped.
	 * e.g. "hello <b>world</b>" -> "hello &lt;b&gt;world&lt;/b&gt;"
	 */
	/* string */ escapeHTML: function(/* string */ text) {
		var span = document.createElement("span");
		span.appendChild(document.createTextNode(text));
		return span.innerHTML;
	},
	
	/*
	 * Sanitizes a fragment of HTML markup by removing all scripting, event
	 * handlers, and "javascript:" link hrefs.
	 */
	/* string */ sanitizeHTML: function(/* string */ html) {
		// convert to DOM, sanitize DOM, then convert back to string.
		var div = document.createElement("div");
		div.innerHTML = html;
		this._sanitizeElement(div);
		return div.innerHTML;
	},
	
	/*
	 * Sanitizes the given DOM subtree rooted at the given element.
	 */
	_sanitizeElement: function(/* Element */ element) {
		// sanitize element
		if (element.nodeName.toLowerCase() == "script") {
			element.parentNode.removeChild(element);
			return;
		}
		
		// sanitize attributes
		var isAnchor = element.nodeName.toLowerCase() == "a";
		var attrs = element.attributes;
		var attrsToRemove = [];
		for (var i=0;i<attrs.length;i++) {
			var attr = attrs[i];
			var name = attr.name.toLowerCase();
			var value = attr.value.toLowerCase();
			// strip event handlers (e.g. onload, onclick) and "javascript:" link hrefs
			if (name.indexOf("on") == 0 || (isAnchor && name == "href" && value.indexOf("javascript:") != -1)) {
				attrsToRemove.push(name);
			}
		}
		for (var i=0;i<attrsToRemove.length;i++) {
			// deleting on the fly changes indexes, so delete after
			element.removeAttribute(attrsToRemove[i]);
		}
		
		// sanitize child elements recursively
		var children = element.childNodes;
		for (var i=0;i<children.length;i++) {
			var node = children[i];
			if (node.nodeType == 1) {
				this._sanitizeElement(node);
			}
		}
	}
};


})();

